//==============================================================================
// The Return of the Gods AI
//==============================================================================
// ROTGAI.xs
// This is a modification of Georg Kalus' extension of the default aomx ai file
// by Loki_GdD ("AoMod AI")
// and... slightly modified by Retherichus, to ensure online play without desyncs.
// as well with some other fixes.. though! all credit still goes to Loki_GdD!
// Hacks to make it work for ROTG (and even more extras) by JeHathor and nate_e.
//
// This is the main ai file. If you want to use rotg ai in your scenario,
// this would be the file you need to select. All other rotgai*.xs files are
// just helper files. They are no stand-alone ai files.
//==============================================================================

//==============================================================================
//The first part of this file is just a long list of global variables.  The
//'extern' keyword allows them to be used in any of the included files.  These
//are here to facilitate information sharing, etc.  The global variables are
//attempted to be named appropriately, but you should take a look at how they are
//used before making any assumptions about their actual utility.

//==============================================================================
//Map-Related Globals.
extern bool gWaterMap=false;              // Set true if fishing is likely to be good.
extern bool gTransportMap=false;          // Set true if transports are needed or very useful, i.e. island and shallow-chokepoint maps.

//==============================================================================
//Housing & PopCap.
extern int gHouseAvailablePopRebuild=10;     // Build a house when pop is within this amount of the current pop limit
extern int gHardEconomyPopCap=-1;            // Sets an absolute upper limit on the number of villagers maintained in the updateEM rules
extern int gEarlySettlementTarget = 3;       // How many age 1/2 settlements do we want?

//==============================================================================
//Econ Globals.
extern int   gGatherGoalPlanID=-1;
extern int   gCivPopPlanID=-1;
extern int   gNumBoatsToMaintain=6;       // Target number of fishing boats
extern bool	 gFarming=false;              // Set true when a farming plan is created, used to forecast farm resource needs.
extern bool  gFishing=false;              // Set true when a fishing plan is created, used to forecast fish boat wood demand
extern float gGoldForecast = 0.0;			// Forecasted demand over the next few minutes
extern float gWoodForecast = 0.0;
extern float gFoodForecast = 0.0;
extern int   gHerdPlanID = -1;            // Herds animals to base
extern float gGlutRatio = 0.0;            // 1.0 indicates all resources at 3 min forecast.  2.0 means all at least double.  Used to trim econ pop.
extern float gFoodGlutRatio = 0.0;
extern float gWoodGlutRatio = 0.0;
extern float gGoldGlutRatio = 0.0;
extern int   gLastAgeHandled = cAge1;     // Set to cAge2..cAge5 as the age handlers run. Used to detect age-ups granted via triggers and starting conditions, 

// Trade globals
extern int gMaxTradeCarts = 22;           // Max trade carts
extern int gTradePlanID = -1;
extern bool gExtraMarket = false;          // Used to indicate if an extra (non-trade) market has been requested
extern int gTradeMarketUnitID = -1;       // Used to identify the market being used in our trade plan.
extern vector gTradeMarketLocation = cInvalidVector; // location of our trade market
extern vector gTradeMarketDesiredLocation = cInvalidVector; // location, where we want to build our trade market.
extern int gExtraMarketUnitID = -1;       // Used to identify the extra market
extern bool gResetTradeMarket = false;

//==============================================================================
//Military Globals.
extern bool gBuildWalls = true;
extern int gWallPlanID = -1;
extern int gRushUPID=-1;            // Unit picker ID for age 2 (cAge2) armies.
extern int gLateUPID=-1;            // Unit picker for age 3/4 (cAge3 and cAge4).
extern int gNavalUPID=-1;
extern int gNumberBuildings=3;      // Number of buildings requested for late unit picker
extern int gNavalAttackGoalID=-1;
extern int gRushGoalID=-1;
extern int gLandAttackGoalID=-1;
extern int gIdleAttackGID=-1;       // Attack goal, inactive, used to maintain mil pop after rush and/or before age 3 (cAge3) attack.
extern int gDefendPlanID = -1;      // Uses military units to defend main base while waiting to mass an attack army
extern int gWonderDefendPlan = -1;     // Uber-plan to defend my wonder
extern int gEnemyWonderDefendPlan = -1;   // Uber-uber-plan to attack or defend other wonder
extern int gObeliskClearingPlanID = -1;   // Small attack plan used to remove enemy obelisks
extern int gTargetNavySize = 0;     // Set periodically based on difficulty, enemy navy/fish boat count. Units, not pop slots.

//==============================================================================
//Minor Gods.
extern int gAge2MinorGod = -1;
extern int gAge3MinorGod = -1;
extern int gAge4MinorGod = -1;

//==============================================================================
//God Powers
extern int gAge1GodPowerPlanID = -1;
extern int gAge2GodPowerPlanID = -1;
extern int gAge3GodPowerPlanID = -1;
extern int gAge4GodPowerPlanID = -1;
extern int gAge5GodPowerPlanID = -1;
extern int gTownDefenseGodPowerPlanID = -1;
extern int gTownDefenseEvalModel = -1;
extern int gTownDefensePlayerID = -1;
extern int gTownDefenseTargetingModel = -1;
extern vector gTownDefenseLocation = cInvalidVector;

extern int gPlaceTitanGatePlanID = -1;
extern int gHeavyGPTechID = -1;
extern int gHeavyGPPlanID = -1;

//==============================================================================
//Special Case Stuff
extern int gLandScout = -1;
extern int gAirScout = -1;
extern int gWaterScout = -1;

extern int gMaintainNumberLandScouts = 1;
extern int gMaintainNumberAirScouts = 1;
extern int gMaintainNumberWaterScouts = 1;

extern int gRelicGatherPlanID = -1;
extern int gMaintainWaterXPortPlanID=-1;
extern int gVinlandsagaTransportExplorePlanID=-1;
extern int gVinlandsagaInitialBaseID=-1;
extern int gNomadExplorePlanID1=-1;
extern int gNomadExplorePlanID2=-1;
extern int gNomadExplorePlanID3=-1;
extern int gNomadSettlementBuildPlanID=-1;
extern int gKOTHPlentyUnitID=-1;
extern int gLandExplorePlanID=-1;
extern int gFarmBaseID = -1;
extern int gUlfsarkMaintainPlanID = -1;   // Used to maintain a small pop of ulfsarks for building

//New globals
extern int gGatherRelicType = -1;
extern bool gBuildWallsAtMainBase = true;
extern vector gBackAreaLocation = cInvalidVector;
extern vector gHouseAreaLocation = cInvalidVector;
extern int gBackAreaID = -1;
extern int gHouseAreaID = -1;
extern bool gResetWallPlans = false;

extern float gMainBaseAreaWallRadius = 45;
extern int gMainBaseAreaWallTeam1PlanID = -1;
extern int gMainBaseAreaWallTeam2PlanID = -1;
extern int gOtherBaseRingWallTeam1PlanID = -1;
extern float gOtherBaseWallRadius = 21.0;
extern int gBuildBuilding1AtOtherBasePlanID = -1;

extern int gMBDefPlan1ID = -1;
extern int gOtherBase1ID = -1;			// globals for defend plans for other bases
extern int gOtherBase2ID = -1;
extern int gOtherBase3ID = -1;
extern int gOtherBase4ID = -1;
extern int gOtherBase1UnitID = -1;
extern int gOtherBase2UnitID = -1;
extern int gOtherBase3UnitID = -1;
extern int gOtherBase4UnitID = -1;
extern int gOtherBase1DefPlanID = -1;
extern int gOtherBase2DefPlanID = -1;
extern int gOtherBase3DefPlanID = -1;
extern int gOtherBase4DefPlanID = -1;
extern int gOtherBase1RingWallTeamPlanID = -1;
extern int gOtherBase2RingWallTeamPlanID = -1;
extern int gOtherBase3RingWallTeamPlanID = -1;
extern int gOtherBase4RingWallTeamPlanID = -1;

extern int gHero1MaintainPlan = -1;
extern int gHero2MaintainPlan = -1;
extern int gHero3MaintainPlan = -1;
extern int gHero4MaintainPlan = -1;

extern int gEnemySettlementAttPlanID = -1;
extern int gEnemySettlementAttPlanTargetUnitID = -1;
extern vector gEnemySettlementAttPlanLastAttPoint = cInvalidVector;
extern vector gSettlementPosDefPlanDefPoint = cInvalidVector;
extern int gSettlementPosDefPlanID = -1;

extern int gRaidingPartyAttackID = -1;
extern int gRaidingPartyTargetUnitID = -1;
extern vector gRaidingPartyLastTargetLocation = cInvalidVector;
extern vector gRaidingPartyLastMarketLocation = cInvalidVector;

extern int gRushCount = 0;
extern int gNumRushAttacks = 0;
extern int gRushSize = 0;
extern int gFirstRushSize = 0;
extern int gRushAttackCount = 0; 
extern int gLandAttackPlanID = -1; 

extern int gDockBaseID = -1;
extern int gWaterExploreID = -1;
extern int gFishPlanID = -1;

extern int gAirScout1PlanID = -1;
extern int gAirScout2PlanID = -1;

extern vector gBaseUnderAttackLocation = cInvalidVector;
extern int gBaseUnderAttackID = -1;
extern int gBaseUnderAttackDefPlanID = -1;

extern bool gHuntersExist = false;
extern int gAlliedBaseDefPlanID = -1;
//New globals end

//==============================================================================
//Base Globals.
extern int gGoldBaseID=-1;          // Base used for gathering gold, although main base is used if gold exists there
extern int gWoodBaseID=-1;          // Ditto for wood
extern float gMaximumBaseResourceDistance = 85.0;

//==============================================================================
//Age Progression Plan IDs.
extern int gAge2ProgressionPlanID = -1;
extern int gAge3ProgressionPlanID = -1;
extern int gAge4ProgressionPlanID = -1;

//==============================================================================
//Forward declarations.
mutable void setParameters(void) {}       // Used in loader file to set control parameters, called at start of main()
mutable void setMilitaryUnitPrefs(int primaryType = -1, int secondaryType = -1, int tertiaryType = -1) {}   // Used by loader to override unitPicker choices
mutable void age2Handler(int age=1) { }
mutable void age3Handler(int age=2) { }
mutable void age4Handler(int age=3) { }
mutable int createSimpleMaintainPlan(int puid=-1, int number=1, bool economy=true, int baseID=-1) { }
mutable int createSimpleBuildPlan(int puid=-1, int number=1, int pri=100,
bool military=false, bool economy=true, int escrowID=-1, int baseID=-1, int numberBuilders=1) { }
mutable void buildHandler(int protoID=-1) { }
mutable void gpHandler(int powerID=-1)    { }
mutable int createBuildSettlementGoal(string name="BUG", int minAge=-1, int maxAge=-1, int baseID=-1, int numberUnits=1, 
int builderUnitTypeID=-1, bool autoUpdate=true, int pri=100) { }
mutable int getSoftPopCap(void) {}
mutable void unbuildHandler() { }
mutable void age5Handler(int age=4) { }
mutable int createTransportPlan(string name="BUG", int startAreaID=-1, int goalAreaID=-1,
bool persistent=false, int transportPUID=-1, int pri=-1, int baseID=-1) {}
mutable int createSimpleAttackGoal(string name="BUG", int attackPlayerID=-1,
int unitPickerID=-1, int repeat=-1, int minAge=-1, int maxAge=-1,
int baseID=-1, bool allowRetreat=false) {}
mutable bool mapPreventsHousesAtTowers()    { }
mutable void updateGlutRatio(void) { }
mutable void ExplorerUpdater(int ageid=-1) { }

//TODO: Check if they are really necessary!
mutable void findTownDefenseGP(int baseID=-1) { }
mutable void releaseTownDefenseGP() { }
mutable bool mapRestrictsMarketAttack() { }
mutable void pullBackUnits(int planID = -1, vector retreatPosition = cInvalidVector) { }

// Reth Placeholder for Int's etc
extern bool ShowAIDebug = false;
extern int KOTHTransportPlan = -1;
extern int KOTHTHomeTransportPlan = -1;
extern int SendBackCount = 0;
extern bool KoTHOkNow = false;
extern bool DestroyTransportPlan = false;  
extern bool DestroyHTransportPlan = false;
extern int rExploreIsland = -1;
extern const int MyFortress = cUnitTypeHillFort;
extern const int cBuilderType = cUnitTypeVillagerAztec;
extern const int cNumResourceTypes = 3;

extern int fCitadelPlanID = -1;
extern bool AutoDetectMap = false;
extern bool NeedTransportCheck = false;
extern int gShiftingSandPlanID= -1;
   mutable void wonderDeathHandler(int playerID=-1) { }
extern bool gHuntingDogsASAP = false;     // Will automatically be called upon if there is hunt nearby the MB.
extern bool RiverSLowBoar = false;
extern bool RetardedLowBoarSpawn = false;
extern bool gpDelayMigration = false;
extern int wonderBPID = -1;
extern bool IsRunHuntingDogs = false;
extern int gDefendPlentyVault = -1;
extern int gHeavyGPTech=-1;
extern int gHeavyGPPlan=-1;
extern int gDefendPlentyVaultWater=-1;
extern int WallAllyPlanID=-1;
extern int FailedToTrain = 0;
extern int defWantedCaravans = 20;
extern int MedicMaintain = -1;
extern bool KOTHStopRefill = false;
extern vector KOTHGlobal = cInvalidVector;
extern bool IhaveAllies = false;
extern bool mRusher = false;
extern bool BeenmRusher = false;
extern int MoreFarms = 26;
extern bool TitanAvailable = false;
extern bool KoTHWaterVersion = false;
extern int KOTHBASE = -1;
extern bool KothDefPlanActive = false;
extern bool WaitForDock = false;
extern int CataMaintain = -1;
extern int StuckTransformID = 0;
extern int ResourceBaseID = -1;
extern bool HasHumanAlly = false;
extern int gExaminationID = -1;
extern int MigrationAreaID = -1;
extern int HealDefPlan = -1;
extern int gSomeData = -1;
extern bool AoModAllies = false;
extern const int Tellothers = 30;
extern const int admiralTellothers = 31;
extern const int AttackTarget = 35;
extern const int cAttackTC = 36;
extern int aEnemyTCID = -1;
extern int aLastTCIDTime = 0;
extern const int cEmergency = 38;
extern const int cLowPriority = 39;
extern const int VectorData = 40;
extern bool ChangeMHP = false;
extern int MHPTime = 0;
extern const int INeedHelp = 32;
extern int HelpSettleID = -1;
extern const int Yes = 60;
extern const int No = 61;
extern int gLastSentTime = 0;
extern const int RequestFood = 70;
extern const int RequestWood = 71;
extern const int RequestGold = 72;
extern const int ExtraFood = 73;
extern const int ExtraWood = 74;
extern const int ExtraGold = 75;
extern const int RequestTower = 76;
extern const int EcoPercentage = 80;
extern const int MilPercentage = 81;
extern const int RootPercentage = 82;
extern const int LandAttackTarget = 85;
extern const int SettlementAttackTarget = 86;
extern const int MainUnit = 87;
extern const int SecondaryUnit = 88;
extern const int ThirdUnit = 89;
extern const int PlayersData = 100;

int try_tech(int unitid=-1, int techid=-1, int counter=0)
{
   if ( (kbGetTechStatus(techid) % 4) != 0 )
      aiTaskUnitResearch( unitid, techid );
   else  //Unobtainable or Active
      counter++;
   return( counter );
}

//==============================================================================
//Basics Include.
include "ROTGAIBasics.xs";

// Placeholder Reth
include "ROTGAIExtra.xs";

// Placeholder JeHathor
include "ROTGAIMicro.xs";

//==============================================================================

//==============================================================================
//BuildRules Include.
include "ROTGAIBuild.xs";

//==============================================================================
//Economy Include.
include "ROTGAIEcon.xs";

//==============================================================================
//God Powers Include.
include "ROTGAIGPs.xs";

//==============================================================================
//Map Specifics Include.
include "ROTGAIMapSpec.xs";

//==============================================================================
//Military Include.
include "ROTGAIMil.xs";

//==============================================================================
//Naval Include.
include "ROTGAINaval.xs";

//==============================================================================
//Personality Include.
include "ROTGAIPers.xs";

//==============================================================================
//Progress Include.
include "ROTGAIProgr.xs";

//==============================================================================
//TechRules Include.
include "ROTGAITechs.xs";

//==============================================================================
//trainRules Include.
include "ROTGAITrain.xs";


//extern const bool cvDelayStart = false;
//extern const int cvPlayerToAttack = -1;
//extern int gStartTime = 0;
//extern bool gBuildTowers = true;
//extern int gTargetNumTowers = 0;
//mutable void init(void) { }
//include "aomxaicv.xs";  //Mod AI doesn't actually use ES's cv system.
//include "aomxaicvfuncs.xs";


//==============================================================================
rule updatePlayerToAttack   //Updates the player we should be attacking.
minInterval 27 //starts in cAge1
inactive
{
   static int lastTargetPlayerIDSaveTime = -1;
   static int lastTargetPlayerID = -1;
   static int randNum = 0;
   static bool increaseStartIndex = false;

   if (ChangeMHP) {
      if (xsGetTime() > MHPTime + 1*60*1000)
         ChangeMHP = false;
      return;
   }
   //Determine a random start index for our hate loop.
   static int startIndex = -1;
   if ((startIndex < 0) || (xsGetTime() > lastTargetPlayerIDSaveTime + (10 + randNum)*60*1000) && (aiRandInt(5) < 1))
      startIndex = getRandomPlayerByRel(cPlayerRelationEnemy);

   int comparePlayerID = -1;
   for (i = 0; < cNumberPlayers) {
      //If we're past the end of our players, go back to the start.
      int actualIndex = i + startIndex;
      if (actualIndex >= cNumberPlayers)
         actualIndex = actualIndex - cNumberPlayers;
      if (actualIndex <= 0)
         continue;
      if ((kbIsPlayerEnemy(actualIndex)) &&
          (kbIsPlayerResigned(actualIndex) == false) &&
          (kbHasPlayerLost(actualIndex) == false)) {
         comparePlayerID = actualIndex;
         break;
      }
   }

   //Pass the comparePlayerID into the AI to see what he thinks.  He'll take care
   //of modifying the player in the event of wonders, etc.
   if (cvPlayerToAttack == -1)
      int actualPlayerID = aiCalculateMostHatedPlayerID(comparePlayerID);
   else
      actualPlayerID = cvPlayerToAttack;

   if (actualPlayerID != lastTargetPlayerID) {
      lastTargetPlayerID = actualPlayerID;
      lastTargetPlayerIDSaveTime = xsGetTime();
      randNum = aiRandInt(5);
   }
   if (actualPlayerID != -1)
      aiSetMostHatedPlayerID( actualPlayerID );

   if (AoModAllies) {
      if (aiGetCaptainPlayerID(cMyID) != cMyID)
         return;
      MessageRel(cPlayerRelationAlly, AttackTarget, aiGetMostHatedPlayerID());
   }	 
}

//==============================================================================
rule checkEscrow    //Verify that escrow totals and real inventory are in sync
minInterval 6 //starts in cAge1
active
{
   for(resourceid=0; <= cResourceFavor) {
      float fAmount = ( (kbEscrowGetAmount(0, resourceid) + kbEscrowGetAmount(1, resourceid) + kbEscrowGetAmount(2, resourceid)) -
                         kbResourceGet(resourceid) );
      if ( fAmount >= 20.0 ) {
         kbEscrowAllocateCurrentResources();
         break;
      }
   }
}

//==============================================================================
void updateEM(int econPop=-1, int milPop=-1, float econPercentage=0.5,
              float rootEscrow=0.2, float econFoodEscrow=0.5, float econWoodEscrow=0.5,
              float econGoldEscrow=0.5, float econFavorEscrow=0.5)
{
   //Econ Pop (if we're allowed to change it).
   if ((gHardEconomyPopCap > 0) && (econPop > gHardEconomyPopCap))
      econPop=gHardEconomyPopCap;
   if ( (econPop > cvMaxGathererPop)  && (cvMaxGathererPop >= 0) )
      econPop = cvMaxGathererPop;

   // Check if we're second age.  If so, consider capping the mil lower for boomers
   if (kbGetAge() == cAge2) {
      if ( (gRushGoalID == -1) ||   /* If we don't have a rush goal, or... */
         (aiPlanGetVariableInt(gRushGoalID, cGoalPlanExecuteCount, 0) >= aiPlanGetVariableInt(gRushGoalID, cGoalPlanRepeat, 0)) )
      {
         // Let's decrease our military pop
         float milPopDelta = (cvRushBoomSlider*4.0)/5.0;    // Zero for balanced, -.80 for extreme boom
         if (milPopDelta > 0) 
            milPopDelta = 0;  // Don't increase for rushers
         // Adjust it for econ/mil scale.  If military, soften the decrease, if economic, preserve full.
         milPopDelta = milPopDelta / (2.0 + cvMilitaryEconSlider);
         milPop = milPop + (milPop * milPopDelta);
      }
      if (milPop < 40)
         milPop = 40;	
   }

   if ((milPop < 0) && (cvMaxMilPop >= 0))
      milPop = cvMaxMilPop;      
   if ((milPop > cvMaxMilPop) && (cvMaxMilPop >= 0))
      milPop = cvMaxMilPop;

   aiSetEconomyPop(econPop);
   aiSetMilitaryPop(milPop);

   // Check to make sure attack goals have ranges below our milPop limit
   int upID = gRushUPID;
   if (kbGetAge() > cAge2)
      upID = gLateUPID;
   int milMin = kbUnitPickGetMinimumPop(upID);
   int milMax = kbUnitPickGetMaximumPop(upID);
   if (milMax > milPop) {
      kbUnitPickSetMaximumPop(upID,(milPop*4)/5);
      kbUnitPickSetMinimumPop(upID,(milPop*3)/5);
   }

   //Percentages.
   aiSetEconomyPercentage(1.0);
   aiSetMilitaryPercentage(1.0);

   //Get the amount of the non-root pie.
   float nonRootEscrow=1.0-rootEscrow;
   //Track whether or not we need to redistribute the resources.
   //Econ Food Escrow.
   float v=nonRootEscrow*econFoodEscrow;
   kbEscrowSetPercentage(cEconomyEscrowID, cResourceFood, v);
   //Econ Wood Escrow
   v=nonRootEscrow*econWoodEscrow;
   kbEscrowSetPercentage(cEconomyEscrowID, cResourceWood, v);
   //Econ Gold Escrow
   v=nonRootEscrow*econGoldEscrow;
   kbEscrowSetPercentage(cEconomyEscrowID, cResourceGold, v);
   //Econ Favor Escrow
   v=nonRootEscrow*econFavorEscrow;
   kbEscrowSetPercentage(cEconomyEscrowID, cResourceFavor, v);
   //Military Escrow.
   kbEscrowSetPercentage(cMilitaryEscrowID, cResourceFood, nonRootEscrow*(1.0-econFoodEscrow));
   kbEscrowSetPercentage(cMilitaryEscrowID, cResourceWood, nonRootEscrow*(1.0-econWoodEscrow));
   kbEscrowSetPercentage(cMilitaryEscrowID, cResourceGold, nonRootEscrow*(1.0-econGoldEscrow));
   kbEscrowSetPercentage(cMilitaryEscrowID, cResourceFavor, nonRootEscrow*(1.0-econFavorEscrow));
   if (gSomeData != -1) {
      aiPlanSetUserVariableFloat(gSomeData, EcoPercentage, 0 , v * 100);
      aiPlanSetUserVariableFloat(gSomeData, MilPercentage, 0 , nonRootEscrow*(1.0-econFoodEscrow) * 100);
      aiPlanSetUserVariableFloat(gSomeData, RootPercentage, 0 , rootEscrow * 100);
   }
   int vilPop= aiGetEconomyPop();      // Total econ
   if (gFishing) {
      int fishCount = gNumBoatsToMaintain;
      if ( (aiGetGameMode() == cGameModeLightning) && (fishCount > 5) )
         fishCount = 5;
      vilPop = vilPop - fishCount; // Less fishing
   }

   int numTradeUnits = kbUnitCount(cMyID, cUnitTypeAbstractTradeUnit, cUnitStateAlive);
   if (numTradeUnits > 0) {
      int tradeCount = numTradeUnits;
      if ((aiGetGameMode() == cGameModeLightning) && (tradeCount > 5))
         tradeCount = 5; 
      vilPop = vilPop - tradeCount;     // Vils = total-trade
   }

   if ((vilPop < 15) && (kbGetAge() > cAge1))
      vilPop = 15;
   // Brutal hack to make Lightning work.
   if (aiGetGameMode() == cGameModeLightning) {  //Make sure we don't try to overtrain villagers
      const int lightningLimit = 25;      //Greek/Egyptian;
      if (vilPop > lightningLimit)
         vilPop = lightningLimit;
   }

   //Update the number of vils to maintain.
   const int gathererPUID = cUnitTypeVillagerAztec;
   if ((kbCanAffordUnit(gathererPUID, cEconomyEscrowID)) && (kbUnitCount(cMyID, gathererPUID, cUnitStateAlive) < vilPop * 0.8) && 
      (kbUnitCount(cMyID, cUnitTypeAbstractSettlement, cUnitStateAlive) > 0) && (kbGetPopCap() != kbGetPop()) && (AgingUp() == false))
   {
      static int LastCount = 0;
      int CurrentTrained = aiPlanGetNumberVariableValues(gCivPopPlanID, cTrainPlanTrainedUnitID);
      if ((CurrentTrained > LastCount) || (kbUnitCount(cMyID, gathererPUID, cUnitStateBuilding) > 0)) {
         FailedToTrain = 0;
         LastCount = CurrentTrained;
      }
      else if (CurrentTrained == LastCount)
         FailedToTrain++;
      if (FailedToTrain >= 4) {
         aiPlanDestroy(gCivPopPlanID);
         FailedToTrain = 0;
         LastCount = 0;
         createCivPopPlan();
      }
   }
   aiPlanSetVariableInt(gCivPopPlanID, cTrainPlanNumberToMaintain, 0, vilPop);
   if (gSomeData != -1)
      aiPlanSetUserVariableInt(gSomeData, 7, 0 , vilPop);
}

//==============================================================================
rule updateEMAllAges
minInterval 12 //starts in cAge1
active
{
   int civPopTarget=-1;
   int milPopTarget=-1;
   static int ageStartTime3 = -1;
   static int ageStartTime4 = -1;		
   float econPercent = 1.0;     // Econ priority rating, range 0..1
   float econEscrow = 1.0;      // Economy's share of non-root escrow, range 0..1
   updateGlutRatio();

   if ((RiverSLowBoar) && (kbGetAge() < cAge2) && (aiGetWorldDifficulty() < cDifficultyNightmare)) {
      milPopTarget = 60;
      if ( RetardedLowBoarSpawn )
         civPopTarget = 13;	
      else
         civPopTarget = 18;
   }
   else if ( (kbGetAge() < cAge2) && (aiGetWorldDifficulty() >= cDifficultyModerate) ) {
      civPopTarget = 25;
      milPopTarget = 60;
   }
   else if (aiGetWorldDifficulty() <= cDifficultyModerate) {
      if (aiGetWorldDifficulty() == cDifficultyEasy) {
         civPopTarget = 18;
         milPopTarget = 32;
      }
      else {
         milPopTarget = 50;
         civPopTarget = 34;
         if (aiGetGameMode() == cGameModeLightning)
            civPopTarget = 15;
      }
   }
   else {
      civPopTarget = 61;
      if (getSoftPopCap() > 115)
         civPopTarget = civPopTarget + 0.2 * (getSoftPopCap()-115);  // Plus 20% over 115		
      if (gGlutRatio > 1.0)
         civPopTarget = civPopTarget / gGlutRatio;
      if ((aiGetGameMode() == cGameModeLightning) && (civPopTarget > 35))  
         civPopTarget = 35;
      if (civPopTarget > 80)
         civPopTarget = 80;
      if (civPopTarget < 20)
         civPopTarget = 20;		
      milPopTarget = getSoftPopCap() - civPopTarget;
      if ((kbGetAge() == cAge3) && (ageStartTime3 == -1))
         ageStartTime3 = xsGetTime();
      if ((kbGetAge() == cAge4) && (ageStartTime4 == -1))
         ageStartTime4 = xsGetTime();		
   }

   if (kbGetAge() > cAge1) {
      if ((aiGetWorldDifficulty() >= cDifficultyHard) && (kbGetAge() >= cAge3)) {
         kbUnitPickSetMinimumPop( gLateUPID, milPopTarget*.5 );
         kbUnitPickSetMaximumPop( gLateUPID, milPopTarget*.75 );
      }
      if (kbGetAge() == cAge2) {
         econPercent = 0.5;
         econEscrow = 0.5; 
      }
      else if (kbGetAge() == cAge3) {
         econPercent = 0.3;
         econEscrow = 0.3; 
      }
      else {
         econPercent = 0.15;
         econEscrow = 0.15; 
      }

      if ( kbProtoUnitAvailable(cUnitTypeCaravanGreek) ) {
         int BonusCaravan = 0;
         if (gGlutRatio < 1.0)
            BonusCaravan = BonusCaravan + 4;
         if ((aiGetWorldDifficulty() >= cDifficultyHard) && (aiGetGameMode() != cGameModeLightning)) {
            int numWoodPlans = aiPlanGetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumWoodPlans, 0);
            if (numWoodPlans < 1)
               BonusCaravan = BonusCaravan + 4;
            int numGoldPlans = aiPlanGetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumGoldPlans, 0);
            if (numGoldPlans < 1)
               BonusCaravan = BonusCaravan + 4;
         }
         if (gGlutRatio >= 1.15) {
            float Increase = 0.0;
            if (gGlutRatio >= 1.15)
               Increase = gGlutRatio - 1.15;
            Increase = Increase * 20;
            //Dont drop below 10 carvans! Had a situation where AI kept deleting all caravans!
            gMaxTradeCarts = xsMax( defWantedCaravans + BonusCaravan - Increase, 10 );	
         }
         else
            gMaxTradeCarts = defWantedCaravans + BonusCaravan;
         if (gSomeData != -1)
            aiPlanSetUserVariableInt(gSomeData, 9, 0 , gMaxTradeCarts);
      }
      float econAdjust = -.5 * cvMilitaryEconSlider;
      float econShortage = aiGetAvailableEconomyPop();
      float econTarget = aiGetEconomyPop();
      econShortage = econShortage / econTarget;
      econAdjust = econAdjust + econShortage;
      if (econAdjust > 1.0)
         econAdjust = 1.0;
      if (econAdjust < -1.0)
         econAdjust = -1.0;

      econPercent = adjustSigmoid(econPercent, econAdjust, 0.0, 1.0);
      econEscrow = econPercent;	

      if ((xsGetTime() > 10*60*1000) && (kbGetAge() == cAge2) && (AgingUp() == false) || 
         (xsGetTime() > 22*60*1000) && (AgingUp() == false) && (xsGetTime() - ageStartTime3 > 6*60*1000) && (kbGetAge() == cAge3)
      || (kbGetAge() == cAge3) && (AgingUp() == false) && (xsGetTime() - ageStartTime3 > 3*60*1000) && (ShouldIAgeUp()) 
      || (TitanAvailable) && (kbGetAge() == cAge4) && (AgingUp() == false) && (gTransportMap == false) && (xsGetTime() - ageStartTime4 > 3*60*1000))
      {
         if (econEscrow < 0.35)
            econEscrow = 0.35;
      }
      if (econEscrow < 0.15)
         econEscrow = 0.15;
   }
   //Update all the econ stuff
   if ( (kbGetAge() >= cAge2) && (aiGetWorldDifficulty() >= cDifficultyHard) &&
      (kbUnitCount(cMyID, cUnitTypeAbstractSettlement, cUnitStateAliveOrBuilding) < 1) )
   {
      int Vills = kbUnitCount(cMyID,cUnitTypeAbstractVillager, cUnitStateAlive);
      int numTradeUnits = kbUnitCount(cMyID, cUnitTypeAbstractTradeUnit, cUnitStateAlive);
      int FishShips = kbUnitCount(cMyID, cUnitTypeUtilityShip, cUnitStateAlive);
      int ModdedCivPop = Vills + FishShips + numTradeUnits;
      civPopTarget = ModdedCivPop;
      if (civPopTarget < 25)
         civPopTarget = 25;
      milPopTarget = getSoftPopCap() - civPopTarget;
   }
   updateEM( civPopTarget, milPopTarget, econPercent, 0.20, econEscrow, econEscrow, econEscrow, econEscrow );
}
void updateGlutRatio(void)
{
   float goldSupply = kbResourceGet(cResourceGold);
   float woodSupply = kbResourceGet(cResourceWood);
   float foodSupply = kbResourceGet(cResourceFood);

   gGlutRatio = 100.0;     // ludicrously high

   if ( (goldSupply/gGoldForecast) < gGlutRatio )
      gGlutRatio = goldSupply/gGoldForecast;
   if ( (woodSupply/gWoodForecast) < gGlutRatio )
      gGlutRatio = woodSupply/gWoodForecast;
   if ( (foodSupply/gFoodForecast) < gGlutRatio )
      gGlutRatio = foodSupply/gFoodForecast;
   gGlutRatio = gGlutRatio * 1.5; // Double it, i.e. start reducing civ pop when all resources are > 50% of forecast

   if (gGlutRatio <= 0.0)
      gGlutRatio = 0.0;

   gFoodGlutRatio = (foodSupply / gFoodForecast) * 1.5;
   gWoodGlutRatio = (woodSupply / gWoodForecast) * 1.5;
   gGoldGlutRatio = (goldSupply / gGoldForecast) * 1.5;

   if ( (kbGetAge() < cAge4) || (TitanAvailable && (kbGetTechStatus(cTechSecretsoftheTitans) < cTechStatusResearching) &&
      (gTransportMap == false)) )
   gGlutRatio = 0.0;

   if (gSomeData != -1) {
      aiPlanSetUserVariableFloat(gSomeData, 10, 0, gGlutRatio);
      aiPlanSetUserVariableFloat(gSomeData, 11, 0, gFoodGlutRatio);
      aiPlanSetUserVariableFloat(gSomeData, 12, 0, gGoldGlutRatio);
      aiPlanSetUserVariableFloat(gSomeData, 13, 0, gWoodGlutRatio);
   }
}

void UpdateUnitPicksForAge(int ageid=-1)
{
  //TODO: Revamp numbers...
   int theUP = gRushUPID;
   if ( ageid >= cAge3 )
      theUP = gLateUPID;
   kbUnitPickSetPreferenceFactor( theUP, cUnitTypeSpearmanAztec, 0.1*(aiRandInt(7)+3) );
   kbUnitPickSetPreferenceFactor( theUP, cUnitTypeArcherAztec, 0.1*(aiRandInt(7)+3) );
   kbUnitPickSetPreferenceFactor( theUP, cUnitTypeRunnerAztec, 0.1*(aiRandInt(7)+3) );
   kbUnitPickSetPreferenceFactor( theUP, cUnitTypeCuexWarrior, 0.1*(aiRandInt(7)+3) );
   if ( ageid >= cAge3 ) {
//      kbUnitPickDestroy( gRushUPID ); gRushUPID = -1;
      kbUnitPickSetPreferenceFactor( theUP, cUnitTypeJaguarKnight, 0.1*(aiRandInt(8)+2) );
      kbUnitPickSetPreferenceFactor( theUP, cUnitTypeOtomKnight, 0.1*(aiRandInt(8)+2) );
      kbUnitPickSetPreferenceFactor( theUP, cUnitTypeEagleKnight, 0.1*(aiRandInt(8)+2) );
   }
   if ( ageid >= cAge4 )
      switch( MyCiv ) {
      case cCivQuetz: { kbUnitPickSetPreferenceFactor(theUP, cUnitTypeFeatheredWarrior, 0.1*(aiRandInt(2)+1)); break; }
      case cCivTezca: { kbUnitPickSetPreferenceFactor(theUP, cUnitTypeSkullKnight, 0.1*(aiRandInt(2)+1)); break; }
      default:        { kbUnitPickSetPreferenceFactor(theUP, cUnitTypeShornWarrior, 0.1*(aiRandInt(2)+1)); break; } }
}

//==============================================================================
rule updatePrices   // This rule constantly compares actual supply vs. forecast, updates AICost // values (internal resource prices), and buys/sells at the market as appropriate
minInterval 7
active
{
   // check for valid forecasts, exit if not ready
   updateGlutRatio();
   if (((gGoldForecast + gWoodForecast + gFoodForecast) < 100))
      return; 

   xsSetRuleMinIntervalSelf(6);								
   static int lastRequestTime = 0;	
   float scaleFactor = 5.0;      // Higher values make prices more volatile
   float goldStatus = 0.0;
   float woodStatus = 0.0;
   float foodStatus = 0.0;
   float minForecast = 200.0 * (1 + kbGetAge());	// 200, 400, 600, 800 in ages 1-5, prevents small amount from looking large if forecast is very low
   if (gGoldForecast > minForecast)
      goldStatus = scaleFactor * kbResourceGet(cResourceGold)/gGoldForecast;
   else
      goldStatus = scaleFactor * kbResourceGet(cResourceGold)/minForecast;
   if (gFoodForecast > minForecast)
      foodStatus = scaleFactor * kbResourceGet(cResourceFood)/gFoodForecast;
   else
      foodStatus = scaleFactor * kbResourceGet(cResourceFood)/minForecast;
   if (gWoodForecast > minForecast)
      woodStatus = scaleFactor * kbResourceGet(cResourceWood)/gWoodForecast;
   else
      woodStatus = scaleFactor * kbResourceGet(cResourceWood)/minForecast;

   // Status now equals inventory/forecast
   // Calculate value rate of wood:gold and food:gold.  1.0 means they're of the same status, 2.0 means 
   // that the resource is one forecast more scarce, 0.5 means one forecast more plentiful, i.e. lower value.
   float woodRate = (1.0 + goldStatus)/(1.0 + woodStatus);
   float foodRate = (1.0 + goldStatus)/(1.0 + foodStatus);

   // The rates are now the instantaneous price for each resource.  Set the long-term prices by averaging this in
   // at a 5% weight.
   float cost = 0.0;

   // wood
   cost = kbGetAICostWeight(cResourceWood);
   cost = (cost * 0.95) + (woodRate * .05);
   kbSetAICostWeight(cResourceWood, cost);

   // food
   cost = kbGetAICostWeight(cResourceFood);
   cost = (cost * 0.95) + (foodRate * .05);
   kbSetAICostWeight(cResourceFood, cost);

   // Gold
   kbSetAICostWeight(cResourceGold, 1.00);	// gold always 1.0, others relative to gold

   // Favor
   float favorCost = 15.0 - (14.0*(kbResourceGet(cResourceFavor)/100.0));     // 15 when empty, 2.0 when full
   if (favorCost < 1.0)
      favorCost = 1.0;
   kbSetAICostWeight(cResourceFavor, favorCost);  

   // Update the gather plan goal
   for (i = 0; < 3) {
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanResourceCostWeight, i, kbGetAICostWeight(i));
   }

   if ((aiGetGameMode() == cGameModeDeathmatch) && (xsGetTime() < 11*60*1000))
      return;

   //Compare that to the market price.  Buy if
   // the market price is lower and we have at least 
   // 1/3 forecast of gold.  Sell if market price is higher and
   // we have at least 1/3 forecast of the resource.
   if (kbUnitCount(cMyID, cUnitTypeMarket, cUnitStateAlive) > 0) {
      float reserve = 500.0;
      if (kbGetAge() > cAge3)
         reserve = 800.0;
      if ( (goldStatus > 0.33) && (kbResourceGet(cResourceGold) > reserve) ) {
         if (((aiGetMarketBuyCost(cResourceFood)/100.0) < kbGetAICostWeight(cResourceFood)) && (kbResourceGet(cResourceFood) < 600)) {
            aiBuyResourceOnMarket(cResourceFood);
         }
         if (((aiGetMarketBuyCost(cResourceWood)/100.0) < kbGetAICostWeight(cResourceWood)) && (kbResourceGet(cResourceWood) < 600)) {
            aiBuyResourceOnMarket(cResourceWood);
         }
      }

      if ((woodStatus > 0.33) && (kbResourceGet(cResourceWood) > reserve)) {
         if (((aiGetMarketSellCost(cResourceWood)/100.0) > kbGetAICostWeight(cResourceWood)) && (kbResourceGet(cResourceGold) < 600))
            aiSellResourceOnMarket(cResourceWood);
      }
      if ((foodStatus > 0.33) && (kbResourceGet(cResourceFood) > reserve)) {
         if (((aiGetMarketSellCost(cResourceFood)/100.0) > kbGetAICostWeight(cResourceFood)) && (kbResourceGet(cResourceGold) < 600))
            aiSellResourceOnMarket(cResourceFood);
      }
      if (kbResourceGet(cResourceGold) > 1300) {
         if (kbResourceGet(cResourceFood) < 1200) {
            if (kbResourceGet(cResourceGold) > 1800) {
               aiBuyResourceOnMarket(cResourceFood);
               xsSetRuleMinIntervalSelf(1);
            }
            else
               aiBuyResourceOnMarket(cResourceFood);
         }
         if (kbResourceGet(cResourceWood) < 800) {
            if (kbResourceGet(cResourceGold) > 1800) {
               aiBuyResourceOnMarket(cResourceWood);
               xsSetRuleMinIntervalSelf(1);
            }
            else
               aiBuyResourceOnMarket(cResourceWood);
         }
      }

      if (kbResourceGet(cResourceFood) > 1400) {
         if (kbResourceGet(cResourceGold) < 1200) {
            if (kbResourceGet(cResourceFood) > 1800) {
               aiSellResourceOnMarket(cResourceFood);
               xsSetRuleMinIntervalSelf(1);
            }				
            else {
               aiSellResourceOnMarket(cResourceFood);
            }
         }
      }

      // Special Treatment for resource equalizing
      if (gFoodGlutRatio > 1.50) {
         if (gGoldGlutRatio < 1.50) {
            aiSellResourceOnMarket(cResourceFood);
            xsSetRuleMinIntervalSelf(1);

         }
      }
      if (gWoodGlutRatio > 1.50) {
         if (gGoldGlutRatio < 1.50) {
            aiSellResourceOnMarket(cResourceWood);
            xsSetRuleMinIntervalSelf(1);

         }
      }
      if (gGoldGlutRatio > 1.50) {
         if (gFoodGlutRatio < 1.50) {
            aiBuyResourceOnMarket(cResourceFood);
            xsSetRuleMinIntervalSelf(1);
         }
         if (gWoodGlutRatio < 1.50) {
            aiBuyResourceOnMarket(cResourceWood);
            xsSetRuleMinIntervalSelf(1);
         }		
      }
   }

   if ( AoModAllies && (kbGetAge() > cAge1) ) {
      if ( ((xsGetTime() - lastRequestTime) > 300000) && ((xsGetTime() - gLastSentTime) > 120000) ) {
         float totalResources = kbResourceGet(cResourceFood) + kbResourceGet(cResourceWood) + kbResourceGet(cResourceGold);
         if (totalResources > 1000.0) {
            if ((kbResourceGet(cResourceFood) < (totalResources / 10.0) && (kbResourceGet(cResourceFood) < 3500))) {
               MessageRel(cPlayerRelationAlly, RequestFood, cLowPriority);
               lastRequestTime = xsGetTime();
            }
            if ((kbResourceGet(cResourceGold) < (totalResources / 10.0) && (kbResourceGet(cResourceGold) < 3500))) {
               MessageRel(cPlayerRelationAlly, RequestGold, cLowPriority);
               lastRequestTime = xsGetTime();
            }
            if ((kbResourceGet(cResourceWood) < (totalResources / 10.0) && (kbResourceGet(cResourceWood) < 3500))) {
               MessageRel(cPlayerRelationAlly, RequestWood, cLowPriority);
               lastRequestTime = xsGetTime();
            }
         }
      }
   } 	
}
//==============================================================================
void updateGathererRatios(void) //Check the forecast variables, check inventory, set assignments
{
   float goldSupply = kbResourceGet(cResourceGold);
   float woodSupply = kbResourceGet(cResourceWood);
   float foodSupply = kbResourceGet(cResourceFood);
   gFoodForecast = gFoodForecast * 1.2;

   float goldShortage = gGoldForecast - goldSupply;
   if (goldShortage < 0)
      goldShortage = 0;
   float woodShortage = gWoodForecast - woodSupply;
   if (woodShortage < 0)
      woodShortage = 0;
   float foodShortage = gFoodForecast - foodSupply;
   if (foodShortage < 0)
      foodShortage = 0;

   updateGlutRatio();
   float totalShortage = goldShortage + woodShortage + foodShortage;
   if (totalShortage < 1)
      totalShortage = 1;

   float worstShortageRatio = goldShortage/(gGoldForecast+1);
   if ( (woodShortage/(gWoodForecast+1)) > worstShortageRatio)
      worstShortageRatio = woodShortage/(gWoodForecast+1);
   if ( (foodShortage/(gFoodForecast+1)) > worstShortageRatio)
      worstShortageRatio = foodShortage/(gFoodForecast+1);


   float totalForecast = gGoldForecast + gWoodForecast + gFoodForecast;
   if (totalForecast < 1)
      totalForecast = 1;	// Avoid div by 0.

   int numGatherers = kbUnitCount(cMyID,cUnitTypeAbstractVillager, cUnitStateAlive);
   int numTradeCarts = kbUnitCount(cMyID, cUnitTypeAbstractTradeUnit, cUnitStateAlive);
   int numFishBoats = kbUnitCount(cMyID, cUnitTypeUtilityShip, cUnitStateAlive);
   if (numFishBoats >= 1)
      numFishBoats--; // Ignore scout

   int civPopTotal = numGatherers + numTradeCarts + numFishBoats;

   int doomedID = -1;   // Who to kill...
   if (civPopTotal > (aiGetEconomyPop() + 5)) {
      bool Caravan = false;
      if ( numTradeCarts > gMaxTradeCarts ) {
         //find idle units first
         doomedID = findUnit(cUnitTypeAbstractTradeUnit, cUnitStateAlive, cActionIdle, cMyID);
         if (doomedID < 0)
            doomedID = findUnit(cUnitTypeAbstractTradeUnit);
      }
      else {
         doomedID = findUnit(cUnitTypeAbstractVillager, cUnitStateAlive, cActionIdle, cMyID);
         if (doomedID < 0)
            doomedID = findUnit(cUnitTypeAbstractVillager);
         if (doomedID != -1)
            Caravan = true;
      }
      aiTaskUnitDelete(doomedID);
      civPopTotal--;	
      if ( Caravan )
         numTradeCarts--;
      else
         numGatherers--;
   }

   // Figure out what percent of our total civ pop we want working on each resource.  To do that,
   // figure out what the percentages would be to match our shortages, and the percents to match
   // our forecast, and come up with a weighted average of the two. That way, if we don't have a wood shortage
   // at the moment, but we do expect to keep using wood, we'll keep some villagers on wood.
   // This much (forecastWeight) of the allocation is based on forecast, the rest on shortages.
   // If the biggest shortage is nearly equal to the forecast (nothing on hand), let
   // the shortage dominate. If the shortage is relatively small, let the forecast dominate
   float forecastWeight = 1.0 - worstShortageRatio;	
      float goldForecastRatio = gGoldForecast / totalForecast;
   float woodForecastRatio = gWoodForecast / totalForecast;
   float foodForecastRatio = gFoodForecast / totalForecast;

   float goldShortageRatio = 0.0;
   if (totalShortage > 0)
      goldShortageRatio = goldShortage / totalShortage;
   float woodShortageRatio = 0.0;
   if (totalShortage > 0)
      woodShortageRatio = woodShortage / totalShortage;
   float foodShortageRatio = 0.0;
   if (totalShortage > 0)
      foodShortageRatio = foodShortage / totalShortage;

   float desiredGoldRatio = forecastWeight*goldForecastRatio + (1.0-forecastWeight)*goldShortageRatio;
      float desiredWoodRatio = forecastWeight*woodForecastRatio + (1.0-forecastWeight)*woodShortageRatio;
      float desiredFoodRatio = forecastWeight*foodForecastRatio + (1.0-forecastWeight)*foodShortageRatio;

      // We now have the desired ratios, which can be converted to total civilian units, but then need to be adjusted for trade
   // carts and fishing boats.
   float desiredGoldUnits = desiredGoldRatio * civPopTotal;
   float desiredWoodUnits = desiredWoodRatio * civPopTotal;
   float desiredFoodUnits = desiredFoodRatio * civPopTotal;

   float neededGoldGatherers = desiredGoldUnits - numTradeCarts;
   int mainBaseID = kbBaseGetMainID(cMyID);
   int numMainBaseGoldSites = kbGetNumberValidResources(mainBaseID, cResourceGold, cAIResourceSubTypeEasy);
   int numGoldBaseSites = 0;
   if ((gGoldBaseID >= 0) && (gGoldBaseID != mainBaseID))
      numGoldBaseSites = kbGetNumberValidResources(gGoldBaseID, cResourceGold, cAIResourceSubTypeEasy);
   int numGoldSites = numMainBaseGoldSites + numGoldBaseSites;

   float neededWoodGatherers = desiredWoodUnits;
   int numberMainBaseSites = kbGetNumberValidResources(mainBaseID, cResourceWood, cAIResourceSubTypeEasy);
   int numWoodBaseSites  = 0;
   if ((gWoodBaseID >= 0) && (gWoodBaseID != mainBaseID))
      numWoodBaseSites = kbGetNumberValidResources(gWoodBaseID, cResourceWood, cAIResourceSubTypeEasy);
   int numWoodSites = numberMainBaseSites + numWoodBaseSites;

   float neededFoodGatherers = desiredFoodUnits - numFishBoats;

   int minFoodGatherers = 1.5 + numGatherers*0.25;
   if (minFoodGatherers < 6)
      minFoodGatherers = 6;
   if ((numFishBoats < 4) && (kbGetAge() > cAge2) && (numWoodSites < 1) || (numGoldSites < 1))
      minFoodGatherers = minFoodGatherers*2;	
   if (aiGetWorldDifficulty() == cDifficultyEasy)
      minFoodGatherers = minFoodGatherers / 2;	
   if (neededFoodGatherers < minFoodGatherers)
      neededFoodGatherers = minFoodGatherers;

   if (neededGoldGatherers < 0)
      neededGoldGatherers = 0;
   if (neededFoodGatherers < 0)
      neededFoodGatherers = 0;
   if (neededWoodGatherers < 0)
      neededWoodGatherers = 0;

   float totalNeededGatherers = neededGoldGatherers + neededFoodGatherers + neededWoodGatherers;
   // Note, this total may be different than the total gatherers, if the trade carts are more than needed, or if
   // the fishing boats supply more food than we need, so this number may be lower...and should be used as the basis
   // for assigning villager percentages.

   float goldAssignment = neededGoldGatherers / totalNeededGatherers;
   float woodAssignment = neededWoodGatherers / totalNeededGatherers;
   float foodAssignment = neededFoodGatherers / totalNeededGatherers;
   float lastGoldAssignment = aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceGold);
   float lastWoodAssignment = aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceWood);
   float lastFoodAssignment = aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFood);

   if (kbGetAge() > cAge1) {
      if (neededGoldGatherers > 0) {
         if (goldAssignment > lastGoldAssignment)
            goldAssignment = xsMin( lastGoldAssignment + 0.03, 0.45 );
         else if (goldAssignment < lastGoldAssignment)
            goldAssignment = xsMax( lastGoldAssignment - 0.03, 0.05 );
      }
      if (neededWoodGatherers > 0) {
         if (woodAssignment > lastWoodAssignment)
            woodAssignment = xsMin( lastWoodAssignment + 0.03, 0.45 );
         else if (woodAssignment < lastWoodAssignment)
            woodAssignment = xsMax( lastWoodAssignment - 0.03, 0.05 );
      }
      if (neededFoodGatherers > 0) {
         if (foodAssignment > lastFoodAssignment)
            foodAssignment = xsMin( lastFoodAssignment + 0.03, 0.45 );
         else if (foodAssignment < lastFoodAssignment)
            foodAssignment = xsMax( lastFoodAssignment - 0.03, 0.25 );
      }
   }
   //Test
   //if we lost a lot of villagers, keep them close to our settlements (=farming)
   int minVillagers = 14;
   int numVillagers = kbUnitCount(cMyID, cUnitTypeAbstractVillager, cUnitStateAlive);
   if ((numVillagers <= minVillagers) && (kbGetAge() > cAge2) && (xsGetTime() > 16*60*1000)) {
      goldAssignment = 0.05;
      woodAssignment = 0.05; 
      foodAssignment = 0.90;
   }
   //Test end

   aiSetResourceGathererPercentageWeight(cRGPScript, 1.0);
   aiSetResourceGathererPercentageWeight(cRGPCost, 0.0);
   aiSetResourceGathererPercentage(cResourceGold, goldAssignment, false, cRGPScript);
   aiSetResourceGathererPercentage(cResourceWood, woodAssignment, false, cRGPScript);
   aiSetResourceGathererPercentage(cResourceFood, foodAssignment, false, cRGPScript);

   aiNormalizeResourceGathererPercentages(cRGPScript);
   aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceGold, aiGetResourceGathererPercentage(cResourceGold, cRGPScript));
   aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceWood, aiGetResourceGathererPercentage(cResourceWood, cRGPScript));
   aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFood, aiGetResourceGathererPercentage(cResourceFood, cRGPScript));
   aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFavor, aiGetResourceGathererPercentage(cResourceFavor, cRGPScript));

   if (gSomeData != -1)
      { 
      int numTradeUnits = kbUnitCount(cMyID, cUnitTypeAbstractTradeUnit, cUnitStateAlive);
      aiPlanSetUserVariableInt(gSomeData, cResourceFood, 0 , gFoodForecast);
      aiPlanSetUserVariableInt(gSomeData, cResourceGold, 0 , gGoldForecast);
      aiPlanSetUserVariableInt(gSomeData, cResourceWood, 0 , gWoodForecast);	
      aiPlanSetUserVariableFloat(gSomeData, 4, 0 , aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFood) * 100);
      aiPlanSetUserVariableFloat(gSomeData, 5, 0 , aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceGold) * 100);
      aiPlanSetUserVariableFloat(gSomeData, 6, 0 , aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceWood) * 100);
      aiPlanSetUserVariableInt(gSomeData, 8, 0 , numTradeUnits);
   }
}

void addUnitForecast(int unitTypeID=-1, int qty=1)
{
   gGoldForecast = gGoldForecast + kbUnitCostPerResource(unitTypeID, cResourceGold)*qty;
   gWoodForecast = gWoodForecast + kbUnitCostPerResource(unitTypeID, cResourceWood)*qty;
   gFoodForecast = gFoodForecast + kbUnitCostPerResource(unitTypeID, cResourceFood)*qty;
}

void addTechForecast(int techID=-1)
{
   gGoldForecast = gGoldForecast + kbTechCostPerResource(techID, cResourceGold);
   gWoodForecast = gWoodForecast + kbTechCostPerResource(techID, cResourceWood);
   gFoodForecast = gFoodForecast + kbTechCostPerResource(techID, cResourceFood);
}

//==============================================================================
// setMilitaryUnitCostForecast
// Checks the current age, looks into the appropriate unit picker,
// calculates approximate resource needs for the next few (3?) minutes,
// adds this amount to the global vars.
//==============================================================================
void setMilitaryUnitCostForecast(void)
{
   int upID = -1;  // ID of the unit picker to query
   float totalAmount = 0.0;	// Total resources to be spent in near future
   if (kbGetAge() == cAge2) {
      upID = gRushUPID;
      totalAmount = 1200;
   }
   if (kbGetAge() == cAge3) {
      upID = gLateUPID;
      totalAmount = 3000;
   }
   if (kbGetAge() >= cAge4) {
      upID = gLateUPID;
      totalAmount = 5000;
   }

   float goldCost = 0.0;
   float woodCost = 0.0;
   float foodCost = 0.0;
   float totalCost = 0.0;

   int unitID = kbUnitPickGetResult( upID, 0); // Primary unit
   float weight = 1.0;
   int numUnits = kbUnitPickGetDesiredNumberUnitTypes(upID);

   if (numUnits == 2)
      weight = 0.67; // 2/3 and 1/3
   if (numUnits >= 3)
      weight = 0.50; // 1/2, 1/3, 1/6
   if (gSomeData != -1)
      aiPlanSetUserVariableString(gSomeData, MainUnit, 0, ""+kbGetProtoUnitName(unitID));

   goldCost = kbUnitCostPerResource(unitID, cResourceGold);
   woodCost = kbUnitCostPerResource(unitID, cResourceWood);
   foodCost = kbUnitCostPerResource(unitID, cResourceFood);
   totalCost = goldCost+woodCost+foodCost;

   gGoldForecast = gGoldForecast + goldCost * (totalAmount*weight/totalCost);
   gWoodForecast = gWoodForecast + woodCost * (totalAmount*weight/totalCost);
   gFoodForecast = gFoodForecast + foodCost * (totalAmount*weight/totalCost);

   if (numUnits > 1)
      {  // Do second unit
      unitID = kbUnitPickGetResult(upID, 1);
      weight = 0.30;	// Second is 1/3 regardless 
      if (gSomeData != -1)
         aiPlanSetUserVariableString(gSomeData, SecondaryUnit, 0, ""+kbGetProtoUnitName(unitID));
      goldCost = kbUnitCostPerResource(unitID, cResourceGold);
      woodCost = kbUnitCostPerResource(unitID, cResourceWood);
      foodCost = kbUnitCostPerResource(unitID, cResourceFood);
      totalCost = goldCost+woodCost+foodCost;

      gGoldForecast = gGoldForecast + goldCost * (totalAmount*weight/totalCost);
      gWoodForecast = gWoodForecast + woodCost * (totalAmount*weight/totalCost);
      gFoodForecast = gFoodForecast + foodCost * (totalAmount*weight/totalCost);
   }

   if (numUnits > 2)
      {  // Do third unit
      unitID = kbUnitPickGetResult(upID, 2);
      weight = 0.20;   // Third unit, if used, is 1/6
      if (gSomeData != -1)
         aiPlanSetUserVariableString(gSomeData, ThirdUnit, 0, ""+kbGetProtoUnitName(unitID));			
      goldCost = kbUnitCostPerResource(unitID, cResourceGold);
      woodCost = kbUnitCostPerResource(unitID, cResourceWood);
      foodCost = kbUnitCostPerResource(unitID, cResourceFood);
      totalCost = goldCost+woodCost+foodCost;

      gGoldForecast = gGoldForecast + goldCost * (totalAmount*weight/totalCost);
      gWoodForecast = gWoodForecast + woodCost * (totalAmount*weight/totalCost);
      gFoodForecast = gFoodForecast + foodCost * (totalAmount*weight/totalCost);
   }
}

void AddGenForecastNeeds(void)
{
   // Villagers & Farms   
   int temp = aiPlanGetVariableInt(gCivPopPlanID, cTrainPlanNumberToMaintain, 0);  
   temp = temp - kbUnitCount(cMyID, cUnitTypeAbstractVillager, cUnitStateAlive);
   temp = temp + kbUnitCount(cMyID, cUnitTypeDwarf, cUnitStateAlive);
   if ( temp > 13 )
      temp = 13;
   addUnitForecast( cUnitTypeVillagerAztec, temp );	

   if ((gFarming) || (kbGetAge() >= cAge2)) {
      float foodGatherersWanted = aiPlanGetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFood);   // Percent food gatherers
      foodGatherersWanted = foodGatherersWanted * kbUnitCount(cMyID, cUnitTypeAbstractVillager, cUnitStateAlive); // Actual count
      temp = foodGatherersWanted - kbUnitCount(cMyID, cUnitTypeFarm, cUnitStateAliveOrBuilding);
      if (temp < 1)
         temp = 1;
      else if (temp > 5)
         temp = 5;
      if (temp > 0)
         addUnitForecast(cUnitTypeFarm, temp);		
   }  

   if (gFishing) {
      if (kbUnitCount(cMyID, cUnitTypeDock, cUnitStateAliveOrBuilding) < 1)
         addUnitForecast(cUnitTypeDock, 1);
      int Number = 50;
      if (kbGetAge() < cAge2)
         Number = 125;
      const int fishBoatType = cUnitTypeFishingShipAztec;
      int boatCount = kbUnitCount(cMyID, fishBoatType, cUnitStateAlive);
      temp = gNumBoatsToMaintain - boatCount;
      if (temp <= 1)
         temp = 1;
      gWoodForecast = gWoodForecast + Number*temp;
   }
   if (kbUnitCount(cMyID, cUnitTypeTemple, cUnitStateAliveOrBuilding)  < 1)
      addUnitForecast(cUnitTypeTemple, 1);

   if ((kbGetAge() >= cAge2) || (kbGetTechStatus(gAge2MinorGod) == cTechStatusResearching)) {
      if (gBuildWalls)
         gGoldForecast = gGoldForecast + 300;

      // Settlements
      if (kbUnitCount(0, cUnitTypeAbstractSettlement) > 0)
         addUnitForecast(cUnitTypeSettlementLevel1, 1);

      const int Armory = cUnitTypeGuildMeso;
      if ( kbUnitCount(cMyID, Armory, cUnitStateAliveOrBuilding) < 1 )
         addUnitForecast(Armory, 1);

      if ( kbGetAge() >= cAge3 )
         addUnitForecast(cUnitTypeMarket, 1);	

      // military buildings
      gWoodForecast = gWoodForecast + 200;

      if (gNavalAttackGoalID != -1) {
         // Ships
         int myShips = kbUnitCount(cMyID, cUnitTypeLogicalTypeNavalMilitary, cUnitStateAlive);
         temp = gTargetNavySize+2 - myShips;   // How many yet to train
         if (temp < 0)
            temp = 0;
         if (temp > 0) {
            gWoodForecast = gWoodForecast + 175*temp;
            gGoldForecast = gGoldForecast + 100*temp;
         }		
      }        
      if (kbGetAge() >= cAge3) {
         if (gTransportMap)
            gWoodForecast = gWoodForecast + 350;		
         // Fortress, etc.
         addUnitForecast(MyFortress, 1);

         // Fortified TC
         if (kbGetTechStatus(cTechFortifyTownCenter) < cTechStatusResearching) 
            addTechForecast(cTechFortifyTownCenter);			
         }
   }		
}

//==============================================================================
rule econForecastAge4		// Rule activates when age 4 research begins
minInterval 12
inactive
{	
   static int ageStartTime = -1;
   int temp = 0;

   if ( (kbGetAge() == cAge3) && (kbGetTechStatus(gAge4MinorGod) < cTechStatusResearching) ) {
      xsDisableSelf();
      xsEnableRule("econForecastAge3");
      return;
   }
   else if ((kbGetAge() > cAge3) && (ageStartTime == -1))
      ageStartTime = xsGetTime();

   gGoldForecast = 600;
   gWoodForecast = 600;
   gFoodForecast = 600;

   float goldSupply = kbResourceGet(cResourceGold);
   float woodSupply = kbResourceGet(cResourceWood);
   float foodSupply = kbResourceGet(cResourceFood);

   if ((ageStartTime != -1) && (xsGetTime() - ageStartTime > 5*60*1000)) {
      if (foodSupply < 1400)
         gFoodForecast = gFoodForecast + (1400 - foodSupply);
      if (woodSupply < 1200)
         gWoodForecast = gWoodForecast + (1200 - woodSupply);
      if (goldSupply < 1400)
         gGoldForecast = gGoldForecast + (1400 - goldSupply);
   }
   else {
      if (goldSupply < 500)
         gGoldForecast = gGoldForecast + (500 - goldSupply);
      if (woodSupply < 500)
         gWoodForecast = gWoodForecast + (500 - woodSupply);
      if (foodSupply < 500)
         gFoodForecast = gFoodForecast + (500 - foodSupply);
   }

   if ((TitanAvailable) && (kbGetTechStatus(cTechSecretsoftheTitans) < cTechStatusResearching) && (xsGetTime() - ageStartTime > 4*60*1000))
      addTechForecast(cTechSecretsoftheTitans);

   AddGenForecastNeeds();
   setMilitaryUnitCostForecast(); // add units before scaling down
   updateGathererRatios();
}

//==============================================================================
rule econForecastAge3		// Rule activates when age3 research begins, turns off when age 4 research begins
minInterval 11
inactive
{
   static int ageStartTime = -1;
   if ((kbGetTechStatus(gAge4MinorGod) >=  cTechStatusResearching) || (kbGetAge() > cAge3)) {
      xsEnableRule("econForecastAge4");
      econForecastAge4();
      xsDisableSelf();
      return;		// We're done
   }
   else if ((kbGetAge() == cAge2) && (kbGetTechStatus(gAge3MinorGod) < cTechStatusResearching)) {
      xsDisableSelf();
      xsEnableRule("econForecastAge2");
      return;
   }
   else if ((kbGetAge() == cAge3) && (ageStartTime == -1))
      ageStartTime = xsGetTime();

   gFoodForecast = 500;
   gGoldForecast = 500;
   gWoodForecast = 500;

   float goldSupply = kbResourceGet(cResourceGold);
   float woodSupply = kbResourceGet(cResourceWood);
   float foodSupply = kbResourceGet(cResourceFood);

   if ((ageStartTime != -1) && (xsGetTime() - ageStartTime > 4*60*1000)) {
      if (goldSupply < 1500)
         gGoldForecast = gGoldForecast + (1500 - goldSupply);
      if (foodSupply < 1500)
         gFoodForecast = gFoodForecast + (1500 - foodSupply);
      if (woodSupply < 500)
         gWoodForecast = gWoodForecast + (500 - woodSupply);
   }	
   else {
      if (goldSupply < 350)
         gGoldForecast = gGoldForecast + (350 - goldSupply);
      if (foodSupply < 350)
         gFoodForecast = gFoodForecast + (350 - foodSupply);
      if (woodSupply < 350)
         gWoodForecast = gWoodForecast + (350 - woodSupply);
   }	

   AddGenForecastNeeds();
   setMilitaryUnitCostForecast(); // add units before scaling down
   updateGathererRatios();
}

//==============================================================================
rule econForecastAge2		// Rule activates when age 2 research begins, turns off when age 3 research begins
minInterval 10
inactive
{
   static int ageStartTime = -1;
   if ((kbGetTechStatus(gAge3MinorGod) >= cTechStatusResearching) || (kbGetAge() > cAge2)) {
      xsEnableRule("econForecastAge3");
      econForecastAge3();
      xsDisableSelf();
      return;		// We're done
   }
   else if ((kbGetAge() == cAge1) && (kbGetTechStatus(gAge2MinorGod) < cTechStatusResearching)) {
      xsDisableSelf();
      xsEnableRule("econForecastAge1");
      return;
   }
   else if ((kbGetAge() == cAge2) && (ageStartTime == -1))
      ageStartTime = xsGetTime();

   // If we've made it here, we're in age 2 (or researching it)
   gFoodForecast = 400;
   gGoldForecast = 400;
   gWoodForecast = 400;

   float goldSupply = kbResourceGet(cResourceGold);
   float woodSupply = kbResourceGet(cResourceWood);
   float foodSupply = kbResourceGet(cResourceFood);  

   if ((ageStartTime != -1) && (xsGetTime() - ageStartTime > 3*60*1000)) {
      if (goldSupply < 800)
         gGoldForecast = gGoldForecast + (800 - goldSupply);
      if (foodSupply < 1200)
         gFoodForecast = gFoodForecast + (1200 - foodSupply);
      if (woodSupply < 400)
         gWoodForecast = gWoodForecast + (400 - woodSupply);
   }	
   else {
      if (goldSupply < 250)
         gGoldForecast = gGoldForecast + (250 - goldSupply);
      if (foodSupply < 250)
         gFoodForecast = gFoodForecast + (250 - foodSupply);
      if (woodSupply < 250)
         gWoodForecast = gWoodForecast + (250 - woodSupply);
   }
   // plow etc
   if ((kbGetTechStatus(cTechPlow) < cTechStatusResearching) && (kbUnitCount(cMyID, cUnitTypeFarm, cUnitStateAliveOrBuilding) > 0))
      addTechForecast(cTechPlow);
   if (kbGetTechStatus(cTechShaftMine) < cTechStatusResearching)
      addTechForecast(cTechShaftMine);
   if (kbGetTechStatus(cTechBowSaw) < cTechStatusResearching)
      addTechForecast(cTechBowSaw);

   AddGenForecastNeeds();
   if ((xsGetTime() - ageStartTime > 2*60*1000) && (kbGetAge() == cAge2))
      setMilitaryUnitCostForecast(); // add units before scaling down

   updateGathererRatios();
}

rule QuetzTlatoani minInterval 10 inactive
{
   //This could be improved, but works for now.
   int tlatoaniid = rotgGetTlatoaniID();
   if ( kbUnitGetCurrentHitpoints(tlatoaniid) > 0.0 ) {
      if ( kbUnitGetAnimationActionType(tlatoaniid) == cActionIdle ) {
         int targetid = findUnitByRelByIndex( cUnitTypeMinedResource, 0, cUnitStateAny, -1, cPlayerRelationAny, kbUnitGetPosition(tlatoaniid), 40.0 );
         if ( targetid < 0 )
            targetid = findUnitByRelByIndex( cUnitTypeTree, 0, cUnitStateAny, -1, cPlayerRelationAny, kbUnitGetPosition(tlatoaniid), 40.0 );
         if ( targetid < 0 )
            targetid = findUnitByIndex( cUnitTypeDropsite, 0, cUnitStateBuilding, -1, cMyID, kbUnitGetPosition(tlatoaniid), 40.0 );
         if ( targetid < 0 )
            targetid = findUnitByIndex( cUnitTypeLogicalTypeHouses, 0, cUnitStateBuilding, -1, cMyID, kbUnitGetPosition(tlatoaniid), 40.0 );
         aiTaskUnitWork( tlatoaniid, targetid );
      }
   }
}
rule QuetzTechs runImmediately minInterval 60 inactive  //activates in Age2
{
   int buildid = findUnit( cUnitTypeLogicalTypeHouses );
   if ( buildid < 0 ) {
      buildid = findUnit( cUnitTypeAbstractFarm );
   if ( buildid < 0 ) {
      buildid = findUnit( cUnitTypeAbstractWall );
   }}
   int actives = 0;
   actives = try_tech( buildid, cTechQuetzBook, actives );
   if ( actives > 0 ) {
      actives = try_tech( buildid, cTechGiftOfCorn, actives );
      actives = try_tech( buildid, cTechCoatenpantli, actives );
      actives = try_tech( buildid, cTechSummonSerpent, actives );
      if ( actives >= 4 )
         xsDisableSelf();
   }
}

//============================================================================================================
rule econForecastAge1  //Rule active for mid age 1 (cAge1), gets started in setEarlyEcon rule, ending when next age upgrade starts
minInterval 6
inactive
{
   if (kbGetAge() > cAge1) {
      xsDisableSelf();
      xsEnableRule("econForecastAge2");
      return;
   }
   if ( kbGetTechStatus(gAge2MinorGod) < cTechStatusResearching ) {
      if ( kbResourceGet(cResourceFood) >= 400 )
         aiTaskUnitResearch( findUnit(cUnitTypeAbstractSettlement), gAge2MinorGod );
   }
   else {	// Next age upgrade is on the way
      xsDisableSelf();
      xsEnableRule("econForecastAge2");
      econForecastAge2();	// Since runImmediately doesn't seem to be working
      gHouseAvailablePopRebuild = 18;		
      return;
   }

   gFoodForecast = 600.0;
   gGoldForecast = 100.0;
   gWoodForecast = 200.0;
   if (xsGetTime() > 1*60*1000) {
      gGoldForecast = gGoldForecast + 200;
      gWoodForecast = gWoodForecast + 300;
   }
   AddGenForecastNeeds();
   updateGathererRatios();
}

//Called for ages 1-4
void ExplorerUpdater(int ageid=-1)
{
   switch( ageid ) {
      case cAge1: {
         if ( cvMapSubType != VINLANDSAGAMAP ) {
            int pid = aiPlanCreate( "PriestExplore", cPlanExplore );
            aiPlanSetVariableBool( pid, cExplorePlanDoLoops, 0, false );
            aiPlanAddUnitType( pid, cUnitTypeWarPriest, 1, 1, 1 );
            aiPlanSetDesiredPriority( pid, 55 );
            aiPlanSetActive( pid );
            if ( MyCiv == cCivQuetz ) {
               for(x=0; < 3) {
                  pid = aiPlanCreate( "BirdExplore"+x, cPlanExplore );
                  aiPlanSetVariableBool( pid, cExplorePlanDoLoops, 0, false );
                  aiPlanSetVariableBool( pid, cExplorePlanAvoidingAttackedAreas, 0, true );
                  aiPlanAddUnitType( pid, cUnitTypeQuetzalBird, 1, 1, 1 );
                  aiPlanSetDesiredPriority( pid, 100 );
                  aiPlanSetActive( pid );
               }
            }
            else {
               gLandExplorePlanID = aiPlanCreate( "TlatoaniExplore", cPlanExplore );
               aiPlanSetVariableBool( gLandExplorePlanID, cExplorePlanDoLoops, 0, false );
               aiPlanAddUnitType( gLandExplorePlanID, cUnitTypeTlatoani, 1, 1, 1 );
               aiPlanSetDesiredPriority( gLandExplorePlanID, 55 );
               aiPlanSetActive( gLandExplorePlanID );
            }
            if ( MyCiv == cCivTezca ) {
               pid = aiPlanCreate( "ShadowExplore", cPlanExplore );
               aiPlanSetVariableBool( pid, cExplorePlanDoLoops, 0, false );
               aiPlanAddUnitType( pid, cUnitTypeShadowRunner, 1, 1, 1 );
               aiPlanSetDesiredPriority( pid, 55 );
               aiPlanSetActive( pid );
            }
         }
         if ( gWaterMap ) {
            pid = aiPlanCreate( "WaterExplore", cPlanExplore );
            aiPlanSetVariableBool( pid, cExplorePlanDoLoops, 0, false );
            aiPlanAddUnitType( pid, gWaterScout, 1, 1, 1 );
            aiPlanSetDesiredPriority( pid, 80 );
            aiPlanSetActive( pid );
         }
         break;
      }
      case cAge2: {
         aiPlanDestroy( findPlanByString("PriestExplore", cPlanExplore, -1, true) );
         aiPlanDestroy( findPlanByString("ShadowExplore", cPlanExplore, -1, true) );
         aiPlanDestroy( findPlanByString("TlatoaniExplore", cPlanExplore, -1, true) );
         gLandScout = cUnitTypeRunnerAztec;
         pid = aiPlanCreate( "RunnerExplore", cPlanExplore );
         aiPlanSetVariableBool( pid, cExplorePlanDoLoops, 0, false );
         aiPlanAddUnitType( pid, gLandScout, 1, 1, 1 );
         aiPlanSetDesiredPriority( pid, 100 );
         aiPlanSetActive( pid );
         break;
      }
   }  //End: switch()

   if ( MyCiv == cCivQuetz ) {
      int birdPlan = aiPlanGetIDByTypeAndVariableType(cPlanTrain, cTrainPlanUnitType, cUnitTypeQuetzalBird);
      if ( birdPlan < 0 ) {
         birdPlan = createSimpleMaintainPlan( cUnitTypeQuetzalBird, 1, true, -1 );
         aiPlanSetVariableInt( birdPlan, cTrainPlanBuildFromType, 0, cUnitTypeTemple );
      }
      if ( kbGetAge() > cAge1 )
         aiPlanSetVariableInt( birdPlan, cTrainPlanNumberToMaintain, 0, xsMax(1, 4 - kbGetAge()) );
   }
}

rule SkullTempleMonitor runImmediately minInterval 20 active
{
   if ( kbUnitCount(cMyID, cUnitTypeSkullTemple, cUnitStateAliveOrBuilding) >= kbGetBuildLimit(cMyID, cUnitTypeSkullTemple) )
      return;
   if ( aiPlanGetIDByTypeAndVariableType(cPlanBuild, cBuildPlanBuildingTypeID, cUnitTypeSkullTemple) >= 0 )
      return;
   int escrowid = cMilitaryEscrowID;
   if ( kbGetAge() <= cAge1 )
      escrowid = cEconomyEscrowID;
   vector theLoc = calcMonumentPos( 1 );
   int pid = createSimpleBuildPlan( cUnitTypeSkullTemple, 1, 95, false, true, escrowid, kbBaseGetMainID(cMyID), 0 );
   aiPlanSetVariableInt( pid, cBuildPlanInfluenceUnitTypeID, 0, cUnitTypeSkullTemple );
   aiPlanSetVariableFloat( pid, cBuildPlanInfluenceUnitDistance, 0, 24.0 );
   aiPlanSetVariableFloat( pid, cBuildPlanInfluenceUnitValue, 0, -50.0 );  //Avoid aura of other tzompantlis
   aiPlanSetVariableVector( pid, cBuildPlanInfluencePosition, 0, theLoc );
   aiPlanSetVariableFloat( pid, cBuildPlanInfluencePositionDistance, 0, 15.0 );
   aiPlanSetVariableFloat( pid, cBuildPlanInfluencePositionValue, 0, 500.0 );
   aiPlanSetVariableVector( pid, cBuildPlanCenterPosition, 0, theLoc );
   aiPlanSetVariableFloat( pid, cBuildPlanCenterPositionDistance, 0, 20.0 );
   aiPlanAddUnitType( pid, cUnitTypeWarPriest, 1, 2, 2 );
}


//=======================================================================================
void initAztec(void)
{
   //Aztec.
   gWaterScout = cUnitTypeFishingShipAztec;
   gLandScout = cUnitTypeWarPriest;
   if ( MyCiv == cCivQuetz ) {
      gAirScout = cUnitTypeQuetzalBird;
      gGatherRelicType = cUnitTypeWarPriest;
      createSimpleResearchPlan( cTechWarPriestCeremony, cUnitTypeTemple, cEconomyEscrowID, 90, false );
      xsEnableRule( "QuetzTlatoani" );
   }
   else {
      gGatherRelicType = cUnitTypeTlatoani;
      if ( MyCiv == cCivTezca )
         gWaterScout = cUnitTypeMermaidAztec;
   }
   ExplorerUpdater( cAge1 );

   //Make sure we always have at least 1 Scout
   aiResourceCheat( cMyID, cResourceFavor, 10.0 );  //For now.
   createSimpleMaintainPlan( gLandScout, 1, false, kbBaseGetMainID(cMyID) );
   aiSetAutoFavorGather( false );

   //Default to random minor god choices, override below if needed
   assignAztecMinorGods();

   //Control variable overrides
   if (cvAge2GodChoice != -1)
      gAge2MinorGod = cvAge2GodChoice;
   if (cvAge3GodChoice != -1)
      gAge3MinorGod = cvAge3GodChoice;
   if (cvAge4GodChoice != -1)
      gAge4MinorGod = cvAge4GodChoice;
}

//==============================================================================
int initUnitPicker(string name="BUG", int numberTypes=1, int minUnits=10,
                   int maxUnits=20, int minPop=-1, int maxPop=-1, int numberBuildings=1)
{
   //Create it.
   int upID = kbUnitPickCreate(name);
   //Default init.
   kbUnitPickResetAll(upID);
   //1 Part Preference, 2 Parts CE, 2 Parts Cost.
   kbUnitPickSetPreferenceWeight(upID, 2.0);
   kbUnitPickSetCombatEfficiencyWeight(upID, 4.0);
   kbUnitPickSetCostWeight(upID, 7.0);
   //Desired number units types, buildings.
   kbUnitPickSetDesiredNumberUnitTypes(upID, numberTypes, numberBuildings, true);
   //Min/Max units and Min/Max pop.
   kbUnitPickSetMinimumNumberUnits(upID, minUnits);
   kbUnitPickSetMaximumNumberUnits(upID, maxUnits);
   kbUnitPickSetMinimumPop(upID, minPop);
   kbUnitPickSetMaximumPop(upID, maxPop);
   //Default to land units.
   kbUnitPickSetAttackUnitType(upID, cUnitTypeLogicalTypeLandMilitary);
   kbUnitPickSetGoalCombatEfficiencyType(upID, cUnitTypeLogicalTypeMilitaryUnitsAndBuildings);
   //Setup the military unit preferences.
   kbUnitPickSetPreferenceFactor(upID, cUnitTypeAbstractSiegeWeapon, 0.33);
   kbUnitPickSetPreferenceFactor(upID, cUnitTypeLogicalTypeMythUnitNotTitan, 1.0);
   kbUnitPickSetPreferenceFactor(upID, cUnitTypeDryad, 0.0);  //This should *only* be produced through the hesperides rule
   kbUnitPickSetPreferenceFactor(upID, cUnitTypeHero, 0.0);  //Aztec heroes require special plans.
   //Done.
   return(upID);
}

//==============================================================================
void init(void)
{
   xsEnableRule("updateBreakdowns");
   xsEnableRule("updateFoodBreakdown");
   //We're in a random map.
   aiSetRandomMap(true);

   //Adjust control variable sliders by random amount
   cvRushBoomSlider = (cvRushBoomSlider - cvSliderNoise) + (cvSliderNoise * (aiRandInt(201))/100.0);
   if (cvRushBoomSlider > 1.0)
      cvRushBoomSlider = 1.0;
   if (cvRushBoomSlider < -1.0)
      cvRushBoomSlider = -1.0;
   cvMilitaryEconSlider = (cvMilitaryEconSlider - cvSliderNoise) + (cvSliderNoise * (aiRandInt(201))/100.0);
   if (cvMilitaryEconSlider > 1.0)
      cvMilitaryEconSlider = 1.0;
   if (cvMilitaryEconSlider < -1.0)
      cvMilitaryEconSlider = -1.0;
   cvOffenseDefenseSlider = (cvOffenseDefenseSlider - cvSliderNoise) + (cvSliderNoise * (aiRandInt(201))/100.0);
   if (cvOffenseDefenseSlider > 1.0)
      cvOffenseDefenseSlider = 1.0;
   if (cvOffenseDefenseSlider < -1.0)
      cvOffenseDefenseSlider = -1.0;
   if (ShowAiEcho) aiEcho("Sliders are...RushBoom "+cvRushBoomSlider+", MilitaryEcon "+cvMilitaryEconSlider+", OffenseDefense "+cvOffenseDefenseSlider);

      //Startup messages.
   if (ShowAiEcho) {
      aiEcho("Greetings, my name is "+cMyName+".");
      aiEcho("AI Filename='"+cFilename+"'.");
      aiEcho("MapName="+cvRandomMapName+".");
      aiEcho("Civ="+kbGetCivName(cMyCiv)+".");
      aiEcho("DifficultyLevel="+aiGetWorldDifficultyName(aiGetWorldDifficulty())+".");
      aiEcho("Personality="+aiGetPersonality()+"."); }

   //Find someone to hate.
   if (cvPlayerToAttack <= 0)
      updatePlayerToAttack();
   else
      aiSetMostHatedPlayerID(cvPlayerToAttack);

   //Bind our age handlers.
   aiSetAgeEventHandler(cAge2, "age2Handler");
   aiSetAgeEventHandler(cAge3, "age3Handler");
   aiSetAgeEventHandler(cAge4, "age4Handler");
   aiSetAgeEventHandler(cAge5, "age5Handler");

   if (cvMaxAge <= kbGetAge())
      aiSetPauseAllAgeUpgrades(true);
   //My stuff
   initRethlAge1();
   //Setup god power handler
   aiSetGodPowerEventHandler("gpHandler");
   //Setup the resign handler
   aiSetResignEventHandler("resignHandler");

   //Set our town location.
   int TC = findUnit(cUnitTypeAbstractSettlement);
   if (TC != -1)
      kbSetTownLocation(kbUnitGetPosition(TC));

   //Economy.
   initEcon();

   earlyEcoCheat();	//Only if enabled...

   //God Powers
   initGodPowers();

   //Create bases for all of our settlements.  Ignore any that already have
   //bases set.  If we have an invalid main base, the first base we create
   //will be our main base.
   static int settlementQueryID=-1;
   if (settlementQueryID < 0)
      settlementQueryID=kbUnitQueryCreate("MySettlements");
   if (settlementQueryID > -1) {
      kbUnitQuerySetPlayerID(settlementQueryID, cMyID);
      kbUnitQuerySetUnitType(settlementQueryID, cUnitTypeAbstractSettlement);
      kbUnitQuerySetState(settlementQueryID, cUnitStateAlive);
      kbUnitQueryResetResults(settlementQueryID);
      int numberSettlements=kbUnitQueryExecute(settlementQueryID);
      for (i=0; < numberSettlements) {
         int settlementID=kbUnitQueryGetResult(settlementQueryID, i);
         //Skip this settlement if it already has a base.
         if (kbUnitGetBaseID(settlementID) >= 0)
            continue;
         vector settlementPosition=kbUnitGetPosition(settlementID);
         //Create a new base.
         int newBaseID=kbBaseCreate(cMyID, "Base"+kbBaseGetNextID(), settlementPosition, 85.0);
         if (newBaseID > -1) {
            //Figure out the front vector.
            vector baseFront=xsVectorNormalize(kbGetMapCenter()-settlementPosition);
            kbBaseSetFrontVector(cMyID, newBaseID, baseFront);
            //Military gather point.
            vector militaryGatherPoint=settlementPosition+baseFront*18.0;
            kbBaseSetMilitaryGatherPoint(cMyID, newBaseID, militaryGatherPoint);
            //Set the other flags.
            kbBaseSetMilitary(cMyID, newBaseID, true);
            kbBaseSetEconomy(cMyID, newBaseID, true);
            //Set the resource distance limit.
            kbBaseSetMaximumResourceDistance(cMyID, newBaseID, gMaximumBaseResourceDistance);
            //Add the settlement to the base.
            kbBaseAddUnit(cMyID, newBaseID, settlementID);
            kbBaseSetSettlement(cMyID, newBaseID, true);
            //Set the main-ness of the base.
            kbBaseSetMain(cMyID, newBaseID, true);
         }
      }
   }

  //Map Specific
   initMapSpecific();

      //Set the Explore Danger Threshold.
   aiSetExploreDangerThreshold(300.0);
   //Auto gather our military units.
   aiSetAutoGatherMilitaryUnits(false);

   //Set the housing rebuild bound to 4 for the first age.
   gHouseAvailablePopRebuild = 5;

   //Set the hard pop caps.
   if (aiGetGameMode() == cGameModeLightning)
      gHardEconomyPopCap = 35;
   else if (aiGetGameMode() == cGameModeDeathmatch)
      gHardEconomyPopCap = 25;   // Essentially shut off vill production until age 4.
   else {
      if (aiGetWorldDifficulty() == cDifficultyEasy)
         gHardEconomyPopCap = 20;
      else if (aiGetWorldDifficulty() == cDifficultyModerate)
         gHardEconomyPopCap = 40;
      else
         gHardEconomyPopCap = -1;
   }

   aiSetAttackResponseDistance( 60.0 );
   aiSetDefaultStance( cUnitStanceDefensive );
   if (bWallUp) {
      gBuildWalls = true;
      gBuildWallsAtMainBase = true;
   }
   if ( mapPreventsWalls() )
      gBuildWallsAtMainBase = false;
   if (aiGetGameMode() == cGameModeDeathmatch) {
      if (gWallsInDM) {
         gBuildWalls = true;
         gBuildWallsAtMainBase = true;
      }
   }
   if (cvOkToBuildWalls == false) {
      gBuildWalls = false;
      gBuildWallsAtMainBase = false;
   }

   //Decide whether or not we're doing a rush/raid.
   // Rushers will use a smaller econ to age up faster, send more waves and larger waves.
   // Boomers will use a larger econ, hit age 2 later, make smaller armies, and send zero or few waves, hitting age 3 much sooner.
   int rushCount = 0;
   if (cvRushBoomSlider > -0.5) 
      rushCount = 1;    // Rushcount acts as a bool for the moment.  Rush unless strong boomer.

   int rushSize=70;      // Total pop to use in rush armies.
   rushSize = rushSize + (cvRushBoomSlider*(rushSize*0.6)); // Increase/decrease the size up to 60% for rushing 
   if (cvOffenseDefenseSlider > 0)
      rushSize = rushSize + (cvOffenseDefenseSlider*(rushSize*0.6)); // Increase the size up to 60% for offense 

   if (aiGetWorldDifficulty() == cDifficultyModerate)
      rushSize = rushSize/2;

   if (aiGetWorldDifficulty() == cDifficultyEasy) {
      rushCount = 0;
      rushSize = 10;
   }

   if (aiGetGameMode() == cGameModeDeathmatch)
      rushCount = 0;
   if ((cvRandomMapName == "king of the hill") && (rushCount < 2))
      rushCount = 1;                               // Always rush in KotH, even on easy.
   if ((aiGetWorldDifficulty() > cDifficultyModerate) && (rushCount < 2))
      rushCount = 2;		

   if ((gBuildWallsAtMainBase) && (rushCount > 0)) { //Knock up to 40 pop slots off plan
      if (rushSize > 80) rushSize = rushSize - 40;
      else               rushSize = rushSize/2;
   }

   int numTypes = 2;
   if (rushCount > 0) {  //Finally, adjust rushSize to the per-wave number we need
      rushCount = (rushSize+20)/40;   // +20 to round to closest value
      rushSize = rushSize / rushCount;
   }

   rushSize = xsClamp( rushSize, 50, 20 );  //Give unitpicker something to do
   gRushUPID = initUnitPicker( "Rush", numTypes, -1, -1, rushSize, rushSize*1.25, 2 ); // Rush with rushSize pop slots of 3 types and 2 buildings.
   kbUnitPickSetGoalCombatEfficiencyType( gRushUPID, cUnitTypeLogicalTypeLandMilitary );
   // Set a smaller number for first wave.
   int newRushSize = 0;
   newRushSize = rushSize;
   if (rushCount >= 3)
      newRushSize = rushSize/3;
   else if (rushCount == 2)
      newRushSize = rushSize/2;
   if (newRushSize != rushSize) {
      newRushSize = xsMax(newRushSize, 20);
      kbUnitPickSetMinimumPop(gRushUPID, newRushSize);
   }
   //new stuff for new land attack rule
   gRushSize = rushSize / 3;
   //set the gRushCount in order to enable the tech rules if we only have an idle attack goal in cAge2
   if (gRushCount > 0)
      gRushCount = rushCount + 2;    //since our attack plans don't make several attempts, we increase the rush count
   else
      gRushCount = rushCount;

   //set the gFirstRushSize
   if (rushCount >= 1)
      gFirstRushSize = rushSize / 5;
   else
      gFirstRushSize = newRushSize / 3;

   if (gRushCount > 2) {
      mRusher = true;
      BeenmRusher = true;
   }

   //Create our UP.
   if (gRushUPID >= 0) {
      if (rushCount > 0) {
         //Create the attack.
         gRushGoalID = createSimpleAttackGoal("Rush Land Attack", -1, gRushUPID, rushCount+1, 1, 1, kbBaseGetMainID(cMyID), false);
         if (gRushGoalID > 0)
            aiPlanSetVariableInt( gRushGoalID, cGoalPlanUpgradeFilterType, 0, cUpgradeTypeAny );
      }
      else {
         //Create an idle attack goal that will maintain our military until the next age.
         gIdleAttackGID = createSimpleAttackGoal("Idle Force", -1, gRushUPID, -1, 1, 1, kbBaseGetMainID(cMyID), false);
         if (gIdleAttackGID >= 0) {
            aiPlanSetVariableBool(gIdleAttackGID, cGoalPlanIdleAttack, 0, true);
            aiPlanSetVariableInt(gIdleAttackGID, cGoalPlanUpgradeFilterType, 0, cUpgradeTypeHitpoints);
            //Reset the rushUPID down to 3 unit type and 1 building.
            kbUnitPickSetDesiredNumberUnitTypes(gRushUPID, numTypes, 1, true);	
         }
      }
   }


   //Create our late age attack goal.
   if (aiGetWorldDifficulty() == cDifficultyEasy)
      gLateUPID = initUnitPicker("Late", 1, -1, -1, 8, 16, gNumberBuildings - 1);
   else if (aiGetWorldDifficulty() == cDifficultyModerate) {
      int minPop=20+aiRandInt(14);
      int maxPop=minPop+16;
      if ( aiGetGameMode() != cGameModeDeathmatch )
         gLateUPID=initUnitPicker("Late", 2, -1, -1, minPop, maxPop, gNumberBuildings);   // Attack with at least 20-33 pop slots, no more than 36-49.
      else
         gLateUPID=initUnitPicker("Late", 2, -1, -1, minPop, maxPop, gNumberBuildings+2);   // Attack with at least 20-33 pop slots, no more than 36-49.
   }
   else {
      minPop=40+aiRandInt(20);
      maxPop=70;
      if (aiGetWorldDifficulty() > cDifficultyHard)
         maxPop = 90;
      if (aiGetGameMode() != cGameModeDeathmatch)
         gLateUPID=initUnitPicker("Late", 3, -1, -1, minPop, maxPop, gNumberBuildings);    // Min: 40-59, max 70 pop slots
      else
         gLateUPID=initUnitPicker("Late", 3, -1, -1, minPop, maxPop, gNumberBuildings+2);    // Min: 40-59, max 70 pop slots
   }

   int lateAttackAge = 2;

   if (gLateUPID >= 0) {
      if (aiGetGameMode() == cGameModeDeathmatch)
         lateAttackAge = 3;
      gLandAttackGoalID = createSimpleAttackGoal("Main Land Attack", -1, gLateUPID, -1, lateAttackAge, -1, kbBaseGetMainID(cMyID), true);
      if (gLandAttackGoalID >= 0) {
         //If this is easy, this is an idle attack.
         if (aiGetWorldDifficulty() == cDifficultyEasy)
            aiPlanSetVariableBool(gLandAttackGoalID, cGoalPlanIdleAttack, 0, true);
         aiPlanSetVariableInt(gLandAttackGoalID, cGoalPlanUpgradeFilterType, 0, cUpgradeTypeHitpoints);
      }
   }

   //Create our econ goal (which is really just to store stuff together).
   gGatherGoalPlanID = aiPlanCreate("GatherGoals", cPlanGatherGoal);
   if (gGatherGoalPlanID >= 0) {
      //Overall percentages.
      aiPlanSetDesiredPriority(gGatherGoalPlanID, 90);	
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanScriptRPGPct, 0, 1.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanCostRPGPct, 0, 1.0);
      aiPlanSetNumberVariableValues(gGatherGoalPlanID, cGatherGoalPlanGathererPct, 4, true);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceGold, 0.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceWood, 0.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFood, 1.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanGathererPct, cResourceFavor, 0.0);

      //Standard RB setup.
      aiPlanSetNumberVariableValues(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, 5, true);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeHunt, 0);
      if (aiGetGameMode() == cGameModeDeathmatch) 
         aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeEasy, 0);
      else
         aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeEasy, 1);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeHuntAggressive, 0);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeFarm, 0);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeFish, 0);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumWoodPlans, 0, 0);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumGoldPlans, 0, 0);
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFavorPlans, 0, 0);
      //Hunt on Erebus and River Styx.
      if ((cvRandomMapName == "erebus") || (cvRandomMapName == "river styx")) {
         aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanNumFoodPlans, cAIResourceSubTypeEasy, 0);
         aiSetMinNumberNeedForGatheringAggressvies(1);
      }		 		
      //Cost weights.
      aiPlanSetNumberVariableValues(gGatherGoalPlanID, cGatherGoalPlanResourceCostWeight, 4, true);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanResourceCostWeight, cResourceGold, 1.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanResourceCostWeight, cResourceWood, 1.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanResourceCostWeight, cResourceFood, 1.0);
      aiPlanSetVariableFloat(gGatherGoalPlanID, cGatherGoalPlanResourceCostWeight, cResourceFavor, 1.0);
      //Set our farm limits.
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanFarmLimitPerPlan, 0, 20);  //  Up from 4
      aiPlanSetVariableInt(gGatherGoalPlanID, cGatherGoalPlanMaxFarmLimit, 0, 40);     //  Up from 24
      aiSetFarmLimit(aiPlanGetVariableInt(gGatherGoalPlanID, cGatherGoalPlanFarmLimitPerPlan, 0));
      aiPlanSetActive(gGatherGoalPlanID);
      //Do our late econ init.
      postInitEcon();
      //Lastly, update our EM.
      updateEMAllAges();
   }

   if ((aiGetGameMode() == cGameModeDeathmatch) || (aiGetGameMode() == cGameModeLightning)) {
      createSimpleBuildPlan(cUnitTypeTemple, 1, 100, false, true, cEconomyEscrowID, kbBaseGetMainID(cMyID), 5);
      if (aiGetGameMode() == cGameModeDeathmatch)
         createSimpleBuildPlan(cUnitTypeHouse, 4, 95, false, true, cEconomyEscrowID, kbBaseGetMainID(cMyID), 1);
   }
   xsEnableRule("buildInitialTemple");
   // research husbandry
   xsEnableRule("getHusbandry");
   // research hunting dogs
   xsEnableRule("getHuntingDogs");
   // age1 econ upgrades
   xsEnableRuleGroup("age1EconUpgrades");
   //xsEnableRule("fixJammedDropsiteBuildPlans");
   //enable the tacticalBuildings rule
   xsEnableRule("tacticalBuildings");
   xsEnableRule("relicUnitHandler");
   //enable the age1Progress rule
   xsEnableRule("age1Progress");

   //enable the buildHouse rule
   xsEnableRule("buildHouse");

   //enable the dockMonitor rule
   xsEnableRule("dockMonitor");

   //enable the spotAgeUpgrades rule
   xsEnableRule("spotAgeUpgrades");

   //Relics:  Always on Hard or Nightmare, 50% of the time on Moderate, Never on Easy.
   bool gatherRelics = true;
   if ((aiGetWorldDifficulty() == cDifficultyEasy) || ((aiGetWorldDifficulty() == cDifficultyModerate) && (aiRandInt(2) == 0)))
      gatherRelics = false;
   //If we're going to gather relics, do it.
   if (cvOkToGatherRelics == false)
      gatherRelics = false;
   if ( gatherRelics )
      xsEnableRule("goAndGatherRelics");

   //Enable building repair.
   if (aiGetWorldDifficulty() != cDifficultyEasy)
      xsEnableRule("repairBuildings");

   xsEnableRule("defendPlanRule");
   xsEnableRule("mainBaseDefPlan1");
   xsEnableRule("findMySettlementsBeingBuilt");

   //update player to attack
   xsEnableRule("updatePlayerToAttack");
   //Debug( "gRushUPID="+gRushUPID+", gLateUPID="+gLateUPID );
}

//==============================================================================
void age2Handler(int age=1)
{
   gLastAgeHandled = cAge2;
   if (cvMaxAge == age)
      aiSetPauseAllAgeUpgrades(true);

   xsEnableRule( "Age2QuickStuff" );
   xsEnableRule( "monitorAttPlans");
   xsEnableRule( "monitorDefPlans");
   xsEnableRule( "baseAttackTracker");
   xsEnableRule( "otherBasesDefPlans");
   xsEnableRule( "getNextGathererUpgrade");
   xsEnableRule( "AztecTechs" );
   xsEnableRule( "AztecTechsForce" );
   xsEnableRule( "buildArmory" );
   xsEnableRule( "CaptureModeHandle" );
   xsEnableRule( "GetMythTechs" );
  //Nate: Explicitly tell UPs what to pick, otherwise they pick nothing.
   UpdateUnitPicksForAge( cAge2 );
  //End Nate.

   if ( MyCiv == cCivQuetz )
      xsEnableRule( "QuetzTechs" );
   //Always a few Warrior Priests.
   int pid = aiPlanGetIDByTypeAndVariableType(cPlanTrain, cTrainPlanUnitType, cUnitTypeWarPriest);
   if ( pid < 0 )
      pid = createSimpleMaintainPlan( cUnitTypeWarPriest, 2, false, -1 );
   aiPlanSetVariableInt( pid, cTrainPlanNumberToMaintain, 0, 2 );
   aiPlanSetDesiredPriority( pid, 92 );
   //A Runner for scouting.
   pid = aiPlanGetIDByTypeAndVariableType(cPlanTrain, cTrainPlanUnitType, cUnitTypeRunnerAztec);
   if ( pid < 0 )
      pid = createSimpleMaintainPlan( cUnitTypeRunnerAztec, 1, false, -1 );
   aiPlanSetVariableInt( pid, cTrainPlanNumberToMaintain, 0, 1 );
   aiPlanSetDesiredPriority( pid, 70 );

   ExplorerUpdater( cAge2 );
   createSimpleResearchPlan( cTechWatchTower, -1, cMilitaryEscrowID, 80, false );
   createSimpleResearchPlan( cTechBuyVillagers, cUnitTypeMarketMeso, cEconomyEscrowID, 60, false );

   //activate ObeliskClearingPlan if there is an Egyptian enemy,
   //enable the hesperides rule if there's an Oranos or Gaia player
   bool hesperidesPower = false;
   bool UWGate = false;
   for(playerID=1; < cNumberPlayers) {
      if ( (get_civ(playerID) == cCivOuranos) || (get_civ(playerID) == cCivGaia) ) {
         hesperidesPower = true;
         continue;
      }
      if ( kbIsPlayerAlly(playerID) )
         continue;
      if ( (get_civ(playerID) == cCivZeus) || (get_civ(playerID) == cCivHades) )
         UWGate = true;
   }

   if (hesperidesPower)
      xsEnableRule("hesperides");
   if (UWGate)
      xsEnableRule("IHateUnderworldPassages");		

   //Econ.
   econAge2Handler(age);
   //Progress.
   progressAge2Handler(age);
   //GP.
   gpAge2Handler(age);
   //Naval
   navalAge2Handler(age);

   //Set the housing rebuild bound.
   gHouseAvailablePopRebuild=18;

   //Switch the EM rule.
   updateEMAllAges();	// Make it run right now
   xsEnableRule("getWatchTower");
   xsEnableRule("getCrenellations");
   xsEnableRule("getSignalFires");
   xsEnableRule("buildMBTower");

  //Maintain a water transport, if this is a transport map.
   if ((gTransportMap) && (gMaintainWaterXPortPlanID < 0) || (cvRandomMapName == "king of the hill") && (KoTHWaterVersion)) {
      gMaintainWaterXPortPlanID = createSimpleMaintainPlan(cUnitTypeTransportShipGreek, 0, true, -1);
      aiPlanSetDesiredPriority(gMaintainWaterXPortPlanID, 95);
   }

  //Init our myth unit rule.
   xsEnableRule("trainMythUnit");
   xsEnableRule("createRaidingParty");
   xsEnableRule("attackEnemySettlement");
   xsEnableRule("createLandAttack");

  //variables for our buildplans
   vector location = kbBaseGetLocation(cMyID, kbBaseGetMainID(cMyID));
   vector origLocation = location;
   vector frontVector = kbBaseGetFrontVector(cMyID, kbBaseGetMainID(cMyID));

   float fx = xsVectorGetX(frontVector);
   float fz = xsVectorGetZ(frontVector);
   float fxOrig = fx;
   float fzOrig = fz;
   vector backVector = kbBaseGetBackVector(cMyID, kbBaseGetMainID(cMyID));
   float bx = xsVectorGetX(backVector);
   float bz = xsVectorGetZ(backVector);
   float bxOrig = bx;
   float bzOrig = bz;

   xsEnableRule("buildAztecBarracks");
   xsEnableRule("buildAztecTianguis");

   //Build walls if we should.
   if (gBuildWalls) {
      if (gBuildWallsAtMainBase) {
         xsEnableRule("mainBaseAreaWallTeam1");
         xsEnableRule("DestroyUnnecessaryDropsites");
      }
      xsEnableRule("otherBaseRingWallTeam1");
      //start up the wall upgrades.
      xsEnableRule("WallManager");
      //enable the rule to fix unfinished walls
      xsEnableRule("fixUnfinishedWalls");
      //enable the rule to destroy unnecessary dropsites near our mainbase
      xsEnableRule("DestroyUnnecessaryDropsites");
   }
   //build buildings at other bases
   xsEnableRule("buildBuildingsAtOtherBase");
   //build towers at other bases
   xsEnableRule("buildTowerAtOtherBase");	
}

//==============================================================================
void age3Handler(int age=2)
{
   gLastAgeHandled = cAge3;
   if (cvMaxAge == age)
      aiSetPauseAllAgeUpgrades(true);

  //Nate: Update unitpicks.
   UpdateUnitPicksForAge( cAge3 );
   ExplorerUpdater( cAge3 );
   createSimpleResearchPlan( cTechBuyVillagers2, cUnitTypeMarketMeso, cEconomyEscrowID, 60, false );
   xsEnableRule( "CheatExchange" );

   //Econ.
   econAge3Handler(age);
   //Progress.
   progressAge3Handler(age);
   //GP.
   gpAge3Handler(age);
   //Naval
   navalAge3Handler(age);

   //kill the rush goals
   if (gRushGoalID != -1)
      aiPlanDestroy(gRushGoalID);
   if (gIdleAttackGID != -1)
      aiPlanDestroy(gIdleAttackGID);

   //build as many fortresses as possible 
   xsEnableRule("buildFortress");
   xsEnableRule("getBoilingOil");

   if ((aiGetGameMode() != cGameModeConquest) && (aiGetGameMode() != cGameModeDeathmatch))
      xsEnableRule("watchForFirstWonderStart");

   //Switch the EM rule.
   updateEMAllAges();

   //Up the number of water transports to maintain.
   if (gMaintainWaterXPortPlanID >= 0)
      aiPlanSetVariableInt(gMaintainWaterXPortPlanID, cTrainPlanNumberToMaintain, 0, 2);

   xsEnableRule("buildAztecQuarters");

   // Build a fortress/palace/whatever...or 4 in DM
   const int buildingType = MyFortress;
   const int numBuilders = 2;

   int strongBuildPlanID=aiPlanCreate("Build Strong Building ", cPlanBuild);
   if (strongBuildPlanID >= 0) {
      vector frontVector = kbBaseGetFrontVector(cMyID, kbBaseGetMainID(cMyID));
      float x = xsVectorGetX(frontVector);
      float z = xsVectorGetZ(frontVector);
      x = x * 15;
      z = z * 15;
      frontVector = xsVectorSetX(frontVector, x);
      frontVector = xsVectorSetZ(frontVector, z);
      frontVector = xsVectorSetY(frontVector, 0.0);
      vector location = kbBaseGetLocation(cMyID, kbBaseGetMainID(cMyID));
      location = location + frontVector;
      aiPlanSetInitialPosition(strongBuildPlanID, location);
      aiPlanSetVariableBool(strongBuildPlanID, cBuildPlanInfluenceAtBuilderPosition, 0, false);
      aiPlanSetVariableFloat(strongBuildPlanID, cBuildPlanRandomBPValue, 0, 0.99);
      aiPlanSetVariableVector(strongBuildPlanID, cBuildPlanInfluencePosition, 0, location);
      aiPlanSetVariableFloat(strongBuildPlanID, cBuildPlanInfluencePositionDistance, 0, 40.0);
      aiPlanSetVariableFloat(strongBuildPlanID, cBuildPlanInfluencePositionValue, 0, 10000.0);
      aiPlanSetVariableInt(strongBuildPlanID, cBuildPlanBuildingTypeID, 0, buildingType);
      aiPlanSetDesiredPriority(strongBuildPlanID, 100);
      aiPlanAddUnitType(strongBuildPlanID, cBuilderType, numBuilders, numBuilders, numBuilders);
      aiPlanSetEscrowID(strongBuildPlanID, cMilitaryEscrowID);
      aiPlanSetBaseID(strongBuildPlanID, kbBaseGetMainID(cMyID));
      aiPlanSetActive(strongBuildPlanID);
   }

   xsEnableRule("tradeWithCaravans");
   tradeWithCaravans();
   xsEnableRule("maintainTradeUnits");
   xsEnableRule("sendIdleTradeUnitsToRandomBase");	

   //get tax collectors and ambassadors
   xsEnableRule("getTaxCollectors");  
   if ( IhaveAllies )
      xsEnableRule("getAmbassadors"); //AI is indeed taxed too!
   //enable the maintainSiegeUnits rule
   xsEnableRule("maintainSiegeUnits");
   //research masons
   xsEnableRule("getMasons");
   //research heroic fleet on transport maps
   if (gTransportMap)
      xsEnableRule("getHeroicFleet");
}

rule CheatExchange minInterval 40 inactive
{
   int food = kbResourceGet( cResourceFood );
   if ( food > 800 ) {
      if ( food >= (4*kbResourceGet(cResourceGold)) ) {
         int amount = xsMin( 1000, food );
         aiResourceCheat( cMyID, cResourceFood, (0 - amount) );
         aiResourceCheat( cMyID, cResourceGold, amount );
         return;
      }
   }
   int wood = kbResourceGet( cResourceWood );
   if ( wood > 800 ) {
      if ( wood >= (4*kbResourceGet(cResourceGold)) ) {
         amount = xsMin( 1000, wood );
         aiResourceCheat( cMyID, cResourceWood, (0 - amount) );
         aiResourceCheat( cMyID, cResourceGold, amount );
      }
   }
}

rule CaptureModeHandle minInterval 22 inactive
{
   if ( (kbUnitCount(cMyID, cUnitTypeAbstractTemple, cUnitStateAlive) <= 0) &&
        (kbUnitCount(cMyID, cUnitTypeTlalocan, cUnitStateAlive) <= 0) )
      return;
   const int HumanUnitReq = 16;
  //Enter/exit capture mode when it makes sense.
   int desired = xsMin( 75, (kbGetAge()+1)*18 );
   int prisoners = kbUnitCount(cMyID, cUnitTypePrisonerSmall, cUnitStateAliveOrBuilding) +
                   (kbUnitCount(cMyID, cUnitTypePrisonerBig, cUnitStateAliveOrBuilding) * 2);
   int current = kbResourceGet(cResourceFavor) + (rotgPrisonerFavor() * prisoners);
   if ( current < desired ) {
      if ( kbProtoUnitAvailable(cUnitTypeCaptureEnter) ) {  //Can enter capture mode.
         if ( kbUnitCount(cMyID, cUnitTypeHumanSoldier, cUnitStateAlive) >= HumanUnitReq )
            rotgCaptureModeSet( true );
      }
      else if ( kbProtoUnitAvailable(cUnitTypeCaptureExit) ) {
         if ( kbUnitCount(cMyID, cUnitTypeHumanSoldier, cUnitStateAlive) < HumanUnitReq )
            rotgCaptureModeSet( false );
      }
   }
   else if ( kbProtoUnitAvailable(cUnitTypeCaptureExit) )
      rotgCaptureModeSet( false );
}

//==============================================================================
void age4Handler(int age=3)
{
   if (cvMaxAge == age)
      aiSetPauseAllAgeUpgrades(true);

   gLastAgeHandled = cAge4;
   econAge4Handler(age);
   progressAge4Handler(age);
   gpAge4Handler(age);

  //Nate: Update unitpicks.
   UpdateUnitPicksForAge( cAge4 );
   ExplorerUpdater( cAge4 );
  //End Nate.

   if ( (aiGetGameMode() != cGameModeConquest) && (aiGetGameMode() != cGameModeDeathmatch) )
      xsEnableRule("makeWonder");      // Make a wonder if you have spare resources
   //Switch the EM rule.
   updateEMAllAges();
   xsEnableRule("getOmniscience");
   xsEnableRule("getCoinage");

   //If we're in deathmatch, no more hard pop cap.
   if (aiGetGameMode() == cGameModeDeathmatch) {
      gHardEconomyPopCap=-1;
      kbEscrowAllocateCurrentResources();
   }

   //if we are on a land map or playing conquest make titan
   if ((gTransportMap == false) || (aiGetGameMode() == cGameModeConquest))
      xsEnableRule("getSecretsOfTheTitan");
   xsEnableRule("getCraftsman");
}

//==============================================================================
void age5Handler(int age=4)
{
   gLastAgeHandled = cAge5;
   xsEnableRule("rPlaceTitanGate");
   xsEnableRule("randomUpgrader");
   UpdateUnitPicksForAge( cAge5 );

   kbEscrowSetCap( cEconomyEscrowID, cResourceFood, 300.0);    
   kbEscrowSetCap( cEconomyEscrowID, cResourceWood, 300.0);    
   kbEscrowSetCap( cEconomyEscrowID, cResourceGold, 300.0);
   kbEscrowSetCap( cEconomyEscrowID, cResourceFavor, 30.0);
}

//==============================================================================
rule ShouldIResign
minInterval 10 //starts in cAge1
active
{
   if (cvOkToResign == false) {
      xsDisableSelf();     // Must be re-enabled if cvOkToResign is set true.
      return;     
   }

   //Don't resign too soon.
   if (xsGetTime() < 10*60*1000)
      return;

   //Don't resign if you're teamed with a human.
   static bool checkTeamedWithHuman=true;
   if (checkTeamedWithHuman) {
      for (i=1; < cNumberPlayers) {
         if (i == cMyID)
            continue;
         //Skip if not human.
         if (kbIsPlayerHuman(i) == false)
            continue;
         //If this is a mutually allied human, go away.
         if (kbIsPlayerMutualAlly(i)) {
            xsDisableSelf();
            return;
         }
      }
      //Don't check again.
      checkTeamedWithHuman=false;
   }

   int numSettlements=kbUnitCount(cMyID, cUnitTypeAbstractSettlement, cUnitStateAliveOrBuilding);
   //If on easy, don't only resign if you have no settlements.
   if (aiGetWorldDifficulty() == cDifficultyEasy) {
      if (numSettlements <= 0) {
         aiAttemptResign(cAICommPromptResignQuestion);
         xsDisableSelf();
      }
      return;
   }

   //Don't resign if we have over 30 active pop slots.
   if (kbGetPop() >= 30)
      return;

   //Don't quit if we have at least one settlement.
   if ((numSettlements > 1) || (numSettlements > 0) && (IhaveAllies))
      return;

   //Don't resign if we still have villagers and teamed up.
   int numAliveVils=kbUnitCount(cMyID, cUnitTypeAbstractVillager, cUnitStateAlive);
   if ((numAliveVils > 0) && (IhaveAllies))
      return;


   int builderUnitID = cBuilderType;
   int numBuilders = kbUnitCount(cMyID, cBuilderType, cUnitStateAliveOrBuilding);   

   if ((numSettlements <= 0) && (numBuilders <= 10)) {
      if (kbCanAffordUnit(cUnitTypeSettlementLevel1, cEconomyEscrowID) == false) {
         aiAttemptResign(cAICommPromptResignQuestion);
         xsDisableSelf();
         return;
      }
   }
   //If we don't have any builders, we're not Norse, and we cannot afford anymore, try to resign.
   if ( numBuilders <= 0 ) {
      if (kbCanAffordUnit(builderUnitID, cEconomyEscrowID) == false) {
         aiAttemptResign(cAICommPromptResignQuestion);
         xsDisableSelf();
         return;
      }
   }


   //3. if all of my teammates have left the game.
   int activeEnemies=0;
   int activeTeammates=0;
   int deadTeammates=0;
   float currentEnemyMilPop=0.0;
   float currentMilPop=0.0;
   for (i=1; < cNumberPlayers) {
      if (i == cMyID) {
         currentMilPop=currentMilPop+kbUnitCount(i, cUnitTypeMilitary, cUnitStateAlive);
         continue;
      }

      if (kbIsPlayerAlly(i) == false) {
         //Increment the active number of enemies there currently are.
         if (kbIsPlayerResigned(i) == false) {
            activeEnemies=activeEnemies+1;
            currentEnemyMilPop=currentEnemyMilPop+kbUnitCount(i, cUnitTypeMilitary, cUnitStateAlive);
         }
         continue;
      }

      //If I still have an active teammate, don't resign.
      if (kbIsPlayerResigned(i))
         deadTeammates=deadTeammates+1;
      else
         activeTeammates=activeTeammates+1;
   }

   //3a. if at least one player from my team has left the game and I am the only player left on my team, 
   //    and the other team(s) have 2 or more players in the game.
   if ((activeEnemies >= 2) && (activeTeammates <= 0) && (deadTeammates>0)) {
      aiAttemptResign(cAICommPromptResignQuestion);
      xsDisableSelf();
      return;
   }

   //4. my mil pop is low and the enemy's mil pop is high,
   //Don't do this eval until 4th age and at least 30 min. into the game.
   if ((xsGetTime() < 30*60*1000) || (kbGetAge() < cAge4))
      return;

   static int enemyMilPopTotal=0;
   static int myMilPopTotal=0;
   static int count=0;
   count++;
   enemyMilPopTotal = enemyMilPopTotal + currentEnemyMilPop;
   myMilPopTotal = myMilPopTotal + currentMilPop;
   if (count >= 10.0) {
      if ( (enemyMilPopTotal > (7.0*myMilPopTotal)) || (myMilPopTotal <= count) ) {
         aiAttemptResign(cAICommPromptResignQuestion);
         xsDisableSelf();
         return;
      }
      count = 0;
      enemyMilPopTotal = 0;
      myMilPopTotal = 0;
   }
}

//==============================================================================
void resignHandler(int result =-1)
{
   if (result != 0)
      aiResign();
}

//==============================================================================
rule findFish   //We don't know if this is a water map...if you see fish, it is.
minInterval 9 //starts in cAge1
active
{
   if ((cRandomMapName == "highland") || (cRandomMapName == "nomad") ||(NoFishing) || (cvMapSubType == VINLANDSAGAMAP)) {
      xsDisableSelf();
      return;
   }
   if (kbUnitCount(0, cUnitTypeFish) > 0) {
      gWaterMap = true;
      //Tell the AI what kind of map we are on.
      aiSetWaterMap( gWaterMap );
      xsEnableRule( "fishing" );		
   }
   xsDisableSelf();
}

//==============================================================================
rule watchForFirstWonderStart   //Look for any wonder being built.  If found, activate
//the high-speed rule that watches for completion
minInterval 73 //starts in cAge3    // Hopefully nobody will build one faster than this
inactive
{
   xsSetRuleMinIntervalSelf(73);
   int AnyWonder = findUnitByRel(cUnitTypeWonder, cUnitStateAliveOrBuilding, -1, cPlayerRelationAny);
   if ( AnyWonder > 0 ) {
      xsDisableSelf();
      xsEnableRule("watchForFirstWonderDone");
   }
}

//==============================================================================
rule watchForFirstWonderDone    //See who makes the first wonder, note its ID, make a defend
//plan to kill it, kill defend plan when it's gone
inactive
minInterval 2 //starts in cAge3 activated in watchForFirstWonderStart
{
   static int wonderID = -1;
   static vector wonderLocation = cInvalidVector;
   int Owner = -1;
   int EnemyWonder = findUnitByRel(cUnitTypeWonder, cUnitStateAlive, -1, cPlayerRelationEnemy);
   int AllyWonder = findUnitByRel(cUnitTypeWonder, cUnitStateAlive, -1, cPlayerRelationAlly);

   static int wonderDefendPlanStartTime = -1;

   if (wonderID < 0) {
      int myWonder = findUnit(cUnitTypeWonder, cUnitStateAlive);
      if ( myWonder >= 0 )
         return;
      if ( EnemyWonder >= 0 ) {
         // Create highest-priority defend plan to go kill it
         Owner = kbUnitGetOwner(EnemyWonder);
         wonderID = EnemyWonder;
         wonderLocation = kbUnitGetPosition(wonderID);
         // Making an attack plan instead, they do a better job of transporting and ignoring some targets en route.
         gEnemyWonderDefendPlan = createDefOrAttackPlan("Enemy wonder attack plan", false, -1, 200, cInvalidVector, kbBaseGetMainID(cMyID), 80, false);
         aiPlanSetVariableInt(gEnemyWonderDefendPlan, cAttackPlanPlayerID, 0, Owner);
         // Specify other continent so that armies will transport
         aiPlanSetNumberVariableValues( gEnemyWonderDefendPlan, cAttackPlanTargetAreaGroups,  1, true);  
         aiPlanSetVariableInt(gEnemyWonderDefendPlan, cAttackPlanTargetAreaGroups, 0, kbAreaGroupGetIDByPosition(kbUnitGetPosition(wonderID)));
         aiPlanSetVariableVector(gEnemyWonderDefendPlan, cAttackPlanGatherPoint, 0, kbBaseGetLocation(cMyID, kbBaseGetMainID(cMyID)));
         aiPlanAddUnitType(gEnemyWonderDefendPlan, cUnitTypeLogicalTypeLandMilitary, 200, 200, 200);
         aiPlanSetInitialPosition(gEnemyWonderDefendPlan, kbBaseGetLocation(cMyID, kbBaseGetMainID(cMyID)));
         aiPlanSetVariableBool(gEnemyWonderDefendPlan, cAttackPlanAutoUseGPs, 0, true);
         aiPlanSetVariableBool(gEnemyWonderDefendPlan, cAttackPlanMoveAttack, 0, true);
         aiPlanSetVariableInt(gEnemyWonderDefendPlan, cAttackPlanSpecificTargetID, 0, wonderID);
         aiPlanSetActive(gEnemyWonderDefendPlan);
         wonderDefendPlanStartTime = xsGetTime();
      }
      else if ( AllyWonder >= 0 ) {
         // Create highest-priority defend plan to go protect it
         Owner = kbUnitGetOwner( AllyWonder );
         wonderID = AllyWonder;
         wonderLocation = kbUnitGetPosition( AllyWonder );
         if ( kbAreaGroupGetIDByPosition(wonderLocation) == kbAreaGroupGetIDByPosition(kbBaseGetLocation(cMyID, kbBaseGetMainID(cMyID))) )
         {  // It's on my continent, go help
            gEnemyWonderDefendPlan = createDefOrAttackPlan("Ally Wonder Defend Plan", true, 60, 40, wonderLocation, -1, 96, false);
            if (gEnemyWonderDefendPlan >= 0) {
               aiPlanAddUnitType(gEnemyWonderDefendPlan, cUnitTypeMilitary, 200, 200, 200);    // All mil units
               wonderDefendPlanStartTime = xsGetTime();
               aiPlanSetActive(gEnemyWonderDefendPlan);
               xsEnableRule("BunkerUpThatWonder");
            }
         }
      }
   }
   else {
      if ( aiPlanGetState(gEnemyWonderDefendPlan) == cPlanStateAttack )
         aiPlanSetNoMoreUnits(gEnemyWonderDefendPlan, false);  // Make sure the enemy wonder 'defend' plan stays open
      if ( kbUnitGetCurrentHitpoints(wonderID) <= 0.0 ) {
         aiPlanDestroy(gEnemyWonderDefendPlan);
         gEnemyWonderDefendPlan = -1;
         wonderID = -1;
         xsSetRuleMinInterval("watchForFirstWonderStart", 2);
         xsEnableRule("watchForFirstWonderStart");
         xsDisableSelf();
      }
   }
}

//==============================================================================
rule watchForWonder  // See if my wonder has been placed.  If so, go build it.
minInterval 5 //starts in cAge4, activated in make wonder
inactive
{
   if (kbUnitCount(cMyID, cUnitTypeWonder, cUnitStateAliveOrBuilding) < 1) {
      int WonderPlan = findPlanByString("Wonder Build", cPlanBuild);
      if ( WonderPlan == -1 ) {
         xsEnableRule("makeWonder"); // plan died? reboot.
         xsDisableSelf();
      }
      return;
   }

   int wonderID = findUnit(cUnitTypeWonder, cUnitStateAlive);
   if (wonderID < 0)
      return;
   xsEnableRule("watchWonderLost");    // Kill the defend plan if the wonder is destroyed.
   vector wonderLocation = kbUnitGetPosition(wonderID);

   // Make the defend plan
   gWonderDefendPlan = createDefOrAttackPlan("Wonder Defend Plan", true, 60, 40, wonderLocation, -1, 95, false);
   if (gWonderDefendPlan >= 0) {
      aiPlanAddUnitType(gWonderDefendPlan, cUnitTypeMilitary, 20, 200, 200);    // most mil units
      aiPlanSetActive(gWonderDefendPlan);  
   }
   xsDisableSelf();
}

//==============================================================================
rule watchWonderLost    // Kill the uber-defend plan if wonder falls
minInterval 7 //starts in cAge4, activated in watchForWonder
inactive
{
   if (kbUnitCount(cMyID, cUnitTypeWonder, cUnitStateAliveOrBuilding) > 0 )
      return;
   aiPlanDestroy( gWonderDefendPlan );
   gWonderDefendPlan = -1;
   xsEnableRule("makeWonder");      // Try again if we get a chance
   xsDisableSelf();
}

//==============================================================================
rule goAndGatherRelics
inactive
minInterval 101 //starts in cAge1
{
   if ( kbUnitCount(cMyID, cUnitTypeAbstractTemple, cUnitStateAlive) <= 0 )
      return;
   if ( (kbUnitCount(cMyID, cUnitTypeWarPriest, cUnitStateAlive) < 1) && (xsGetTime() < 5*60*1000) )
      return;

   static int gatherRelicStartTime = -1;
   if ( aiPlanGetNumber(cPlanGatherRelic, -1, true) > 0 ) {
      if ( xsGetTime() > (gatherRelicStartTime + 10*60*1000) ) {
         aiPlanDestroy(gRelicGatherPlanID);
         gRelicGatherPlanID = -1;
         gatherRelicStartTime = -1;
         xsSetRuleMinIntervalSelf(101);
      }
      return;
   }
   gRelicGatherPlanID = aiPlanCreate("RelicGather", cPlanGatherRelic);
   aiPlanAddUnitType(gRelicGatherPlanID, gGatherRelicType, 1, 1, 1);
   aiPlanSetVariableInt(gRelicGatherPlanID, cGatherRelicPlanTargetTypeID, 0, cUnitTypeRelic);
   aiPlanSetVariableInt(gRelicGatherPlanID, cGatherRelicPlanDropsiteTypeID, 0, cUnitTypeTemple);
   aiPlanSetBaseID(gRelicGatherPlanID, kbBaseGetMainID(cMyID));
   aiPlanSetDesiredPriority(gRelicGatherPlanID, 100);
   aiPlanSetActive(gRelicGatherPlanID);
   xsSetRuleMinIntervalSelf( 307 );
   gatherRelicStartTime = xsGetTime();
}

//==============================================================================
rule relicUnitHandler
minInterval 127 //starts in cAge1
inactive
{
   int numPegasus = kbUnitCount(cMyID, cUnitTypePegasus, cUnitStateAlive);   
   if (numPegasus > 0) {
      int exploreID = aiPlanCreate("RelicPegasus_Exp", cPlanExplore);
      aiPlanAddUnitType(exploreID, cUnitTypePegasus, 1, 1, 1);
      aiPlanSetVariableBool(exploreID, cExplorePlanDoLoops, 0, false);
      aiPlanSetDesiredPriority(exploreID, 98);
      aiPlanSetActive(exploreID);
      xsDisableSelf();
   }
}

//==============================================================================
rule spotAgeUpgrades    //detect age upgrades given as starting conditions or via triggers
minInterval 30 //starts in cAge1
inactive
{
   if ( gLastAgeHandled < kbGetAge() ) {  //If my current age is higher than the last upgrade I remember...do the handler
      if (gLastAgeHandled == cAge1)
         age2Handler();
      else if (gLastAgeHandled == cAge2)
         age3Handler();
      else if (gLastAgeHandled == cAge3)
         age4Handler();
      else if (gLastAgeHandled == cAge4)
         age5Handler();
   }
   if ( gLastAgeHandled >= cAge5 )
      xsDisableSelf();
}

rule GetMythTechs minInterval 90 inactive
{
   int doneTechs = 0;
   int reqTechs = (cAge4 * 3);  //Minor dieties have 3 techs each by def.
   if ( kbGetTechStatus(cTechAge2Huehue) >= cTechStatusActive )  //These gods have 4 techs.
      reqTechs++;
   if ( kbGetTechStatus(cTechAge3Xipe) >= cTechStatusActive )
      reqTechs++;
   if ( kbGetTechStatus(cTechAge4Tlaloc) >= cTechStatusActive )
      reqTechs++;

   static bool alternate = true;
   for(techid=cTechSouthernStars; <= cTechSpiritOfPanal)
   {
      switch( kbGetTechStatus(techid) ) {
      case cTechStatusActive:    { doneTechs++; continue; }
      case cTechStatusAvailable: { break; }
      default:                   { break; } }
      int pid = aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techid);
      if ( (pid >= 0) && (alternate) ) {
         aiPlanDestroy( pid );
         pid = createSimpleResearchPlan( techid, -1, cEconomyEscrowID, 55, false );
      }
      aiTaskUnitResearch( findUnit(cUnitTypeBuilding), techid );
   }

   alternate = ( alternate == false );
   if ( doneTechs >= reqTechs )
      xsDisableSelf();
}

//==============================================================================
// god power handler
//==============================================================================
void gpHandler(int powerProtoID=-1)
{
  //If the power is TitanGate, then we need to launch the repair plan to build it..
   if (powerProtoID == cPowerTitanGate)
      xsEnableRule( "repairTitanGate" );
   if ( CanIChat == false )
      return;
   if (powerProtoID == cPowerSpy)
      return;

   int type = cAICommPromptGenericGodPower;
   if ((powerProtoID == cPowerEarthquake) || (powerProtoID == cPowerCurse) || (powerProtoID == cPowerFlamingWeapons) || 
       (powerProtoID == cPowerForestFire) || (powerProtoID == cPowerFrost) || (powerProtoID == cPowerLightningStorm) ||
       (powerProtoID == cPowerLocustSwarm) || (powerProtoID == cPowerMeteor) || (powerProtoID == cPowerAncestors) ||
       (powerProtoID == cPowerFimbulwinter) || (powerProtoID == cPowerTornado) || (powerProtoID == cPowerBolt) )
      type = cAICommPromptOffensiveGodPower;
   else if ( (powerProtoID == cPowerAbundance) || (powerProtoID == cPowerTlalocan) || (powerProtoID == cPowerRain) ||
             (powerProtoID == cPowerLure) || (powerProtoID == cPowerDwarvenMine) || (powerProtoID == cPowerGreatHunt) )
      type = cAICommPromptEconomicGodPower;

   for(i=1; < cNumberPlayers) {
      if (i == cMyID)
         continue;
      if (kbIsPlayerAlly(i))
         continue;
      aiCommsSendStatement(i, type, -1);
   }
}

//===============================================================================================================================
rule _CivTimeOut minInterval 2 maxInterval 2 active {
   xsDisableSelf();
   if ( kbGetTechStatus(cTechAge1Aztec) >= cTechStatusActive )
      return;
   xsDisableRule( "_CivAssigner" );
   xsDisableRule( "_AztecDelay" );
   aiErrorMessage( "Player#"+cMyID+" must aquire Aztec god tech in 2-sec of less." );
   aiResign();
}
rule _AztecDelay highFrequency active {  //MinorGodChoosing, etc. must be delayed until script has activated all right the techs, etc.
   if ( kbGetTechStatus(cTechAge1Aztec) < cTechStatusActive )
      return;
   xsDisableSelf();
   initAztec();
   kbTechTreeAddMinorGodPref( gAge2MinorGod );
   kbTechTreeAddMinorGodPref( gAge3MinorGod );
   kbTechTreeAddMinorGodPref( gAge4MinorGod );
   aiEcho( "Minor Gods: "+kbGetTechName(gAge2MinorGod)+", "+kbGetTechName(gAge3MinorGod)+", "+kbGetTechName(gAge4MinorGod) );
}
rule _CivAssigner highFrequency active
{
   if ( kbGetCiv() != cCivShennong )  //DO NOT use cMyCiv/MyCiv, because we're waiting on rotg_script to change our civ.
      return;
   if ( rotgLaunchCivSelection(MyCiv, MyCulture) == false )
      return;
   xsDisableSelf();
   if ( cvDelayStart )
      xsDisableRule("age1Progress");
   else
      init();
}

void main(void)
{
   rotgCivHandler();
   rotgPreLaunch( MyCulture );

   aiRandSetSeed(-1);
   kbAreaCalculate();
   kbLookAtAllUnitsOnMap();  //Cheating, but only do this once.

   preInitMap();
   persDecidePersonality();  //Set the control variables before anything else

   aiSetAllowAutoDropsites( false );  //Must build dropoffs manually >:(

  //init() and age1Progress thing handled in _CivAssigner
}

