//==============================================================================
// The Return of the Gods AI
//==============================================================================
// ROTGAIGPs.xs
// This is a modification of Georg Kalus' extension of the default aomx ai file
// by Loki_GdD
//
// This is the basic logic behind the casting of the various god powers
// Although some are rule driven, much of the complex searches and casting logic
// is handled by the C++ code.
//==============================================================================
// *****************************************************************************
// An explanation of some of the plan types, etc. in this file:
//
// aiPlanSetVariableInt(planID,  cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModel...
//   CombatDistance - This is the standard one.  The plan will get attached to an 
//   attack plan, and the attack plan performs a query, and when the number and 
//   type of enemy units you specify are within the specified distance of the 
//   attack plan's location, the god power will go off. 
//
//   CombatDistancePosition - *doesn't* get attached to an attack plan.  
//   You specify a position, and when the number and type of enemy units are within 
//   distance of that position, the power goes off.  This, for instance, could see 
//   if there are many enemy units around your town center. 
//
//   CombatDistanceSelf - this one's kind of particular.  It gets attached to an 
//   attack plan.  The query you specify in the setup determines the number and 
//   type of *friendly* units neccessary to satisfy the evaluation.  Addtionally, 
//   there must be at least 5 (currently hardcoded) enemy units within the distance 
//   value of the attack plan for it to be successful.  Then the power will go off.  
//   This is typicaly used for powers that improve friendly units, like bronze, 
//   flaming weapons, and eclipse.
// *****************************************************************************
//==============================================================================

//==============================================================================
bool setupGodPowerPlan(int planID = -1, int powerProtoID = -1)
{
   if (planID == -1)
      return( false );
   int mainBaseID = kbBaseGetMainID(cMyID);
   vector mainBaseLocation = kbBaseGetLocation(cMyID, mainBaseID);
   aiPlanSetBaseID(planID, mainBaseID);

   if ( powerProtoID == cPowerAbundance ) {
      aiPlanDestroy( planID );
      xsEnableRule( "CastAbundance" );
      return( true );
   }
   if ( powerProtoID == cPowerDecoys ) {
      aiPlanDestroy( planID );
      xsEnableRule( "CastDecoys" );
      return( true );
   }
   if ( powerProtoID == cPowerFireStrike ) {
	int queryID = kbUnitQueryCreate("FireStrikeEval");
      kbUnitQuerySetPlayerID(queryID, aiGetMostHatedPlayerID());
      kbUnitQuerySetUnitType(queryID, cUnitTypeMilitary);
      kbUnitQuerySetState(queryID, cUnitStateAlive);
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryID, 0, queryID);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryPlayerID, 0, aiGetMostHatedPlayerID());
      aiPlanSetVariableInt(planID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistance);
      aiPlanSetVariableInt(planID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelNearbyUnitType);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 0, cUnitTypeMilitary);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 1, cUnitTypeMilitary);
      aiPlanSetVariableFloat(planID, cGodPowerPlanDistance, 0, 30.0);
      aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 6);
      return( true );
   }

   if ( (powerProtoID == cPowerJaguarNight) || (powerProtoID == cPowerHandOfFire) ) {
	queryID = kbUnitQueryCreate("Age2PowerEval");
      kbUnitQuerySetPlayerID(queryID, aiGetMostHatedPlayerID());
      kbUnitQuerySetUnitType(queryID, cUnitTypeMilitary);
      kbUnitQuerySetState(queryID, cUnitStateAlive);
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryID, 0, queryID);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryPlayerID, 0, aiGetMostHatedPlayerID());
      aiPlanSetVariableInt(planID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistance);
      aiPlanSetVariableInt(planID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelNearbyUnitType);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 1, cUnitTypeMilitary);
      aiPlanSetVariableFloat(planID, cGodPowerPlanDistance, 0, 35.0);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 0, cUnitTypeMilitary);
      aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 8);
      return( true );  
   }
   if ( powerProtoID == cPowerWarriorBlessing ) {
      queryID = kbUnitQueryCreate("Age2PowerEval");
      kbUnitQuerySetPlayerID(queryID, cMyID);
      kbUnitQuerySetUnitType(queryID, cUnitTypeSpearmanAztec);
      kbUnitQuerySetState(queryID, cUnitStateAlive);
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryID, 0, queryID);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryPlayerID, 0, cMyID);
      aiPlanSetVariableInt(planID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistanceSelf);
	aiPlanSetVariableInt(planID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelNearbyUnitType);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 0, cUnitTypeMilitary);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 1, cUnitTypeSpearmanAztec);
	aiPlanSetVariableFloat(planID, cGodPowerPlanDistance, 0, 20.0);
      aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 5);
      return( true ); 
   }

   if (powerProtoID == cPowerEpidemic) {
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true);
      aiPlanSetVariableInt(planID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistance);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryPlayerID, 0, aiGetMostHatedPlayerID());
      aiPlanSetVariableInt(planID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelLocation);
      aiPlanSetVariableFloat(planID, cGodPowerPlanDistance, 0, 50.0);
      aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 8);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 0, cUnitTypeMilitary);
      aiPlanSetVariableBool(planID, cGodPowerPlanTownDefensePlan, 0, true);
      return( true );
   }
   if ( (powerProtoID == cPowerAmoxForest) || (powerProtoID == cPowerDeathsAwaken) )  {
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true ); 
      aiPlanSetVariableInt(planID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistance);
      aiPlanSetVariableInt(planID, cGodPowerPlanQueryPlayerID, 0, aiGetMostHatedPlayerID());
      aiPlanSetVariableInt(planID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelAttachedPlanLocation);
      aiPlanSetVariableFloat(planID, cGodPowerPlanDistance, 0, 40.0 );
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 0, cUnitTypeMilitary );
      aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 8 );
      aiPlanSetVariableBool(planID, cGodPowerPlanTownDefensePlan, 0, true);
      return( true );
   }

   if (powerProtoID == cPowerDeathFromAbove) {
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true); 
      aiPlanSetVariableInt(planID,  cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistancePosition);
      aiPlanSetVariableInt(planID,  cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelWorld);
      aiPlanSetVariableVector(planID, cGodPowerPlanTargetLocation, 0, mainBaseLocation);
      aiPlanSetVariableFloat(planID,  cGodPowerPlanDistance, 0, 45.0);
      aiPlanSetVariableInt(planID, cGodPowerPlanUnitTypeID, 0, cUnitTypeMilitary);
      aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 10);
      //aiPlanSetVariableBool(planID, cGodPowerPlanTownDefensePlan, 0, true);
      return( true );
   }
   if (powerProtoID == cPowerVolcanoROTG) {
      gHeavyGPTechID = cTechVolcanoROTG;
      gHeavyGPPlanID = planID;
      xsEnableRule("castHeavyGP");
      aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, false); 
      aiPlanSetVariableInt(planID,  cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelNone);
      aiPlanSetVariableInt(planID,  cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelWorld);
      return( true );  
   }
   if (powerProtoID == cPowerTlalocan)
   {
	  aiPlanSetVariableBool(planID, cGodPowerPlanAutoCast, 0, true); 
	  aiPlanSetVariableInt(planID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelNone);
	  aiPlanSetVariableInt(planID, cGodPowerPlanCount, 0, 1);
	  aiPlanSetVariableInt(planID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelTown);
	  //-- override the default building placement distance so that plenty has some room to cast it is pretty big..
	  aiPlanSetVariableFloat(planID, cGodPowerPlanBuildingPlacementDistance, 0, 20.0);
	  return (true);
   }
   return( false );
}
//==============================================================================
void initGodPowers(void)    //initialize the god power module
{
   xsEnableRule("rAge1FindGP");
}

//==============================================================================
rule rAge1FindGP
minInterval 12 //starts in cAge1
inactive
{
   int id = aiGetGodPowerTechIDForSlot(0); 
   if ( (id == -1) || (gpDelayMigration) && (kbGetAge() < cAge2) )
      return;
   gAge1GodPowerPlanID = aiPlanCreate("Age1GodPower", cPlanGodPower);
   if (gAge1GodPowerPlanID == -1) {
      //This is bad, and we most likely can never build a plan, so kill ourselves.
      xsDisableSelf();
      return;
   }
   aiPlanSetVariableInt(gAge1GodPowerPlanID,  cGodPowerPlanPowerTechID, 0, id);
   aiPlanSetDesiredPriority(gAge1GodPowerPlanID, 100);
   aiPlanSetEscrowID(gAge1GodPowerPlanID, -1);
   //Setup the god power based on the type.
   if (setupGodPowerPlan(gAge1GodPowerPlanID, aiGetGodPowerProtoIDForTechID(id)) == false) {
      aiPlanDestroy(gAge1GodPowerPlanID);
      xsDisableSelf();
      return;
   }
   if (cvOkToUseAge1GodPower)
      aiPlanSetActive(gAge1GodPowerPlanID);
   xsDisableSelf();
}

//==============================================================================
rule rAge2FindGP
minInterval 12 //starts in cAge2
inactive
{
   //Figure out the age2 god power and create the plan.
   int id = aiGetGodPowerTechIDForSlot(1); 
   if (id == -1)
      return;
  //Create the plan.
   gAge2GodPowerPlanID=aiPlanCreate("Age2GodPower", cPlanGodPower);
   if (gAge2GodPowerPlanID == -1) {
     //This is bad, and we most likely can never build a plan, so kill ourselves.
      xsDisableSelf();
      return;
   }
   aiPlanSetVariableInt(gAge2GodPowerPlanID, cGodPowerPlanPowerTechID, 0, id);
   aiPlanSetDesiredPriority(gAge2GodPowerPlanID, 100);
   aiPlanSetEscrowID(gAge2GodPowerPlanID, -1);
  //Setup the god power based on the type.
   if (setupGodPowerPlan(gAge2GodPowerPlanID, aiGetGodPowerProtoIDForTechID(id)) == false) {
      aiPlanDestroy(gAge2GodPowerPlanID);
      xsDisableSelf();
      return;
   }
   if (cvOkToUseAge2GodPower)
      aiPlanSetActive(gAge2GodPowerPlanID);
   xsDisableSelf();
}

//==============================================================================
rule rAge3FindGP
minInterval 12 //starts in cAge3
inactive
{
   //Figure out the age3 god power and create the plan.
   int id=aiGetGodPowerTechIDForSlot(2); 
   if (id == -1)
      return;
   //Create the plan
   gAge3GodPowerPlanID=aiPlanCreate("Age3GodPower", cPlanGodPower);
   if (gAge3GodPowerPlanID == -1) {
      //This is bad, and we most likely can never build a plan, so kill ourselves.
      xsDisableSelf();
      return;
   }
   aiPlanSetVariableInt(gAge3GodPowerPlanID, cGodPowerPlanPowerTechID, 0, id);
   aiPlanSetDesiredPriority(gAge3GodPowerPlanID, 100);
   aiPlanSetEscrowID(gAge3GodPowerPlanID, -1);
   //Setup the god power based on the type.
   if (setupGodPowerPlan(gAge3GodPowerPlanID, aiGetGodPowerProtoIDForTechID(id)) == false) {
      aiPlanDestroy(gAge3GodPowerPlanID);
      xsDisableSelf();
      return;
   }
   if (cvOkToUseAge3GodPower)
      aiPlanSetActive(gAge3GodPowerPlanID);
   xsDisableSelf();
}

//==============================================================================
rule rAge4FindGP
minInterval 12 //starts in cAge4
inactive
{
   //Figure out the age4 god power and create the plan.
   int id = aiGetGodPowerTechIDForSlot(3); 
   if (id == -1)
      return;
   //Create the plan.
   gAge4GodPowerPlanID = aiPlanCreate("Age4GodPower", cPlanGodPower);
   if (gAge4GodPowerPlanID == -1) {
      //This is bad, and we most likely can never build a plan, so kill ourselves.
      xsDisableSelf();
      return;
   }

   aiPlanSetVariableInt(gAge4GodPowerPlanID, cGodPowerPlanPowerTechID, 0, id);
   aiPlanSetDesiredPriority(gAge4GodPowerPlanID, 100);
   aiPlanSetEscrowID(gAge4GodPowerPlanID, -1);

   //Setup the god power based on the type.
   if (setupGodPowerPlan(gAge4GodPowerPlanID, aiGetGodPowerProtoIDForTechID(id)) == false) {
      aiPlanDestroy(gAge4GodPowerPlanID);
      xsDisableSelf();
      return;
   }

   if (cvOkToUseAge4GodPower)
      aiPlanSetActive(gAge4GodPowerPlanID);

   xsDisableSelf();
}

//==============================================================================
void gpAge2Handler(int age=1)
{
   xsEnableRule("rAge2FindGP");
}

//==============================================================================
void gpAge3Handler(int age=2)
{
   xsEnableRule("rAge3FindGP");  
}

//==============================================================================
void gpAge4Handler(int age=3)
{
   xsEnableRule("rAge4FindGP");
}

//==============================================================================
rule rPlaceTitanGate
minInterval 11 //starts in cAge5
inactive
{
   //Figure out the age 5 (yes, 5) god power and create the plan.
   int id = aiGetGodPowerTechIDForSlot(4);
   gPlaceTitanGatePlanID = aiPlanCreate("PlaceTitanGate", cPlanGodPower);
   if (gPlaceTitanGatePlanID == -1) { // TODO: does this work at all?
      xsSetRuleMinIntervalSelf(127);
      return;
   }
   // Set the Base
   aiPlanSetBaseID(gPlaceTitanGatePlanID, kbBaseGetMainID(cMyID));
   aiPlanSetVariableInt(gPlaceTitanGatePlanID,  cGodPowerPlanPowerTechID, 0, id);
   aiPlanSetDesiredPriority(gPlaceTitanGatePlanID, 100);
   aiPlanSetEscrowID(gPlaceTitanGatePlanID, -1);
   //Setup the plan.. 
   aiPlanSetVariableBool(gPlaceTitanGatePlanID, cGodPowerPlanAutoCast, 0, true); 
   aiPlanSetVariableInt(gPlaceTitanGatePlanID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelWorkers);
   aiPlanSetVariableInt(gPlaceTitanGatePlanID, cGodPowerPlanCount, 0, 6);
   aiPlanSetVariableInt(gPlaceTitanGatePlanID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelTown);
   //-- override the default building placement distance so that the Titan Gate has some room to cast
   //-- it is pretty big..
   aiPlanSetVariableFloat(gPlaceTitanGatePlanID, cGodPowerPlanBuildingPlacementDistance, 0, 110.0);
   aiPlanSetActive(gPlaceTitanGatePlanID);
   xsDisableSelf();
}

//==============================================================================
rule castHeavyGP
minInterval 12  //starts in cAge4
inactive
{
   //check if we have a gEnemySettlementAttPlanID
   if ( gEnemySettlementAttPlanID < 0 )
      return;
   static int CastAttempt = 0; 
   //get the targetPlayerID, the targetID, its unitType, its health and its position
   int targetPlayerID = aiPlanGetVariableInt(gEnemySettlementAttPlanID, cAttackPlanPlayerID, 0);
   int targetID = aiPlanGetVariableInt(gEnemySettlementAttPlanID, cAttackPlanSpecificTargetID, 0);
   if (targetID < 0)
      targetID = getMainBaseUnitIDForPlayer(targetPlayerID);
   if ( kbUnitIsType(targetID, cUnitTypeAbstractSettlement) == false )
      return;
   if ( kbUnitGetHealth(targetID) < 0.5 )
      return;
   vector targetPosition = kbUnitGetPosition(targetID);
   if ( kbLocationVisible(targetPosition) == false )
      return;

   //check if the settlement is still being built
   int numSettlementsBeingBuiltAtTargetPos = getNumUnits(cUnitTypeAbstractSettlement, cUnitStateBuilding, -1, targetPlayerID, targetPosition, 5.0);
   if (numSettlementsBeingBuiltAtTargetPos > 0)
      return;
   //count the number of enemy buildings in range
   int numMilBuildingsInR50 = getNumUnitsByRel(cUnitTypeMilitaryBuilding, cUnitStateAlive, -1, cPlayerRelationEnemy, targetPosition, 50.0);
   int mainBaseID = kbBaseGetMainID(cMyID);
   vector mainBaseLocation = kbBaseGetLocation(cMyID, mainBaseID);
   float distanceToMainBase = xsVectorLength(mainBaseLocation - targetPosition);

   if (distanceToMainBase > 110.0) {
      if (numMilBuildingsInR50 <= 2)
         return;
      //count the units in range
      int enemyVilUnitsInR50 = getNumUnitsByRel(cUnitTypeAbstractVillager, cUnitStateAlive, -1, cPlayerRelationEnemy, targetPosition, 50.0, true);
      int enemyMilUnitsInR50 = getNumUnitsByRel(cUnitTypeLogicalTypeLandMilitary, cUnitStateAlive, -1, cPlayerRelationEnemy, targetPosition, 50.0, true);
      int Combined = enemyVilUnitsInR50 + enemyMilUnitsInR50;
      if  (Combined < 10)
         return;
      //TODO: Maybe also check if we have enough resources?
   }

   //cast gHeavyGPTechID
   if ( aiCastGodPowerAtPosition(gHeavyGPTechID, targetPosition) ) {
      aiPlanDestroy( gHeavyGPPlanID );
      gHeavyGPPlanID = -1;
      CastAttempt++;
      if ( CastAttempt > 3 )		
         xsDisableSelf();
   }
}

//==============================================================================
void findTownDefenseGP(int baseID = -1)
{
   if (gTownDefenseGodPowerPlanID != -1)
      return;
   gTownDefenseGodPowerPlanID = aiFindBestTownDefenseGodPowerPlan();
   if (gTownDefenseGodPowerPlanID == -1)
      return;
   int mainBaseID = kbBaseGetMainID(cMyID);
   //remember the evaluation model and change it.
   gTownDefenseEvalModel = aiPlanGetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanEvaluationModel, 0);
   aiPlanSetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanEvaluationModel, 0, cGodPowerEvaluationModelCombatDistancePosition);
   //remember the targeting model and change it.
   gTownDefenseTargetingModel = aiPlanGetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanTargetingModel, 0);
   aiPlanSetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanTargetingModel, 0, cGodPowerTargetingModelLocation);
   //remember the playerID.
   gTownDefensePlayerID = aiPlanGetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanQueryPlayerID, 0);
   //remember the location and change it.
   gTownDefenseLocation = aiPlanGetVariableVector(gTownDefenseGodPowerPlanID, cGodPowerPlanQueryLocation, 0);
   aiPlanSetVariableVector(gTownDefenseGodPowerPlanID, cGodPowerPlanQueryLocation, 0, kbBaseGetLocation(cMyID, baseID));
   //change the distance.
   float distance = 40.0;
   if (baseID == mainBaseID)
      distance = 55.0;
   aiPlanSetVariableFloat(gTownDefenseGodPowerPlanID, cGodPowerPlanDistance, 0, distance);
}

//==============================================================================
void releaseTownDefenseGP()
{
   if (gTownDefenseGodPowerPlanID == -1)
      return;
   //Change the evaluation model back.
   aiPlanSetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanEvaluationModel, 0, gTownDefenseEvalModel);
   //Reset the player.
   aiPlanSetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanQueryPlayerID, 0, gTownDefensePlayerID);
   //Change the targeting model back
   aiPlanSetVariableInt(gTownDefenseGodPowerPlanID, cGodPowerPlanTargetingModel, 0, gTownDefenseTargetingModel);
   //change the location back
   aiPlanSetVariableVector(gTownDefenseGodPowerPlanID, cGodPowerPlanQueryLocation, 0, gTownDefenseLocation);
   //change the distance back
   aiPlanSetVariableFloat(gTownDefenseGodPowerPlanID, cGodPowerPlanDistance, 0, 55.0);
   //Release the plan.
   gTownDefenseGodPowerPlanID = -1;
   gTownDefenseEvalModel = -1; 
   gTownDefensePlayerID = -1;
   gTownDefenseTargetingModel = -1;
   gTownDefenseLocation = cInvalidVector;
}

//============================================================================================================
rule CastAbundance minInterval 10 inactive
{
   int dropsiteid = -1;
   int highestGatherers = 0;
   int numPlans = aiPlanGetNumber( cPlanGather, -1, true );
   for(i=0; < numPlans) {
      int planid = aiPlanGetIDByIndex( cPlanGather, -1, true, i );
      int gatherers = aiPlanGetNumberUnits( planid, cUnitTypeUnit );
      if ( gatherers > highestGatherers ) {
         highestGatherers = gatherers;
         dropsiteid = aiPlanGetVariableInt( planid, cGatherPlanDropsiteID, 0 );
      }
   }
   numPlans = aiPlanGetNumber( cPlanFarm, -1, true );
   for(i=0; < numPlans) {
      planid = aiPlanGetIDByIndex( cPlanFarm, -1, true, i );
      gatherers = aiPlanGetNumberUnits( planid, cUnitTypeUnit );
      if ( gatherers > highestGatherers ) {
         highestGatherers = gatherers;
         dropsiteid = aiPlanGetVariableInt( planid, cFarmPlanDropsiteID, 0 );
      }
   }
   if ( highestGatherers < 8 ) {
      if ( kbGetAge() >= cAge2 )
         dropsiteid = findUnitByIndex(cUnitTypeAbstractSettlement, 0, cUnitStateAlive, -1, cMyID, kbGetTownLocation(), 50.0 );
      else
         return;
   }
   if ( aiCastGodPowerAtUnit(cTechAbundance, dropsiteid) )
      xsDisableSelf();
}

rule CastDecoys minInterval 12 inactive
{
   if ( kbGetAge() <= cAge2 )
      return;
   vector position = kbBaseGetMilitaryGatherPoint( cMyID, kbBaseGetMainID(cMyID) );
   if ( aiCastGodPowerAtPosition(cTechDecoys, position) )
      xsDisableSelf();
}
/*
 const int cTechFloodAztec = -100000;
rule CastBigFlood minInterval 16 inactive
{
   static int CastAttempt = 0;
   int HatedPlayer = aiGetMostHatedPlayerID();
   int SettlementID = getMainBaseUnitIDForPlayer( HatedPlayer );
   if (SettlementID == -1)
      return;
   vector TownLocation = kbUnitGetPosition(SettlementID); // uses main TC
   int Houses = findUnit(cUnitTypeLogicalTypeHouses, cUnitStateAlive, -1, HatedPlayer, TownLocation, 50.0, true);
   if ( Houses >= 0 ) {
      vector HouseLocation = kbUnitGetPosition(Houses);
      int HousesThere = getNumUnits(cUnitTypeLogicalTypeHouses, cUnitStateAlive, -1, HatedPlayer, HouseLocation, 20.0);
      if ( HousesThere >= 3 ) {
         vector offset = xsVectorNormalize( kbBaseGetLocation(cMyID, kbBaseGetMainID(cMyID)) - HouseLocation );
         vector target = HouseLocation + (offset * 60.0);
         if ( (xsVectorLength(target-HouseLocation) < 125.0) && kbLocationVisible(target) ) {
            vector finalPosition = TownLocation - (TownLocation - target) * 2.0;
            if ( aiCastGodPowerAtPositionFacingPosition(cTechFloodAztec, target, finalPosition) ) {
               CastAttempt++;
               if ( CastAttempt > 3 )
                  xsDisableSelf();
            }
         }
      }
   }
}
*/
