//==============================================================================
// The Return of the Gods AI
//==============================================================================
// ROTGAIMICRO.xs
// This file contains new micro rules
// by JeHathor
//
//==============================================================================


//==============================================================================
// ORDER: findUnitRM
//
// Will find a unit of the given playerID
//==============================================================================
int findUnitRM(int playerID=0, int state=cUnitStateAny, int unitTypeID=-1, bool random=true, int actionType=-1)
{
   int unitQueryID = kbUnitQueryCreate("miscFindUnitQuery");
   if (playerID < 0) {
      kbUnitQuerySetPlayerRelation(unitQueryID, cPlayerRelationAny);
      kbUnitQuerySetPlayerID(unitQueryID, -1, false);
   }
   else {
      kbUnitQuerySetPlayerID(unitQueryID, playerID, false);
      kbUnitQuerySetPlayerRelation(unitQueryID, -1);
   }
   kbUnitQuerySetUnitType(unitQueryID, unitTypeID);
   kbUnitQuerySetState(unitQueryID, state);
   kbUnitQuerySetActionType(unitQueryID, actionType);
   int retval = -1;
   int number = kbUnitQueryExecute( unitQueryID );
   if ( number > 0 ) {
      if ( random )
         retval = kbUnitQueryGetResult(unitQueryID, aiRandInt(number));
      else 
         retval = kbUnitQueryGetResult(unitQueryID, 0);
   }
   kbUnitQueryResetResults(unitQueryID);
   return( retval );
}

//=============================================================================
//   findWeakestUnitInRadius
//=============================================================================
int findWeakestUnitInRadius(int puid=-1, int playerID=0, int state=cUnitStateAny, int unitTypeID=-1, float radius=10.0, int playerRelation=-1, bool sameArea=false)
{
   int unitQueryID = kbUnitQueryCreate("miscFindUnitQuery");
   if (playerID < 0) {
      kbUnitQuerySetPlayerRelation(unitQueryID, playerRelation);
      kbUnitQuerySetPlayerID(unitQueryID, -1, false); }
   else {
      kbUnitQuerySetPlayerID(unitQueryID, playerID, false);
      kbUnitQuerySetPlayerRelation(unitQueryID, -1); }
   kbUnitQuerySetUnitType(unitQueryID, unitTypeID);
   kbUnitQuerySetState(unitQueryID, state);	

   vector center = kbUnitGetPosition(puid);
   if ( sameArea )
      kbUnitQuerySetAreaGroupID(unitQueryID, kbAreaGroupGetIDByPosition(center));
   kbUnitQuerySetPosition( unitQueryID, center );
   kbUnitQuerySetMaximumDistance( unitQueryID, radius );

   int numberFound = kbUnitQueryExecute(unitQueryID);
   int weakestID = -1;
   float minHP = 9.9e12;
   for(i=0; < numberFound) {
      int unitid = kbUnitQueryGetResult(unitQueryID, i);
      float currentHP = kbUnitGetCurrentHitpoints( unitid );
      if ( currentHP < minHP ) {
         minHP = currentHP;
         weakestID = unitid;
      }
   }
   kbUnitQueryResetResults( unitQueryID );
   return( weakestID );
}

//==============================================================================
// ORDER: findClosestRelTo
//
// Will find the closest unit relative to the given unit
//==============================================================================
int findClosestRelTo(int puid=-1, int playerID=0, int state=cUnitStateAny, int unitTypeID=-1,
                     int playerRelation=-1, bool sameArea=false)
{
   static int unitQueryID=-1;
   if ( unitQueryID < 0 ) {
      unitQueryID = kbUnitQueryCreate("findClosestFunc");
      kbUnitQuerySetAscendingSort( unitQueryID, true );
   }
   if ( playerID < 0 ) {  //-1 for playerRelation
      kbUnitQuerySetPlayerRelation( unitQueryID, playerRelation );
      kbUnitQuerySetPlayerID( unitQueryID, -1, false );
   }
   else {
      kbUnitQuerySetPlayerRelation( unitQueryID, -1 );
      kbUnitQuerySetPlayerID( unitQueryID, playerID );
   }
   if ( sameArea )
      kbUnitQuerySetAreaGroupID( unitQueryID, kbAreaGroupGetIDByPosition(kbUnitGetPosition(puid)) );	
   else
      kbUnitQuerySetAreaGroupID( unitQueryID, -1 );
   kbUnitQuerySetUnitType( unitQueryID, unitTypeID );
   kbUnitQuerySetState( unitQueryID, state );
   int retval = -1;
   if ( kbUnitQueryExecute(unitQueryID) > 0 )
      retval = kbUnitQueryGetResult( unitQueryID, 0 );
   kbUnitQueryResetResults( unitQueryID );
   return( retval );
}

//===============================================================================================================================
int getUnitsAtLocQID(int playerID=0, int state=cUnitStateAny, int unitTypeID=-1, vector center=cInvalidVector, float radius=-1.0)
{
   static int unitQueryID = -1;
   if (unitQueryID < 0)
      unitQueryID = kbUnitQueryCreate( "unitsAtLocSetup" );
   kbUnitQuerySetPlayerID( unitQueryID, playerID, false );
   kbUnitQuerySetUnitType( unitQueryID, unitTypeID );
   kbUnitQuerySetState( unitQueryID, state );
   kbUnitQuerySetPosition( unitQueryID, center );
   kbUnitQuerySetMaximumDistance( unitQueryID, radius );
   return( unitQueryID );
}

//===========================================================================================================
float calcDistanceToUnit(int puidA=-1, int puidB=-1) {
   return( xsVectorLength(kbUnitGetPosition(puidA) - kbUnitGetPosition(puidB)) );
}

//===========================================================================================================
int aiTaskUnitKeepDistanceToUnit(int actorUnitID=-1, int targetUnitID=-1, float fDistance=1.0)
{
   vector A = kbUnitGetPosition(targetUnitID);
   vector path = ( xsVectorNormalize(A - kbUnitGetPosition(actorUnitID)) * fDistance );
   aiTaskUnitMove( actorUnitID, A - path );
}

// *****************************************************************************
// ORDER: landKiting [Age2]
// (Sending damaged archers out of range of hand combative enemies)
// *****************************************************************************
void landKiting(void)
{
   int myUnitID = findUnitRM(cMyID, cUnitStateAlive, cUnitTypeAbstractArcher, true);
   if ( myUnitID < 0 )
      return;

   //nearest enemy unit closer then 8 feet?
   int mhpUnitID = findClosestRelTo(myUnitID, aiGetMostHatedPlayerID(), cUnitStateAlive, cUnitTypeMilitary);
   float distance = calcDistanceToUnit( myUnitID, mhpUnitID );

   if ( kbUnitGetHealth(myUnitID) <= 0.5 ) {
      if ( distance < 8.0 )
         aiTaskUnitKeepDistanceToUnit( myUnitID, mhpUnitID, 12.0 );
      else if ( (distance > 8.0) && (distance < 13.0) )
         aiTaskUnitWork( myUnitID, mhpUnitID );
   }
   else if ( distance < 8.0 ) {
      if ( kbUnitIsType(mhpUnitID, cUnitTypeAbstractTitan) )
         aiTaskUnitKeepDistanceToUnit( myUnitID, mhpUnitID, 12.0 );
   }
}

// *****************************************************************************
// ORDER: milAvoidTowers [Age2]
// (Keeping damaged soldiers away from buildings that shoot)
// *****************************************************************************
void milAvoidTowers(void)
{
   int myUnitID = findUnitRM(cMyID, cUnitStateAlive, cUnitTypeHumanSoldier, true);
   if (myUnitID < 0 )
      return;

   //current Hp lower then half of my max?
   if ( kbUnitGetHealth(myUnitID) <= 0.45 ) {  //nearest building closer then 20 feet?
      int mhpBuildingID = findClosestRelTo( myUnitID, aiGetMostHatedPlayerID(), cUnitStateAlive, cUnitTypeBuildingsThatShoot );
      float objectDistance = calcDistanceToUnit( myUnitID,mhpBuildingID );
      if ( (objectDistance <= 22.0) && (objectDistance >= 8.0) )
         aiTaskUnitKeepDistanceToUnit( myUnitID, mhpBuildingID, 20.0 );  //Run!!!
   }
}

// *****************************************************************************
// ORDER: mythAvoidHeroes [Age2]
// (Sending damaged mythological units away from heroes)
// *****************************************************************************
void mythAvoidHeroes(void)
{
   int myUnitID = findUnitRM(cMyID, cUnitStateAlive, cUnitTypeLogicalTypeMythUnitNotTitan, true, cActionAttack);
   if ( myUnitID < 0 )
      return;
   bool conditionA = ( kbUnitGetHealth(myUnitID) >= 0.8 );
   if ( conditionA == false )
      return;
   //nearest enemy hero closer then 5 feet?
   int mhpUnitID = findClosestRelTo( myUnitID, aiGetMostHatedPlayerID(), cUnitStateAlive, cUnitTypeHero );
   if ( calcDistanceToUnit(myUnitID, mhpUnitID) < 5.0 )
      aiTaskUnitKeepDistanceToUnit( myUnitID, mhpUnitID, 20.0 );  //Run!!!
}

// *****************************************************************************
// ORDER: navyKiting [Age2]
// (Sending damaged ships out of range of enemy military)
// *****************************************************************************
void navyKiting(void)
{
   int myUnitID = findUnitRM(cMyID, cUnitStateAlive, cUnitTypeArcherShip, true, cActionRangedAttack);
   if ( myUnitID < 0 )
      return;
   //current Hp lower then half of my max?
   if ( kbUnitGetHealth(myUnitID) >= 0.5 )
      return;
   //nearest enemy unit closer then my max fire distance?
   float fireAvoidDistance = 10.0;
   if ( kbGetTechStatus(cTechCladding) >= cTechStatusActive )
      fireAvoidDistance = fireAvoidDistance + 2.0;

   int mhpUnitID = findClosestRelTo( myUnitID, aiGetMostHatedPlayerID(), cUnitStateAlive, cUnitTypeMilitary );
   if ( calcDistanceToUnit(myUnitID, mhpUnitID) < fireAvoidDistance )
      aiTaskUnitKeepDistanceToUnit( myUnitID, mhpUnitID, 30.0 );	//Retreat!!!
   else {
      int targetUnitID = findWeakestUnitInRadius(myUnitID,aiGetMostHatedPlayerID(),cUnitStateAlive,cUnitTypeShip,fireAvoidDistance+5,-1);
      aiTaskUnitWork( myUnitID, targetUnitID );		//Hit!!!
   }
}

// *****************************************************************************
// ORDER: helpBuilding [Age1]
// (Helps to build important buildings at the front)
void helpBuilding(void)
{
   int myBuildingID = findUnitRM(cMyID, cUnitStateBuilding, cUnitTypeBuilding, true);
   if ( myBuildingID < 0 )
      return;
   if ( kbUnitIsType(myBuildingID, cUnitTypeFarm) )
      return;
   int myUnitID = findUnitRM(cMyID, cUnitStateAlive, cUnitTypeVillagerAztec, true);
   if ( kbUnitGetActionType(myUnitID) == cActionBuild )
      return;			//No multitasking!
   if ( kbUnitGetHealth(myUnitID) >= 0.75 ) {
      if ( calcDistanceToUnit(myUnitID,myBuildingID) < 10.0 )
         aiTaskUnitWork(myUnitID, myBuildingID);		//Build it!
   }
}

// *****************************************************************************
// RULE: microHandlerSingle
// *****************************************************************************
rule microHandlerSingle
group AttackRules
inactive
highFrequency
{
   landKiting();
   milAvoidTowers();
   mythAvoidHeroes();
   navyKiting();
   //	helpBuilding();
}

// *****************************************************************************
// RULE: microHandlerMulti
// *****************************************************************************
rule microHandlerMulti
group AttackRules
inactive
minInterval 1
{
   landKiting();
   milAvoidTowers();
   mythAvoidHeroes();
   navyKiting();
   //	helpBuilding();
}

// *****************************************************************************
// RULE: microStarter
// *****************************************************************************
rule microStarter
minInterval 5
active
runImmediately
{
   //Reduce traffic for online matches
   if (aiIsMultiplayer()) {
      if (cNumberPlayers > 4)
         xsEnableRule("microHandlerSingle");
      else
         xsEnableRule("microHandlerMulti");
   }
   else
      xsEnableRule("microHandlerSingle");
   xsDisableSelf();	//Go away.

}
