//==============================================================================
// The Return of the Gods AI
//==============================================================================
// ROTGAITechs.xs
// This file contains tech rules


//==============================================================================
rule getOmniscience
minInterval 24 //starts in cAge4
inactive
{
   //If we can afford it twice over, then get it.
   float goldCost=kbTechCostPerResource(cTechOmniscience, cResourceGold) * 2.0;
   float currentGold=kbResourceGet(cResourceGold);
   if (goldCost>currentGold)
      return;
   createSimpleResearchPlan(cTechOmniscience, -1, cMilitaryEscrowID, 100, true);
   xsDisableSelf();
}

//==============================================================================
rule getMasons
minInterval 131 //starts in cAge2
inactive
{
   const int techID = cTechMasons;
   if (kbGetTechStatus(techID) > cTechStatusResearching) {
      xsDisableSelf();
      xsEnableRule("getArchitects");
      return;
   }
   if (kbGetTechStatus(cTechWatchTower) < cTechStatusResearching)
      return;
   int pid = aiPlanGetIDByTypeAndVariableType(cPlanProgression, cProgressionPlanGoalTechID, techID, true);
   if ( aiPlanGetState(pid) < 0 )
      pid = createSimpleResearchPlan( techID, -1, cRootEscrowID, 80, true );
}

//==============================================================================
rule getArchitects
minInterval 131 //starts in cAge3, activated in getMasons
inactive
{
   const int techID = cTechArchitects;
   if (kbGetTechStatus(techID) > cTechStatusResearching) {
      xsDisableSelf();
      return;
   }
   if (kbGetTechStatus(cTechWatchTower) < cTechStatusResearching)
      return;
   int pid = aiPlanGetIDByTypeAndVariableType(cPlanProgression, cProgressionPlanGoalTechID, techID, true);
   if ( aiPlanGetState(pid) < 0 )
      pid = createSimpleResearchPlan( techID, -1, cRootEscrowID, 80, true );
}

//==============================================================================
rule getFortifiedTownCenter
   inactive
minInterval 41 //starts in cAge3
{
   const int techID = cTechFortifyTownCenter;
   createSimpleResearchPlan( techID, -1, cMilitaryEscrowID, 90, false );
   xsDisableSelf();
}

//==============================================================================
rule getEnclosedDeck
minInterval 32 //starts in cAge2
inactive
{
   const int techID = cTechEnclosedDeck;
   static int ruleStartTime = -1;
   if (ruleStartTime == -1)
      ruleStartTime = xsGetTime();
   if ((kbGetTechStatus(cTechWatchTower) < cTechStatusResearching) && (xsGetTime() - ruleStartTime < 5*60*1000))
      return;
   float woodSupply = kbResourceGet(cResourceWood);
   float foodSupply = kbResourceGet(cResourceFood);
   float goldSupply = kbResourceGet(cResourceGold);
   if ((foodSupply > 560) && (goldSupply > 350) && (kbGetAge() == cAge2) || (foodSupply < 400) || (woodSupply < 400))
      return;
   createSimpleResearchPlan( techID, -1, cRootEscrowID, 60, true );
   xsDisableSelf();
}

//==============================================================================
rule getPurseSeine
minInterval 30 //starts in cAge2
inactive
{
   const int techID = cTechPurseSeine;
   createSimpleResearchPlan(techID, -1, cEconomyEscrowID, 45, true);
   xsDisableSelf();
}

//==============================================================================
rule getSaltAmphora
minInterval 30 //starts in cAge3
inactive
{
   const int techID = cTechSaltAmphora;
   if (aiPlanGetIDByTypeAndVariableType(cPlanProgression, cProgressionPlanGoalTechID, techID, true) < 0) {
      createSimpleResearchPlan(techID, -1, cEconomyEscrowID, 80, true);
      xsDisableSelf();
   }
}

//==============================================================================
rule getHusbandry
minInterval 23
inactive
{
   const int techID = cTechHusbandry;
   if ( kbGetTechStatus(techID) > cTechStatusResearching ) {
      xsDisableSelf();
      return;
   }
   if ( aiPlanGetIDByTypeAndVariableType(cPlanProgression, cProgressionPlanGoalTechID, techID, true) >= 0 )
      return;
   if (kbUnitCount(cMyID, cUnitTypeEarlyFoodDropsite, cUnitStateAlive) < 1)
      return;
   if (kbGetAge() < cAge2) {   
      if (kbGetTechStatus(gAge2MinorGod) < cTechStatusResearching)
         return;
   }
   if ( (kbGetTechStatus(cTechPickaxe) < cTechStatusResearching) || (kbGetTechStatus(cTechHandAxe) < cTechStatusResearching) ||
      (gHuntersExist) && (kbGetTechStatus(cTechHuntingDogs) < cTechStatusResearching) )
   return;
   createSimpleResearchPlan(techID, -1, cEconomyEscrowID, 99, true);
}

//==============================================================================
rule getHuntingDogs
minInterval 167 //starts in cAge1, gets set to 11
inactive
{
   const int techID = cTechHuntingDogs;
   if ( (kbGetTechStatus(techID) > cTechStatusResearching) || (kbGetAge() >= cAge3) ) {
      xsDisableSelf();
      return;
   }
   static bool update = false;
   if (update == false) {
      xsSetRuleMinIntervalSelf(11);
      update = true;
   }
   if ( aiPlanGetIDByTypeAndVariableType(cPlanProgression, cProgressionPlanGoalTechID, techID, true) >= 0 )
      return;
   if ( kbUnitCount(cMyID, cUnitTypeGranary, cUnitStateAlive) < 1 )
      return;
   if ( kbUnitCount(cMyID, cUnitTypeAbstractTemple, cUnitStateAliveOrBuilding) < 1 )
      return;
   int count = aiGetResourceBreakdownNumberPlans( cResourceFood, cAIResourceSubTypeHuntAggressive, kbBaseGetMainID(cMyID) );
   gHuntersExist = ( count > 0 );
   if ( gHuntersExist ) {
      createSimpleResearchPlan(techID, -1, cEconomyEscrowID, 100, true);
      xsSetRuleMinIntervalSelf(167);
      update = false;
   }
}

rule AztecTechsForce minInterval 4 inactive //Enabled in Age2.
{
   //   int tcid = findUnit( cUnitTypeSettlementsThatTrainVillagers );
   int buildingid = findUnit( cUnitTypeLogicalTypeHouses );
   if ( buildingid < 0 ) {
      buildingid = findUnit( cUnitTypeAbstractWall );
   if ( buildingid < 0 ) {
      buildingid = findUnit( cUnitTypeAbstractFarm );
   if ( buildingid < 0 ) {
      buildingid = findUnit( cUnitTypeEarlyFoodDropsite );
   if ( buildingid < 0 ) {
      buildingid = findUnit( cUnitTypeTower );
   }}}}

   int actives = 0;
   int required = 9999;
   static int tier = cAge2;
   switch( tier ) {
      case cAge2: {
         actives = try_tech( buildingid, cTechWarPriestCeremony, actives );
         actives = try_tech( buildingid, cTechWatchTower, actives );
         actives = try_tech( buildingid, cTechCrenellations, actives );
         actives = try_tech( buildingid, cTechHusbandry, actives );
         actives = try_tech( buildingid, cTechPlow, actives );
         actives = try_tech( buildingid, cTechHandAxe, actives );
         actives = try_tech( buildingid, cTechPickaxe, actives );
         required = 7;
         break;
      }
      case cAge3: {
         actives = try_tech( buildingid, cTechStoneWall, actives );
         actives = try_tech( buildingid, cTechSignalFires, actives );
         actives = try_tech( buildingid, cTechCarrierPigeons, actives );
         actives = try_tech( buildingid, cTechBoilingOil, actives );
         actives = try_tech( buildingid, cTechMasons, actives );
         actives = try_tech( buildingid, cTechIrrigation, actives );
         actives = try_tech( buildingid, cTechBowSaw, actives );
         actives = try_tech( buildingid, cTechShaftMine, actives );
         actives = try_tech( buildingid, cTechArchitects, actives );
         required = 9;
         break;
      }
      case cAge4: {
         actives = try_tech( buildingid, cTechFloodControl, actives );
         actives = try_tech( buildingid, cTechCarpenters, actives );
         actives = try_tech( buildingid, cTechQuarry, actives );
         actives = try_tech( buildingid, cTechCoinage, actives );
         required = 4;
         break;
      }
   }  //End: switch()
   if ( actives >= required )
      tier++;
   if ( aiGetPauseAllAgeUpgrades() )
      return;
   buildingid = findUnit( cUnitTypeSettlementsThatTrainVillagers );
   switch( kbGetAge() ) {
   case cAge1: { aiTaskUnitResearch(buildingid, gAge2MinorGod); break; }
   case cAge2: { aiTaskUnitResearch(buildingid, gAge3MinorGod); break; }
   case cAge3: { aiTaskUnitResearch(buildingid, gAge4MinorGod); break; }
   case cAge4: { aiTaskUnitResearch(buildingid, cTechSecretsoftheTitans); break; }
   case cAge5: { if (tier >= cAge5) xsDisableSelf(); break; } }
}

rule AztecTechs runImmediately minInterval 60 inactive
{
   const int MaxTechsDone = 5;  //Rule Disables if this many plan-end-techs complete.
   int techsdone = 0;
   if ( kbGetTechStatus(cTechWarPriestCeremony) >= cTechStatusActive ) {
      techsdone++;
      if ( gGatherRelicType != cUnitTypeWarPriest ) {
         gGatherRelicType = cUnitTypeWarPriest;
         aiPlanAddUnitType( gRelicGatherPlanID, cUnitTypeWarPriest, 1, 1, 1 );
         aiPlanAddUnitType( gRelicGatherPlanID, cUnitTypeTlatoani, 0, 0, 0 );
      }
   }
   else if ( aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, cTechWarPriestCeremony) < 0 )
      createSimpleResearchPlan( cTechWarPriestCeremony, -1, cEconomyEscrowID, 80, false );

   static int barracks = -1;
   if ( kbGetTechStatus(cTechWarCulture2) >= cTechStatusActive )
      techsdone++;
   else if ( aiPlanGetState(barracks) < 0 ) {
      aiPlanDestroy( barracks );
      barracks = aiPlanCreate( "barrack", cPlanProgression );
      aiPlanSetNumberVariableValues( barracks, cProgressionPlanGoalTechID, 5, true );
      aiPlanSetVariableInt( barracks, cProgressionPlanGoalTechID, 0, cTechMediumYouthBarracks );
      aiPlanSetVariableInt( barracks, cProgressionPlanGoalTechID, 1, cTechWarCulture );
      aiPlanSetVariableInt( barracks, cProgressionPlanGoalTechID, 2, cTechHeavyYouthBarracks );
      aiPlanSetVariableInt( barracks, cProgressionPlanGoalTechID, 3, cTechChampionYouthBarracks );
      aiPlanSetVariableInt( barracks, cProgressionPlanGoalTechID, 4, cTechWarCulture2 );
      aiPlanSetEscrowID( barracks, cMilitaryEscrowID );
      aiPlanSetDesiredPriority( barracks, 52 );
      aiPlanSetActive( barracks );
   }

   if ( kbGetAge() >= cAge3 ) {
      static int nobles = -1;
      if ( kbGetTechStatus(cTechInitiation2) >= cTechStatusActive )
         techsdone++;
      else if ( aiPlanGetState(nobles) < 0 ) {
         aiPlanDestroy( nobles );
         nobles = aiPlanCreate( "noble", cPlanProgression );
         aiPlanSetNumberVariableValues( nobles, cProgressionPlanGoalTechID, 4, true );
         aiPlanSetVariableInt( nobles, cProgressionPlanGoalTechID, 0, cTechHeavyNobles );
         aiPlanSetVariableInt( nobles, cProgressionPlanGoalTechID, 1, cTechInitiation );
         aiPlanSetVariableInt( nobles, cProgressionPlanGoalTechID, 2, cTechChampionNobles );
         aiPlanSetVariableInt( nobles, cProgressionPlanGoalTechID, 3, cTechInitiation2 );
         aiPlanSetEscrowID( nobles, cMilitaryEscrowID );
         aiPlanSetDesiredPriority( nobles, 52 );
         aiPlanSetActive( nobles );
      }
      static int guild = -1;
      if ( kbGetTechStatus(cTechBurningPitch) >= cTechStatusActive )
         techsdone++;
      else if ( aiPlanGetState(guild) < 0 ) {
         aiPlanDestroy( guild );
         guild = aiPlanCreate( "guildTechs", cPlanProgression );
         aiPlanSetNumberVariableValues( guild, cProgressionPlanGoalTechID, 4, true );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 0, cTechMesoHackAttack1 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 1, cTechMesoPierceAttack1 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 2, cTechMesoInfArmor1 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 3, cTechMesoArchArmor1 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 4, cTechMesoHackAttack2 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 5, cTechMesoPierceAttack2 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 6, cTechMesoInfArmor2 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 7, cTechMesoArchArmor2 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 8, cTechMesoHackAttack3 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 9, cTechMesoPierceAttack3 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 10, cTechMesoInfArmor3 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 11, cTechMesoArchArmor3 );
         aiPlanSetVariableInt( guild, cProgressionPlanGoalTechID, 12, cTechBurningPitch );
         aiPlanSetEscrowID( guild, cMilitaryEscrowID );
         aiPlanSetDesiredPriority( guild, 65 );
         aiPlanSetActive( guild );
      }
   }
   if ( kbGetAge() >= cAge4 ) {
      if ( kbGetTechStatus(cTechWarriorSociety) >= cTechStatusActive )
         techsdone++;
      else if ( aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, cTechWarriorSociety) < 0 )
         createSimpleResearchPlan( cTechWarriorSociety, -1, cMilitaryEscrowID, 40, false );
      aiTaskUnitResearch( findUnit(cUnitTypeBuilding), cTechWarriorSociety );
   }
   if ( techsdone >= MaxTechsDone )
      xsDisableSelf();
}

//==============================================================================
rule getAmbassadors
inactive
minInterval 60 //starts in cAge3
{
   const int techID = cTechAmbassadors;
   if (kbUnitCount(cMyID, cUnitTypeMarket, cUnitStateAliveOrBuilding) < 1)
      return;

   float goldSupply = kbResourceGet(cResourceGold);
   if ((goldSupply < 500) || (kbGetAge() < cAge4))
      return;
   createSimpleResearchPlan(techID, -1, cEconomyEscrowID, 25);
   xsDisableSelf();
}
//==============================================================================
rule getTaxCollectors
inactive
minInterval 47 //starts in cAge3
{
   const int techID = cTechTaxCollectors;
   if (kbGetTechStatus(techID) > cTechStatusResearching) {
      xsDisableSelf();
      return;
   }


   float goldSupply = kbResourceGet(cResourceGold);
   float foodSupply = kbResourceGet(cResourceFood);

   if (aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techID, true) >= 0) {
      if (kbGetTechStatus(techID) < cTechStatusResearching) {
         if ((foodSupply > 700) && (goldSupply > 700) && (kbGetAge() == cAge3)) {
            aiPlanDestroy(aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techID, true));
            xsSetRuleMinIntervalSelf(47);
         }
      }
      return;
   }

   if (kbUnitCount(cMyID, cUnitTypeMarket, cUnitStateAliveOrBuilding) < 1)
      return;
   if ((goldSupply < 600) || (foodSupply < 600))
      return;
   if ((foodSupply > 700) && (goldSupply > 700) && (kbGetAge() == cAge3))
      return;

   createSimpleResearchPlan(techID, -1, cEconomyEscrowID, 50);
}

//==============================================================================
rule getHeroicFleet inactive minInterval 300 //starts in cAge2
{
   const int techID = cTechHeroicFleet;
   createSimpleResearchPlan(techID, -1, cMilitaryEscrowID, 50);
   xsDisableSelf();
}

//==============================================================================
rule getCrenellations
inactive
minInterval 79 //starts in cAge2
{
   const int techID = cTechCrenellations;
   if (kbGetTechStatus(techID) > cTechStatusResearching) {
      xsDisableSelf();
      return;
   }
   float goldSupply = kbResourceGet(cResourceGold);
   float foodSupply = kbResourceGet(cResourceFood);
   if (aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techID, true) >= 0) {
      if (kbGetTechStatus(techID) < cTechStatusResearching) {
         if ((foodSupply > 560) && (goldSupply > 350) && (kbGetAge() == cAge2)) {
            aiPlanDestroy(aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techID, true));
            xsSetRuleMinIntervalSelf(79);
         }
      }
      return;
   }
   if ( kbGetTechStatus(cTechWatchTower) < cTechStatusResearching )
      return;
   if ( (kbResourceGet(cResourceWood) < 300) || (foodSupply < 300) )
      return;
   if ( (foodSupply > 560) && (goldSupply > 350) && (kbGetAge() == cAge2) )
      return;
   createSimpleResearchPlan( techID, -1, cMilitaryEscrowID, 70, false );
}

//==============================================================================
rule getSignalFires
inactive
minInterval 107 //starts in cAge2
{
   const int techID = cTechSignalFires;
   if (kbGetTechStatus(techID) > cTechStatusResearching) {
      xsEnableRule("getCarrierPigeons");
      xsDisableSelf();
      return;
   }
   if (aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techID, true) >= 0)
      return;
   if ( kbGetTechStatus(cTechWatchTower) < cTechStatusResearching )
      return;
   if (kbResourceGet(cResourceWood) < 500)
      return;
   createSimpleResearchPlan( techID, -1, cMilitaryEscrowID, 5, false );
}

//==============================================================================
rule getBoilingOil
inactive
minInterval 79 //starts in cAge3
{
   int techID = cTechBoilingOil;
   int numTowers = kbUnitCount(cMyID, cUnitTypeTower, cUnitStateAliveOrBuilding);
   int numMarkets = kbUnitCount(cMyID, cUnitTypeMarket, cUnitStateAliveOrBuilding);

   if ((numTowers < 1) || (numMarkets < 1) || (kbGetTechStatus(cTechWatchTower) < cTechStatusResearching))
      return;

   createSimpleResearchPlan(techID, -1, cMilitaryEscrowID, 70);
   xsDisableSelf();
}

//==============================================================================
rule getCarrierPigeons
inactive
minInterval 107 //starts in cAge2 activated in getSignalFires
{
   int techID = cTechCarrierPigeons;
   int numFortresses = kbUnitCount(cMyID, cUnitTypeAbstractFortress, cUnitStateAliveOrBuilding);
   int numMarkets = kbUnitCount(cMyID, cUnitTypeMarket, cUnitStateAliveOrBuilding);
   if ((numFortresses < 1) || (numMarkets < 1) || (kbGetAge() < cAge3))
      return;

   float woodSupply = kbResourceGet(cResourceWood);
   if (woodSupply < 800)
      return;

   createSimpleResearchPlan(techID, -1, cMilitaryEscrowID, 5);
   xsDisableSelf();
}

//==============================================================================
rule getWatchTower
inactive
minInterval 10 //starts in cAge2
{
   int techID = cTechWatchTower;
   createSimpleResearchPlan(techID, -1, cMilitaryEscrowID, 100);
   xsDisableSelf();
}

//==============================================================================
rule WallManager
inactive
minInterval 37
{
   int techid = cTechStoneWall;
   int status = kbGetTechStatus(techid);
   if ( status >= cTechStatusActive ) {
      if ( MyCiv == cCivQuetz ) {
         techid = cTechCoatenpantli;
         status = kbGetTechStatus(techid);
      }
      if ( (status >= cTechStatusActive) && (kbGetTechStatus(cTechAge4Ton) >= cTechStatusActive) ) {
         techid = cTechTezontleWalls;
         status = kbGetTechStatus(techid);
      }
   }
   if ( (kbGetAge() >= cAge4) && (status >= cTechStatusActive) ) {
      xsDisableSelf();
      return;
   }

   float goldSupply = kbResourceGet(cResourceGold);
   float foodSupply = kbResourceGet(cResourceFood);
   int planid = aiPlanGetIDByTypeAndVariableType(cPlanResearch, cResearchPlanTechID, techid, true);
   if (planid >= 0) {
      if ( status < cTechStatusResearching) {
         if ((foodSupply > 560) && (goldSupply > 350) && (kbGetTechStatus(gAge3MinorGod) < cTechStatusResearching) && (kbGetAge() == cAge2) 
            || (foodSupply > 800) && (goldSupply > 750) && (kbGetTechStatus(gAge4MinorGod) < cTechStatusResearching) && (kbGetAge() == cAge3))
         {
            aiPlanDestroy( planid );
            xsSetRuleMinIntervalSelf(37);
         }		
      }
      return;
   }
   if ( (foodSupply < 200) || (goldSupply < 200) || (kbGetTechStatus(cTechWatchTower) < cTechStatusResearching) && (gTransportMap == false) )
      return; 
   createSimpleResearchPlan( techid, -1, cMilitaryEscrowID, 30, false );
   xsSetRuleMinIntervalSelf(4);
}

//==============================================================================
rule getCraftsman
inactive
minInterval 20 //starts in cAge4
{
   const int techID = cTechCraftsmanship;
   int numSiegeWeapons = kbUnitCount(cMyID, cUnitTypeAbstractSiegeWeapon, cUnitStateAliveOrBuilding);
   if (numSiegeWeapons < 2)
      return;
   float goldSupply = kbResourceGet(cResourceGold);
   float foodSupply = kbResourceGet(cResourceFood);
   if ((goldSupply < 800) || (foodSupply < 600))
      return;

   createSimpleResearchPlan(techID, -1, cMilitaryEscrowID, 50, true);
   createSimpleResearchPlan(cTechBurningPitch, -1, cMilitaryEscrowID, 50, true);
   xsDisableSelf();
}

//==============================================================================
rule getCoinage
inactive
minInterval 20 //starts in cAge4
{
   createSimpleResearchPlan(cTechCoinage, -1, cEconomyEscrowID, 100);
   xsDisableSelf();
}
//==============================================================================
rule getSecretsOfTheTitan
minInterval 17 //starts in cAge4
inactive
{

   if (TitanAvailable == false) {
      xsDisableSelf();
      return;
   }
   const int techID = cTechSecretsoftheTitans;
   if (aiPlanGetIDByTypeAndVariableType(cPlanProgression, cProgressionPlanGoalTechID, techID, true) >= 0) {
      xsDisableSelf();
      return;
   }

   // Make a progression to get Titan
   int titanPID = aiPlanCreate("TitanAge", cPlanProgression);
   if (titanPID != 0) {
      aiPlanSetVariableInt(titanPID, cProgressionPlanGoalTechID, 0, techID);
      aiPlanSetDesiredPriority(titanPID, 100);
      if ( gTransportMap )
         aiPlanSetEscrowID(titanPID, cMilitaryEscrowID);
      else
         aiPlanSetEscrowID(titanPID, cEconomyEscrowID);
      aiPlanSetActive(titanPID);
      xsDisableSelf();
   }
}

