//=============================================================================================================
// ROTG_SCRIPT
// This file contains all the new XS code related to the Return of the Gods mod to make many of its new features
// possible. It is crucially important that rotg_constants.xs be up-to-date and in the same folder as this. Run
// MakeConstantsFile.exe anytime entries are added/removed/reordered in the proto/techtree files.
// File written entirely by NateE ;-).
//=============================================================================================================

include "rotg_constants.xs";

//FAKE DELETE THESE!
extern const int cUnitTypeHeroTwinM = 99999999;
extern const int cUnitTypeHeroTwinR = 99999999;
extern const int cUnitTypeEvilTwinM = 99999999;
extern const int cUnitTypeEvilTwinR = 99999999;
extern const int cTechAhPuchVillRebirth = cTechAge1;
extern const int cUnitTypePorter = cUnitTypeOxCart;
extern const int cUnitTypeVillagerMayaDeath = cUnitTypeSlinger;
extern const int cUnitTypeShadowMaya = cUnitTypeShadowRunner;
extern const int cUnitTypeShadowMayaAluxe = cUnitTypeShadowArcher;
extern const int cUnitTypeTitanInca = cUnitTypeTitanBad;
extern const int cUnitTypeTitanMaya = cUnitTypeTitanKronos;
extern const int cUnitTypeHealCeremony = 99999999;  //Must be the lowest id-numbered ceremony!
extern const int cUnitTypeSapaIncaCeremony = 999999999;
extern const int cUnitTypeSpeedCeremony = 999999999;
extern const int cUnitTypeFavorCeremony = 999999999;  //Must be the highest id-numbered ceremony!
extern const int EVIL_TWIN_DIVINE_STRIKE_COOLDOWN = 99999999;
extern const int cUnitTypeWarlordInca = cUnitTypeHeroChineseGeneral;
extern const int cTechProgMilitary1 = -1;
extern const int cTechProgReligion6 = -2;
extern const int cUnitTypeFarmBig = 99999999;
extern const int cUnitTypeCottonFarmBig = 99999899;
extern const int FARM_BUILD_QRY = 999999999;
extern const int TEMPLE_BUILD_QRY = 999999999;

//NOTES TO REMEMBER:
//1. XS doesn't support static_cast, but int/float coersion can be controlled by which operand is first.
// - decltype(float * int) = float; decltype(int * float) = int.
//2. Compare "xsStringToInt(trGetUnitScenarioName(unitid))"s to determine entity creation order (lower = earlier).
//3. Queries never contain the same unit twice (unless cleared inbetween obv), even if execute called again.
//4. Some TR funcs fail in multiplayer (why?) and alternatives are needed.

//ALL MESO-CIVS GLOBALS.
extern const int AI_NEWCIV_CHOOSE_TIME_MS = 1600;  //Time in MS that AIs have to choose one of the new civs.
extern const int TIANGUIS_PRICE_INTERVAL = 4;      //Every X food-wood swaps at the Tianguis causes price to change.
//AZTEC GLOBALS.
extern const float TLATOANI_BUILDING_AURA_RANGE = 14.75;
extern const float TLATOANI_UNIT_AURA_RANGE = 12.3;
extern const float SKULL_TEMPLE_RADIUS = 24.3;    //Radius of Skull Temple's frighten aura. Sync with SkullTemple LOS/anim.
extern const int   NUM_INVIS_WORKERS = 4;         //Max buildings affected by Tlatoani aura. Sync with stringtable desc.
extern const float SHADOW_DAMAGE_RATE = 0.8;      //Shadows of Mictlan health decay rate if not-near the Tlatoani.
extern const float CAPTURE_COST_INCR_BASE = 1.08;
extern const float CAPTURE_COST_DECREASE = 45.0;
extern const int   ERR_AI_PLAN_TIMEOUT = -2;  //Indicate we won't look for the AztecAI data-plan anymore due it being too late.
//MAYA GLOBALS.
extern const int   MAX_MAYA_SHADOWS = 6;
extern const float HERO_TWINS_XP_THRESHOLD = 120.0; //Hero Twins attacking for this many non-continuous seconds will make them summon a soldier.
extern const float HERO_DISTANCE_INTERVAL = 15.0;   //This meters apart counts as "interval" and makes the Hero Twins each get weaker.
extern const int   MAX_HERO_INTERVALS = 10;         //Twins are weakest when they are >= (HERO_DISTANCE_INTERVAL * MAX_HERO_INTERVALS) meters apart.
extern const float HERO_HP_PENALTY = -15.0;         //Hero Twins lose this HP for each interval that they are apart (negative means subtract).
extern const float HERO_ATTACK_PENALTY = -0.5;      //Ditto attack.
//INCA GLOBALS.
extern const int   MITA_ACCUMULATOR = cUnitTypeBlood;  //float
extern const float SAPA_INCA_RESPAWN_POINTS = 180.0;  //In seconds.

int IntMin(void) {
   int ret = -2147483648.0;
   return( ret );
}
int IntMax(void) {
   return( IntMin() - 1 );
}
float FloatMax(void) {
   return( 3.40282e38 );
}
float FloatMin(void) {
   return( 1.17549e-38 );
}
float Infinity(void) {
   return( 1.0 / 0.0 );
}
float NaN(void) {
   return( 0.0 / 0.0 );
}
string ResourceName(int resourceid=-1) {
   switch( resourceid ) {
   case cResourceGold:  { return("Gold"); }
   case cResourceWood:  { return("Wood"); }
   case cResourceFood:  { return("Food"); }
   case cResourceFavor: { return("Favor"); }
   case cAllResources:  { return("All Resources"); } }
   return( "Invalid" );
}
string AgeName(int ageid=-1) {
   switch( ageid ) {
   case cAge1: { return("Archaic Age"); }
   case cAge2: { return("Classical Age"); }
   case cAge3: { return("Heroic Age"); }
   case cAge4: { return("Mythic Age"); }
   case cAge5: { return("Titan Age"); } }
   return( "Invalid" );
}
int Shift1By(int shiftBy=0) {
   return( xsPow(2, shiftBy) );
}
bool BitAndPowOf2(int ivalue=0, int powerOf2=1) {
   return( ((ivalue / powerOf2) % 2) == 1 );
}

//Ternary operator for ints/floats! Tricky part is remembering to use "," instead of "?:" lol.
//Unlike the real ternary op, both operands will be evaluated (in order to be passed) so keep that in
//mind for high-performance or side-effect-relevant code.
int choose(bool condition=true, int ifTrue=0, int ifFalse=0) {
   if ( condition )
      return( ifTrue );
   return( ifFalse );
}
float negate(float fValue=0.0) {
   return( fValue * -1 );
}
void mutate(int unitid=-1, int mutateTo=-1) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   trMutateSelected( mutateTo );
}
void change_proto(int unitid=-1, int changeTo=-1) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   trUnitChangeProtoUnit( kbGetProtoUnitName(changeTo) );
}
void convert(int unitid=-1, int toPlayer=-1) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   trUnitConvert( toPlayer );
}
float damage_unit(int unitid=-1, float fValue=1.0) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   trDamageUnit( fValue );
   return( fValue );
}
void destroy(int unitid=-1) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   trUnitChangeProtoUnit( "Sling Stone" );
}
void teleport(int unitid=-1, vector toLoc=cOriginVector) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   trUnitTeleport( xsVectorGetX(toLoc), xsVectorGetY(toLoc), xsVectorGetZ(toLoc) );
}
void damage_units_in_area(int centerid=-1, int playerid=-1, int unitType=-1, float fRange=0.0, float fRate=0.0) {
   trUnitSelectClear();
   trUnitSelectByID( centerid );
   trDamageUnitsInArea( playerid, kbGetUnitTypeName(unitType), fRange, fRate );
}
int percent_damaged(int unitid=-1) {
   trUnitSelectClear();
   trUnitSelectByID( unitid );
   return( trUnitPercentDamaged() );
}
int percent_complete(int buildingid=-1) {
   trUnitSelectClear();
   trUnitSelectByID( buildingid );
   return( trUnitPercentComplete() );
}
//Pass -1 results (default val) to execute the query. Else pass number results (means query is done executing).
int destroy_results(int qid=-1, int startAt=0, int results=-1)
{
   if ( results < 0 )
      results = kbUnitQueryExecute( qid );
   for(i=startAt; < results) {
      trUnitSelectClear();
      trUnitSelectByID( kbUnitQueryGetResult(qid, i) );
      trUnitChangeProtoUnit( "Sling Stone" );
   }
   if ( startAt <= 0 )
      kbUnitQueryResetResults( qid );
   return( results );
}
int transform_results(int qid=-1, int unitTypeID=-1, int transfromTo=-1, bool doMutate=false)
{
   if ( unitTypeID >= 0 )
      kbUnitQuerySetUnitType( qid, unitTypeID );
   int results = kbUnitQueryExecute( qid );
   if ( doMutate )
      for(i=0; < results) {
         trUnitSelectClear();
         trUnitSelectByID( kbUnitQueryGetResult(qid, i) );
         trMutateSelected( transfromTo );
      }
   else {
      string protoName = kbGetProtoUnitName( transfromTo );
      for(i=0; < results) {
         trUnitSelectClear();
         trUnitSelectByID( kbUnitQueryGetResult(qid, i) );
         trUnitChangeProtoUnit( protoName );
      }
   }
   kbUnitQueryResetResults( qid );
   return( results );
}
int damage_results(int qid=-1, int results=-1, float fRate=1.0)
{
   if ( results < 0 )
      results = kbUnitQueryExecute( qid );
   for(i=0; < results) {
      trUnitSelectClear();
      trUnitSelectByID( kbUnitQueryGetResult(qid, i) );
      trDamageUnit( fRate );
   }
   kbUnitQueryResetResults( qid );
   return( results );
}
void damage_results_range(int qid=-1, float fRate=1.0, int beginn=0, int endd=0, float fMinHP=-1.0)
{
   if ( fMinHP <= 0.0 )
      for(i=beginn; < endd) {
         trUnitSelectClear();
         trUnitSelectByID( kbUnitQueryGetResult(qid, i) );
         trDamageUnit( fRate );
      }
   else for(i=beginn; < endd) {
      int unitid = kbUnitQueryGetResult( qid, i );
      if ( kbUnitGetCurrentHitpoints(unitid) < fMinHP )
         continue;
      trUnitSelectClear();
      trUnitSelectByID( unitid );
      trDamageUnit( fRate );
   }
}
//Change a BuildLimit, but have it stop at Min/Max. Returns: What the new limit assigned-to is.
int adjust_buildlimit(int playerid=-1, int protoUnit=-1, int addAmount=1, int minLimit=0, int maxLimit=999999999)
{
   int current = kbGetBuildLimit( playerid, protoUnit );
   if ( addAmount >= 0 )
      addAmount = xsMin(addAmount, maxLimit - current);
   else
      addAmount = xsMax(addAmount, minLimit - current);
   if ( addAmount != 0 )
      trModifyProtounit( kbGetProtoUnitName(protoUnit), playerid, cModifyBuildLimit, addAmount );
   return( current + addAmount );
}
//Neutrals count as allies here (since they cant be attacked and there's no cPlayerRelationNeutral).
int diplomacy(int player1id=-1, int player2id=-1) {
   int retVal = cPlayerRelationEnemy;
   if ( player1id == player2id )
      retVal = cPlayerRelationSelf;
   else if ( trPlayerGetDiplomacy(player1id, player2id) != "enemy" )
      retVal = cPlayerRelationAlly;
   return( retVal );
}

//================================================================================================================================
//Use get_int()/set_int() to appropriate the build-limit of an unused proto!
//Range for buildlimit appears to be [ -1..pow(2,24) ). (Altho inconsistent. I suspect issue is the float in trModifyProtounit().)
//If something is "referenced in techtree" it shouldn't be used for any other purpose even across a different civ.
//Everybody/Shared FakeProtos.
extern const int BASIC_QUERY = cUnitTypeBush;
extern const int NUM_RELICS = cUnitTypeConstructionSmall;  //Referenced in techtree
extern const int ARRAY_ID = cUnitTypeWave;
extern const int POSITION_QRY = cUnitTypeSmoke;
extern const int SCRIPT_TECHS = cUnitTypeCamera;  //Referenced in techtree
extern const int HEALING_SPRING_QRY = cUnitTypeFootprintVillager;
extern const int AI_DATAPLAN = cUnitTypeMiscStairs;
extern const int STARTING_UNITS_TECH = cUnitTypeDustSmall;
extern const int NUM_SACRED_DEER = cUnitTypeOarFootprint;
extern const int ODIN_REGEN_QRY = cUnitTypeLightningStrike;
extern const int BARTER_COOLDOWN = cUnitTypeUIArmyBannerSiege;
//Natives (so also "everybody" lol).
extern const int NATIVES_QRY = cUnitTypeFlamingFootprint;
extern const int STONE_POWER_DELAY = cUnitTypeCinematicBlock;
extern const int SACRED_EAGLE_TRACKER = cUnitTypeIcyFootprint;
extern const int CHICHIMEC_ANIMALS_QRY = cUnitTypeRugs;
extern const int CHICHIMEC_VISION_CASTS = cUnitTypeArrowFlaming;
extern const int CHICHIMEC_VISION_RECHARGE = cUnitTypeLightningCloud;
extern const int HUASTEC_TREE_TYPE = cUnitTypeWaterReeds;  //Referenced in techtree.
extern const int ZAPOTEC_FESTIVAL_REWARD = cUnitTypeLightningRain;
//AZTEC FakeProtos.
extern const int TLATOANI_UNITAURA_QRY = cUnitTypeAxe;  //Config differently per god for his Tlatoani aura.
extern const int TLATOANI_BUILDAURA_QRY = cUnitTypeWhale;
extern const int NEAR_SKULLS_QRY = cUnitTypeBlood;
extern const int ATTACKERS_QRY = cUnitTypeScarabBlood;
extern const int NEARBY_TEMPLES_QRY = cUnitTypeWaypointFlag;
extern const int DEAD_UNIT_QRY = cUnitTypeMoveto;
extern const int INVIS_WORKER_QRY = cUnitTypeTestObject;
extern const int HUITZ_CAPTURE_TIMER = cUnitTypeRunestone;
extern const int SHADOW_SUMMON_TIMER = cUnitTypeFirePit;
extern const int SHADOW_SPAWN_TYPE = cUnitTypeRockSymbols;
extern const int SACRIFICE_REWARD_TRACKER = cUnitTypeStalagmite;
extern const int SACRIFICE_REWARDS_OWED = cUnitTypeWaterLilly;
extern const int CAPTURE_MODE_PRISONERS = cUnitTypeConstructionMedium;
extern const int CAPTURE_MODE_COOLDOWN = cUnitTypeRockSnow;
extern const int PREV_NUM_TEMPLES = cUnitTypeYardstick;
extern const int PREV_NUM_TLALOCANS = cUnitTypeCoralReef;
extern const int INVIS_OBSTRUCT_ID = cUnitTypeIceSheet;
extern const int MICTLAN_SHADOWS_QRY = cUnitTypeArrow;
extern const int FREE_TLATOANIS = cUnitTypeCameraStart;  //Referenced in techtree.
extern const int CAPTURES_BEFORE_COOLDOWN = cUnitTypeTornado;  //Referenced in techtree.
extern const int PREV_DEAD_UNITS_COST = cUnitTypeConstructionExtraLarge;
extern const int CAMAXTLI_BARRACK_TRACKER = cUnitTypeRainfall;  //Referenced in techtree.
extern const int CAMAXTLI_NOBLES_TRACKER = cUnitTypeInfernoFireGround;  //Referenced in techtree.
//MAYA FakeProtos.
extern const int CURRENT_AGE_UNLOCKED = cUnitTypeLightningsparks;
extern const int PROGRESS_TECHS = cUnitTypeCrate;  //Referenced in techtree.
extern const int TOTAL_PROGRESS_TECHS = cUnitTypeConstructionLarge;
extern const int HERO_TWIN_DISTANCE = cUnitTypeRevealer;
extern const int TC_BUILD_MAYA_QRY = cUnitTypeDustLarge;
extern const int MAYA_FISHBOAT_QRY = cUnitTypeCameraMotionArrow;
extern const int RESEARCH_REWARDS = cUnitTypeSplash;  //Referenced in techtree.
extern const int PREV_TRADE_PROFIT = cUnitTypeMarketStall;
extern const int RESEARCH_REWARD_PROTO = cUnitTypeMeteor;
//INCA FakeProtos.
extern const int MITA_LEVEL = cUnitTypeLightningScorch;
extern const int NEW_USHNU_CEREMONY = cUnitTypeCatapultShot;
extern const int PREV_USHNU_CEREMONY = cUnitTypeAttackRevealer;
extern const int PREV_USHNU_DANCERS = cUnitTypePalmStump;
extern const int FAILED_USHNU_CEREMONY = cUnitTypeBallistaShot;
extern const int USHNU_TECHID = cUnitTypePapyrus;
extern const int HEAL_CEREMONY_QRY = cUnitTypeHeroBirth;
extern const int MITA_COOLDOWN = cUnitTypeMiscObjects;
extern const int MYTH_CEREMONY_SPAWN = cUnitTypeDustMedium;
extern const int MYTH_CEREMONY_POINTS = cUnitTypeRevealertoPlayer;
//Left off at: cUnitTypeUIArmyBannerPlain
//Can use: cUnitTypeUIArmyBannerFurled

string get_civ_name(int civid=-1) {
   if ( civid == cTechAge1Tezca )  string retVal = "Tezcatlipoca";
   else if ( civid == cTechAge1Huitz )    retVal = "Huitzilopochtli";
   else if ( civid == cTechAge1Quetz )    retVal = "Quetzalcoatl";
   else if ( civid == cTechAge1Itzam )    retVal = "Itzamana";
   else if ( civid == cTechAge1Chaac )    retVal = "Chaac";
   else if ( civid == cTechAge1AhPuch )   retVal = "Ah Puch";
   else if ( civid == cTechAge1Vira )     retVal = "Viracocha";
   else if ( civid == cTechAge1Inti )     retVal = "Inti";
   else if ( civid == cTechAge1Pacha )    retVal = "Pachamama";
   else if ( civid == cTechAge1Camaxtli ) retVal = "Camaxtli";
   else if ( civid == cCivFuxi )          retVal = "Fu Xi";
   else if ( civid == cCivNuwa )          retVal = "Nu Wa";
   else                                   retVal = kbGetCivName( civid );
   return( retVal );
}
int get_my_civ(void) {
   return( kbProtoUnitGetPopulationCount(cUnitTypeHawk) );
}
int get_my_culture(void) {
   return( kbGetPopCapAddition(xsGetContextPlayer(), cUnitTypeHawk) );
}
int get_civ(int playerid=-1) {
   return( kbGetBuildLimit(playerid, cUnitTypeHawk) );
}
int get_culture(int playerid=-1) {
   return( kbGetPopCapAddition(playerid, cUnitTypeHawk) );
}
void set_civ_and_culture(int playerid=-1, int civid=0, int cultureid=0) {
  //Added the BL one too. Redundant, but it's sometimes helpful to be able to get another player's civ w/o changing context (esp in AIs).
   trModifyProtounit( "Hawk", playerid, cModifyBuildLimit, (civid - kbGetBuildLimit(playerid, cUnitTypeHawk)) );
   trModifyProtounit( "Hawk", playerid, cModifyPopulationCount, (civid - kbProtoUnitGetPopulationCount(cUnitTypeHawk)) );
   trModifyProtounit( "Hawk", playerid, cModifyPopCapAddition, (cultureid - kbGetPopCapAddition(playerid, cUnitTypeHawk)) );
}

int get_int(int fakeProto=-1, int playerid=-1) {
   return( kbGetBuildLimit(playerid, fakeProto) );
}
void set_int(int fakeProto=-1, int ivalue=-1, int playerid=-1) {
   ivalue = ivalue - kbGetBuildLimit(playerid, fakeProto);
   if ( ivalue != 0 )
      trModifyProtounit( kbGetProtoUnitName(fakeProto), playerid, cModifyBuildLimit, ivalue );
}
void add_int(int fakeProto=-1, int ivalue=-1, int playerid=-1) {
   trModifyProtounit( kbGetProtoUnitName(fakeProto), playerid, cModifyBuildLimit, ivalue );
}
void subtract_int(int fakeProto=-1, int ivalue=-1, int playerid=-1) {
   trModifyProtounit( kbGetProtoUnitName(fakeProto), playerid, cModifyBuildLimit, (0 - ivalue) );
}
float get_float(int fakeProto=-1) {
   return( kbProtoUnitGetMaximumVelocity(fakeProto) );
}
void set_float(int fakeProto=-1, float fvalue=0.0) {
   fvalue = fvalue - kbProtoUnitGetMaximumVelocity(fakeProto);
   if ( fvalue != 0.0 )
      trModifyProtounit( kbGetProtoUnitName(fakeProto), xsGetContextPlayer(), cModifySpeed, fvalue );
}
void add_float(int fakeProto=-1, float fvalue=0.0) {
   trModifyProtounit( kbGetProtoUnitName(fakeProto), xsGetContextPlayer(), cModifySpeed, fvalue );
}
void subtract_float(int fakeProto=-1, float fvalue=0.0) {
   trModifyProtounit( kbGetProtoUnitName(fakeProto), xsGetContextPlayer(), cModifySpeed, (0.0 - fvalue) );
}

//=================================================================================================================================
//PRE: Context is set to appropriate player.
int array_create(string arrname="", int length=0) {
   int arrayid = kbUnitPickCreate( arrname );
   kbUnitPickSetDesiredNumberUnitTypes( arrayid, length, 1 );
   kbUnitPickSetAttackUnitType( arrayid, trTimeMS() );  //For get_time_modifier
   return( arrayid );
}
int array_size(int arrid=-1) {
   return( kbUnitPickGetDesiredNumberUnitTypes(arrid) );
}
int array_get(int index=0, int arrid=0) {
   return( kbUnitPickGetDesiredNumberBuildings(arrid, index) );
}
void array_set(int index=0, int value=0, int arrid=0) {
   kbUnitPickSetDesiredNumberBuildings( arrid, index, value );
}
void array_destroy(int arrid=-1) {
   kbUnitPickDestroy( arrid );
}

//These members auto-come with a UnitPick and have the full range of i32 [-pow(2,31)..pow(2,31)) so I'd be a fool not to use them.
//I still miss legit arrays.
//kbUnitPickSet/GetAttackUnitType() RESERVED!
//AZTEC.
void set_tlatoani_id(int ivalue=-1, int arrid=-1) {
   kbUnitPickSetMinimumPop( arrid, ivalue );
}
int get_tlatoani_id(int arrid=-1) {
   return( kbUnitPickGetMinimumPop(arrid) );
}
void set_prev_tlatoani_id(int ivalue=-1, int arrid=-1) {
   kbUnitPickSetMaximumPop( arrid, ivalue );
}
int get_prev_tlatoani_id(int arrid=-1) {
   return( kbUnitPickGetMaximumPop(arrid) );
}
//MAYA.
void set_herotwin1(int ivalue=-1, int arrid=-1) {
   kbUnitPickSetMaximumNumberUnits( arrid, ivalue );
}
int get_herotwin1(int arrid=-1) {
   return( kbUnitPickGetMaximumNumberUnits(arrid) );
}
void set_herotwin2(int ivalue=-1, int arrid=-1) {
   kbUnitPickSetMinimumNumberUnits( arrid, ivalue );
}
int get_herotwin2(int arrid=-1) {
   return( kbUnitPickGetMinimumNumberUnits(arrid) );
}
//INCA.
void set_ushnu_id(int ivalue=-1, int arrid=-1) {
   kbUnitPickSetMinimumPop( arrid, ivalue );
}
int get_ushnu_id(int arrid=-1) {
   return( kbUnitPickGetMinimumPop(arrid) );
}

//=====================================================HELPER FUNCS=================================================================
//"Tactics" are always implemented the same way: A script var storing the ID of the tactic ProtoUnit, build-limits always 1,
//0 for the current tactic, search for all the tactic-protos, then destroy_results() on all but the first result, blah blah.
//So this simplifies everything!
//RETURNS: New tactic.
int TacticChange(int playerid=-1, int tokenid=-1, int scriptVar=-1, int prevTactic=-1000)
{
   if ( prevTactic == -1000 )
      prevTactic = get_int( scriptVar, playerid );
   int newTactic = kbUnitGetProtoUnitID( tokenid );
   if ( newTactic != prevTactic ) {
      add_int( newTactic, -1, playerid );
      add_int( prevTactic, 1, playerid );
      add_int( scriptVar, (newTactic - prevTactic), playerid );
   }
   destroy( tokenid );
   return( newTactic );
}
//Formerly specific to TlalocanSacrificePref, I expect this will see more use ;)
void TacticChangeBool(int playerid=-1, int tokenid=-1, int enablerProto=-1, int offProto=-1, int onProto=-1)
{
   int addToOff = 1;
   if ( kbUnitGetProtoUnitID(tokenid) == onProto )
      trUnforbidProtounit(playerid, kbGetProtoUnitName(enablerProto));
   else {
      trForbidProtounit(playerid, kbGetProtoUnitName(enablerProto));
      addToOff = -1;
   }
   adjust_buildlimit( playerid, offProto, addToOff, 0, 1 );
   adjust_buildlimit( playerid, onProto, negate(addToOff), 0, 1 );
   destroy( tokenid );
}


//This is how AoM's source code *should* grant god powers! This func tries the following:
//1st: Add GP to slot already devoted to that GP (power casting is messy if same power found in > 1 slot, which can happen!)
//2nd: Search for earliest unused slot.
//3rd: Grab earliest slot with a redded-out GP.
//4th: Use the slot specified in the param (lowest pri).
//Granting the GP will fail iff all of the above fail.
//Returns: Slot [0..4] the GP uses were added to, -1 if the grant failed.
/*
int GrantGodPower(int playerid=-1, int gpTechID=-1, int uses=0, int lowPrioritySlot=-1)
{
   int bestSlot = -1;
   int firstOpenSlot = -1;
   int firstRedSlot = -1;
   for(thisSlot=cAge1; <= cAge5) {
      int thisTech = trGetGPData( playerid, cGPDataTechID, thisSlot );
      if ( thisTech == gpTechID ) {
         bestSlot = thisSlot;
         break;
      }
      if ( thisTech == -1 ) {
         firstOpenSlot = thisSlot;
         break;
      }
      if ( firstRedSlot < 0 )
         if ( (trGetGPData(playerid, cGPDataImmediateCount, thisSlot) +
               trGetGPData(playerid, cGPDataRecharges, thisSlot)) <= 0 )
            firstRedSlot = thisSlot;
   }
   if ( bestSlot < 0 ) {
      bestSlot = firstOpenSlot;
   if ( bestSlot < 0 ) {
      bestSlot = firstRedSlot;
   if ( bestSlot < 0 ) {
      bestSlot = lowPrioritySlot;
   }}}
  //trTechGodPowerAtPosition() works better than trSetGPData() with cGPDataRecharges/cGPDataImmediateCount because
  //it respects the power's cooldown timer if applicable. (If using cGPDataRecharges, the "use count light bulbs"
  //don't display for non-Atties.)
   if ( bestSlot >= 0 )
      trTechGodPowerAtPosition( playerid, kbGetTechName(gpTechID), uses, bestSlot+1 );
   return( bestSlot );
}*/

extern const int RELATION_ANY = 99999;
extern const int RELATION_SELF = 100000;
extern const int RELATION_ALLY = 100001;
extern const int RELATION_ENEMY = 100002;
//BOTH RETURN: The QueryID.
int ConfigQuery(int qid=-1, int playerData=-1, int unitTypeID=-1, int uState=-1, bool seeableOnly=true)
{
   static int _qnameid = 0;
   if ( qid < 0 ) {
      qid = kbUnitQueryCreate( "cq"+_qnameid );
      _qnameid++;
   }
   if ( playerData < 100 ) {
      kbUnitQuerySetPlayerRelation( qid, -1 );
      kbUnitQuerySetPlayerID( qid, playerData, false );
   }
   else {
      kbUnitQuerySetPlayerRelation( qid, (playerData - 100000) );
      kbUnitQuerySetPlayerID( qid, -1, false );
   }
   kbUnitQuerySetUnitType( qid, unitTypeID );
   kbUnitQuerySetState( qid, uState );
   kbUnitQuerySetSeeableOnly( qid, seeableOnly );
   return( qid );
}
int CreateQuery(int playerData=-1, int unitTypeID=-1, int uState=-1, bool seeableOnly=true) {
   return( ConfigQuery(-1, playerData, unitTypeID, uState, seeableOnly) );
}

//Since 1 civ now may have multiple new mechanics that grant the same resource, store resources owed in an accumulator and grant at end.
//Formerly, integer grants could freely use trPlayerGrantResources() if they wanted.
//With the introduction of the Mita System (the need for offsetting the total) "Resource Grant" mechanics should always use these!
extern float food_grant = 0.0;
extern float wood_grant = 0.0;
extern float gold_grant = 0.0;
extern float favor_grant = 0.0;
extern int _FarmingVillPop = -1;
extern bool gFullPrisonerRetask = false;

//Returns: cResourceXXX of the highest/lowest resource type, excluding Favor.
int highest_resource_type(void)
{
   int retval = 0;
   float fCompareTo = kbResourceGet( retval );
   for(i=1; <= 2) {
      float fTemp = kbResourceGet(i);
      if ( fTemp > fCompareTo ) {
         fCompareTo = fTemp;
         retval = i;
      }
   }
   return( retval );
}
int lowest_resource_type(void)
{
   int retval = 0;
   float fCompareTo = kbResourceGet( retval );
   for(i=1; <= 2) {
      float fTemp = kbResourceGet(i);
      if ( fTemp < fCompareTo ) {
         fCompareTo = fTemp;
         retval = i;
      }
   }
   return( retval );
}

string GetPlayerName(int playerid=0) {  //For chat messages and such.
   return( "{PlayerNameString("+playerid+",92585)}" );
}
int find_unit(int qid=-1, int unitTypeID=-1) {
   int retval = -1;
   kbUnitQuerySetUnitType( qid, unitTypeID );
   if ( kbUnitQueryExecute(qid) > 0 ) {
      retval = kbUnitQueryGetResult( qid, 0 );
      kbUnitQueryResetResults( qid );
   }
   return( retval );
}
int TRSelectUnimportantUnit(void) {
   static int lastid = 0;
   while( kbUnitGetOwner(lastid) < 0 ) {
      lastid++;
      if ( lastid < 0 )
         lastid = 0;
   }
   trUnitSelectClear();
   trUnitSelectByID( lastid );
   return( lastid );
}
//Less efficent if you just know the resource to add to, but convenient for vars/loops/etc.
void add_resource_grant(int resourceid=-1, float fAmount=0.0) {
   switch( resourceid ) {
   case cResourceGold:  { gold_grant = gold_grant + fAmount; break; }
   case cResourceWood:  { wood_grant = wood_grant + fAmount; break; }
   case cResourceFood:  { food_grant = food_grant + fAmount; break; }
   case cResourceFavor: { favor_grant = favor_grant + fAmount; break; } }
}

//Convenience func for script-techs that reward based on elapsed game time.
float RewardPerMinute(float fPerMinute=1.0, float fMaxMinutes=30.0) {
   float fTimer = trTime();  //Ensure float arithmetic in XS's weird implicit casting rules.
   return( fPerMinute * xsMin(fTimer/60.0, fMaxMinutes) );
}

//Not a high-performance func, but for use when a queryid doesn't exist under the context of playerVantage, but
//it needs to mimic setting PlayerRelation to Ally or Enemy of that player.
int PlayerLoopQuery(int qid=-1, int playerVantage=-1, int relation=-1, int prematureCutOff=-1, int playerStart=1)
{
   int results = 0;
   if ( prematureCutOff < 0 )
      prematureCutOff = 999999999;
   for(loopid=playerStart; < cNumberPlayers) {
      if ( diplomacy(playerVantage, loopid) != relation )
         continue;
      kbUnitQuerySetPlayerID( qid, loopid, false );
      results = kbUnitQueryExecute( qid );
      if ( results >= prematureCutOff )
         break;
   }
   return( results );
}

int _rng_bonus = -1;
//Not thrilling for an RNG, but aiRandInt() when used by human players causes OOS.
int RNG(int modulo=100) {
   static int remember = 1001;
   remember = remember + trTimeMS() + _rng_bonus;
   if ( remember < 0 )
      remember = _rng_bonus;
   return( remember % modulo );
}
//This one is much slower (avoid for high-performance code) but produces a much better result.
int RNG_Slow(int modulo=100) {
   trQuestVarSetFromRand( "_rng", 0, modulo-1, true );
   modulo = trQuestVarGet( "_rng" );
   return( modulo );
}
void InitMath(void) {
   _rng_bonus = 1000 + RNG_Slow(2000);
   if ( (_rng_bonus % 2) == 0 )  //Odd numbers are better.
      _rng_bonus++;
}

int MultiUnitTypeQuery(int qid=-1, int unitType1=-1, int unitType2=-1, int unitType3=-1, int unitType4=-1)
{
   kbUnitQuerySetUnitType( qid, unitType1 );
   int results = kbUnitQueryExecute( qid );
   if ( unitType2 >= 0 ) { kbUnitQuerySetUnitType(qid, unitType2); results = kbUnitQueryExecute(qid);
   if ( unitType3 >= 0 ) { kbUnitQuerySetUnitType(qid, unitType3); results = kbUnitQueryExecute(qid);
   if ( unitType4 >= 0 ) { kbUnitQuerySetUnitType(qid, unitType4); results = kbUnitQueryExecute(qid); }}}
   return( results );
}
//A versatile but not very efficent func, this should only be used as 1x events (ie ScriptTechs) anyway.
int RevealLOS(int forPlayer=0, int revealThisPlayerData=-1, int unitType1=-1, int unitType2=-1, int unitType3=-1, int unitType4=-1)
{
   int prev = xsGetContextPlayer();
   xsSetContextPlayer( 0 );
   kbLookAtAllUnitsOnMap();

   int startAt = -1;
   int qid = kbUnitQueryCreate( "RevealQry" );
   if ( revealThisPlayerData < 100 )  //A playerID.
      kbUnitQuerySetPlayerID( qid, revealThisPlayerData, false );
   else if ( revealThisPlayerData == RELATION_ANY )  //Any.
      kbUnitQuerySetPlayerRelation( qid, cPlayerRelationAny );
   else {
      revealThisPlayerData = revealThisPlayerData - 100000;
      startAt = choose(revealThisPlayerData == cPlayerRelationEnemy, 0, 1);
   }

   int results = 0;
   if ( startAt == -1 )  //No loop.
      results = MultiUnitTypeQuery( qid, unitType1, unitType2, unitType3, unitType4 );
   else for(playerid=startAt; < cNumberPlayers) {
      if ( diplomacy(forPlayer, playerid) != revealThisPlayerData )
         continue;
      kbUnitQuerySetPlayerID( qid, playerid, false );
      results = MultiUnitTypeQuery( qid, unitType1, unitType2, unitType3, unitType4 );
   }
   string sArmyName = forPlayer+",0";
   for(i=0; < results) {
      vector pos = kbUnitGetPosition( kbUnitQueryGetResult(qid, i) );
      trArmyDispatch( sArmyName, "Attack Revealer", 1, xsVectorGetX(pos), xsVectorGetY(pos), xsVectorGetZ(pos), 0, true );
   }
   kbUnitQueryResetResults( qid );

   xsSetContextPlayer( prev );
   return( results );
}

//Dumb! aiResourceCheat() fails in multiplayer (b/c "cheat" is a scary word?) and trPlayerGrantResources()'s
//amount param is an int. So we now have to do this in multiplayer whenever accumulating into a float is needed.
void _ResourceGrantAccumulator(int playerid=-1, int resourceid=-1, float fValue=0.0) {
   int fakeProto = cUnitTypeRevealer + resourceid;
   float fCurrent = kbProtoUnitGetMaximumVelocity( fakeProto );
   float fTotal = fCurrent + fValue;
   int iTotal = fTotal;
   if ( iTotal != 0 ) {
      trPlayerGrantResources( playerid, ResourceName(resourceid), iTotal );
      fValue = (fTotal - iTotal) - fCurrent;
   }
   trModifyProtounit( kbGetProtoUnitName(fakeProto), playerid, cModifySpeed, fValue );
}
void GrantResourcesOwed(int playerid=-1)
{
   if ( aiIsMultiplayer() == false ) {
      if ( food_grant != 0.0 )
         aiResourceCheat( playerid, cResourceFood, food_grant );
      if ( wood_grant != 0.0 )
         aiResourceCheat( playerid, cResourceWood, wood_grant );
      if ( gold_grant != 0.0 )
         aiResourceCheat( playerid, cResourceGold, gold_grant );
      if ( favor_grant != 0.0 )
         aiResourceCheat( playerid, cResourceFavor, favor_grant );
   }
   else {
      if ( food_grant != 0.0 )
         _ResourceGrantAccumulator( playerid, cResourceFood, food_grant );
      if ( wood_grant != 0.0 )
         _ResourceGrantAccumulator( playerid, cResourceWood, wood_grant );
      if ( gold_grant != 0.0 )
         _ResourceGrantAccumulator( playerid, cResourceGold, gold_grant );
      if ( favor_grant != 0.0 )
         _ResourceGrantAccumulator( playerid, cResourceFavor, favor_grant );
   }
   food_grant = 0.0;
   wood_grant = 0.0;
   gold_grant = 0.0;
   favor_grant = 0.0;
   _FarmingVillPop = -1;
}

void copy_cost_trainpoints(int playerid=-1, int toProto=-1, int fromProto=-1) {
   string toName = kbGetProtoUnitName( toProto );
   for(resource=0; <= cResourceFavor)
      trModifyProtounit( toName, playerid, (cModifyCostGold + resource), kbUnitCostPerResource(fromProto, resource) - kbUnitCostPerResource(toProto, resource) );
   trModifyProtounit( toName, playerid, cModifyTrainPoints, kbProtoUnitGetTrainPoints(fromProto) - kbProtoUnitGetTrainPoints(toProto) );
}
void UnitCreate(int playerid=-1, int protoUnit=-1, int count=0, vector loc=cInvalidVector, int rotation=90) {
   trArmyDispatch( playerid+",0", kbGetProtoUnitName(protoUnit), count, xsVectorGetX(loc), xsVectorGetY(loc), xsVectorGetZ(loc), rotation, true );
}

void TerrainChange(vector Location=cInvalidVector, float fRadius=3.0, int tType=0, int tSubType=0, bool recalcWalkable=false, bool isVolcanoGP=false)
{
  //Paint new terrain in a circle around dead volcano.
  //BIG thank you to nottud's trigger packs for helping me figure out how these terrain funcs work!
   float tempX0 = xsVectorGetX(Location) / 2;
   float tempZ0 = xsVectorGetZ(Location) / 2;
   float tempMinX = xsMax( negate(tempX0), negate(fRadius) );
   float tempMinZ = xsMax( negate(tempZ0), negate(fRadius) );
   float tempMaxX = xsMin(kbGetMapXSize() / 2 - tempX0 - 1, fRadius);
   float tempMaxZ = xsMin(kbGetMapZSize() / 2 - tempZ0 - 1, fRadius);
   float tempRadiusCheck = (fRadius * fRadius) + fRadius;
   float fZ = tempMaxZ;
   while( fZ >= tempMinZ ) {
      float fX = tempMaxX;
      while( fX >= tempMinX ) {
	   if ( tempRadiusCheck >= (fX*fX + fZ*fZ) )
            trPaintTerrain( fX+tempX0, fZ+tempZ0, fX+tempX0, fZ+tempZ0, tType,tSubType, false );
         fX = fX - 1.0;
	}
      fZ = fZ - 1.0;
   }

   if ( isVolcanoGP ) {  //Make sure TCs remain buildable!
      const float TCRadius = 5.0;
      int qid = CreateQuery( RELATION_ANY, cUnitTypeAbstractSettlement, cUnitStateAny, false );
      kbUnitQuerySetPosition( qid, Location );
      kbUnitQuerySetMaximumDistance( qid, (fRadius * 2.0) + 6.95 );
      int settlements = kbUnitQueryExecute( qid );
      for(i=0; < settlements) {
         Location = kbUnitGetPosition( kbUnitQueryGetResult(qid, i) );
         vector loc1 = Location + vector(TCRadius,0,TCRadius);
         vector loc2 = Location - vector(TCRadius,0,TCRadius);
         tempX0 = xsMin(xsVectorGetX(loc1), xsVectorGetX(loc2)) / 2.0;
	   tempZ0 = xsMin(xsVectorGetZ(loc1), xsVectorGetZ(loc2)) / 2.0;
	   float tempX1 = xsMax(xsVectorGetX(loc1), xsVectorGetX(loc2)) / 2.0;
	   float tempZ1 = xsMax(xsVectorGetZ(loc1), xsVectorGetZ(loc2)) / 2.0;
        //3,0 = UnderwaterRockA. 0,84 = HadesBuildable1
	   trPaintTerrain( tempX0, tempZ0, tempX1, tempZ1, 3,0, false );
      }
      kbUnitQueryDestroy( qid );
      recalcWalkable = true;
   }
   if ( recalcWalkable )  //Update passability.
      trPaintTerrain( 0, 0, 0, 0, trGetTerrainType(0,0), trGetTerrainSubType(0,0), true );
}

//RETURNS: Num units spawned.
//PRE: Make sure Building-type can garrison the unittype!
//BuildingId is valid and player-owned (allied maybe, but that would be dumb).
//Pass 0s (not -1s) for protos/quantities to ignore.
int BannerArmySpawn(int playerid=-1, int buildingid=-1, bool doEject=false, int unitType1=0, int qty1=1,
                    int unitType2=0, int qty2=0, int unitType3=0, int qty3=0) {
   int nextid = trGetNextUnitScenarioNameNumber();
   int range1 = qty1;
   int range2 = range1;
   int range3 = range1;
   vector loc = kbUnitGetPosition( buildingid );
   float fX = xsVectorGetX( loc );
   float fY = xsVectorGetY( loc );
   float fZ = xsVectorGetZ( loc );
   string sBuildingName = trGetUnitScenarioName( buildingid );
   string sArmyName = playerid+",0";

   if ( (unitType2 > 0) && (qty2 > 0) ) {
      range2 = range1 + qty2;
      range3 = range2;
   }
   if ( (unitType3 > 0) && (qty3 > 0) )
      range3 = range2 + qty3;
   if ( doEject ) {
      string sBuildingType = kbGetProtoUnitName( kbUnitGetProtoUnitID(buildingid) );
      trModifyProtounit( sBuildingType, playerid, cModifyMaxContained, range3 );
   }
   trArmyDispatch( sArmyName, kbGetProtoUnitName(unitType1), qty1, fX,fY,fZ, 180, true );
   if ( range2 > range1 )
      trArmyDispatch( sArmyName, kbGetProtoUnitName(unitType2), qty2, fX,fY,fZ, 180, true );
   if ( range3 > range2 )
      trArmyDispatch( sArmyName, kbGetProtoUnitName(unitType3), qty3, fX,fY,fZ, 180, true );
   for(i=0; < range3) {
      trUnitSelectClear();
      trUnitSelect( (nextid+i)+"" );
      trImmediateUnitGarrison( sBuildingName );
      if ( doEject )
         aiTaskUnitEject( trGetSelectedUnitID(0), buildingid );
   }
   if ( doEject )
      trModifyProtounit( sBuildingType, playerid, cModifyMaxContained, negate(range3) );
   return( range3 );
}

//A potentially painful, cpu-intensive calculation... Tried to optimize as much as possible.
int AttackIncreaseAura(int playerid=-1, int resultsQid=-1, int actionid=-1, float fPercent=0.5, int startAt=0)
{
/*  //COMMENT OUT UNTIL READY!
   kbUnitQuerySetActionType( resultsQid, actionid );
   int results = kbUnitQueryExecute( resultsQid );
   for(i=startAt; < results) {
      int attackerid = kbUnitQueryGetResult( resultsQid, i );
      if ( kbUnitGetAnimationActionType(attackerid) != actionid )
         continue;
      int attackerType = kbUnitGetProtoUnitID( attackerid );
      int targetid = kbUnitGetTargetUnitID( attackerid );
      int targetType = kbUnitGetProtoUnitID( targetid );
      float fTotal = 0.0;
      float fDamageBonus = kbProtoUnitGetDamage( attackerType, actionid, 0, targetType );
      float fHackDamage = kbProtoUnitGetDamageOfType( attackerType, actionid, cDamageHack );
      float fPierceDamage = kbProtoUnitGetDamageOfType( attackerType, actionid, cDamagePierce );
      float fCrushDamage = kbProtoUnitGetDamageOfType( attackerType, actionid, cDamageCrush );
      xsSetContextPlayer( kbUnitGetOwner(targetid) );
      if ( fHackDamage > 0.0 )
         fTotal = xsMax(fHackDamage * fDamageBonus * (1.00-kbProtoUnitGetArmorOfType(targetType, cDamageHack)), 1.0);
      if ( fPierceDamage > 0.0 )
         fTotal = fTotal + xsMax(fPierceDamage * fDamageBonus * (1.00-kbProtoUnitGetArmorOfType(targetType, cDamagePierce)), 1.0);
      if ( fCrushDamage > 0.0 )
         fTotal = fTotal + xsMax(fCrushDamage * fDamageBonus * (1.00-kbProtoUnitGetArmorOfType(targetType, cDamageCrush)), 1.0);
      xsSetContextPlayer( playerid );
      damage_unit( targetid, (fPercent * fTotal) );
   }
   return( results );
*/
   return( 0 );
}

void ResourceExchange(int playerid=-1, int fromResource=0, int toResource=0, float fMultiplier=1.0, int MaxSwap=0)
{
   string sFromResource = ResourceName( fromResource );
   int amount = trPlayerResourceCount( playerid, sFromResource );
   if ( MaxSwap > 0 )
      amount = xsMin( MaxSwap, amount );
   amount = xsRound( fMultiplier * amount );
   trPlayerGrantResources( playerid, sFromResource, IntMin() );
   trPlayerGrantResources( playerid, ResourceName(toResource), amount );
   add_float( MITA_ACCUMULATOR, amount );
}

int GetAIDataPlan(int playerid=-1)
{
   int planid = get_int( AI_DATAPLAN, playerid );
   if ( planid < 0 )
      if ( trTimeMS() >= AI_NEWCIV_CHOOSE_TIME_MS )  //It's not going to be found (a different AI being used).
         planid = ERR_AI_PLAN_TIMEOUT;
      else {
         int numPlans = aiPlanGetNumber( 8, -1, false );  //8=cPlanData.
         for(i=0; < numPlans) {
            int thisPlan = aiPlanGetIDByIndex( 8, -1, false, i );
            string planName = aiPlanGetName( thisPlan );
            if ( (planName == "rotg:"+thisPlan) || (planName == "oldc:"+thisPlan) ) {
               planid = thisPlan;
               break;
            }
         }
         if ( planid >= 0 )
            set_int( AI_DATAPLAN, planid, playerid );
      }
   return( planid );
}

//Even with min/max interval set to 1 and priority to 100, there is still no way *ensure* the new civ rules run exactly 1 sec apart.
//This multiplier is therefore necessary to scale mechanics that depend crucially on per-second execution (ie, Gathering/Healing) to
//prevent losses from accumulating as game time elapses.
float get_time_modifier(int arrayid=-1) {
   int currentMS = trTimeMS();
   int previousMS = kbUnitPickGetAttackUnitType( arrayid );
   kbUnitPickSetAttackUnitType( arrayid, currentMS );
   return( 0.001 * (currentMS - previousMS) );
}
//Streamlines the turning of a post-game stat into an in-game mechanic! Since this is becoming more common.
//Reserve a FakeProto/"scriptVar" that starts at -1 (ie, mechanic not initialized) for it.
int StatDifference(int playerid=-1, int statVar=0, int scriptVar=-1) {
   int script_value = get_int( scriptVar, playerid );
   int update_value = trGetStatValue(playerid, statVar) - script_value;
   int retval = update_value;
   if ( script_value == -1 )  //Uninitialized.
      retval = 0;
   if ( update_value != 0 )
      add_int( scriptVar, update_value, playerid );
   return( retval );
}
int GetFarmingVillPop(int qid=-1)
{
  if ( _FarmingVillPop < 0 ) {
   int total = 0;
   kbUnitQuerySetUnitType( qid, cUnitTypeAbstractFarm );
   int results = kbUnitQueryExecute( qid );
   for(i=0; < results) {
      int unitid = kbUnitQueryGetResult( qid, i );
      int kbworkers = kbUnitGetNumberWorkers( unitid );
      for(i2=0; < kbworkers) {
         int workerid = kbUnitGetWorkerID( unitid, i2 );
         if ( kbUnitGetActionType(workerid) == cActionGather )
            total = total + kbProtoUnitGetPopulationCount(kbUnitGetProtoUnitID(workerid));
      }
   }
   kbUnitQueryResetResults( qid );
   _FarmingVillPop = total;
  }
  return( _FarmingVillPop );
}

mutable void reset_nearest_extmech_qid(int nearestQid=-1) { }

include "rotg_buildingbonus.xs";
include "rotg_capturing.xs";
include "rotg_bigheroes.xs";
include "rotg_natives.xs";  //AFTER a lot of forward-declarations.

//=========================================END HELPER FUNCS====================================================================
rule CivSetup1 highFrequency active
{
   if ( trTimeMS() < 100 )  //Give scenarios a moment to SetTechStatus, etc...
      return;
   bool setupStillNeeded = false;
   for(playerid=1; < cNumberPlayers) {
      xsSetContextPlayer( playerid );
      if ( kbGetCiv() != cCivShennong )
         continue;
      if ( kbGetTechStatus(cTechAge1Shennong) >= cTechStatusActive )
         continue;
     //Don't check 'Age1Meso' here b/c that could indicate RandomCiv was used and CivSelectFunc may still have work to do!
      if ( kbGetTechStatus(cTechAge1Aztec) >= cTechStatusActive )
         continue;
      if ( kbGetTechStatus(cTechAge1Maya) >= cTechStatusActive )
         continue;
      if ( kbGetTechStatus(cTechAge1Inca) >= cTechStatusActive )
         continue;
      setupStillNeeded = true;
      break;
   }
   if ( setupStillNeeded == false ) {
      xsDisableSelf();
      xsEnableRule( "CivSetup2" );
   }
}
rule CivSetup2 highFrequency inactive
{
   bool setupStillNeeded = false;
   for(playerid=1; < cNumberPlayers) {
      if ( get_int(STARTING_UNITS_TECH, playerid) >= 0 )
         continue;
      xsSetContextPlayer( playerid );
      int startingTech = cTechStartingUnitsGreek;
      int deathmatchTech = cTechDeathmatchGreek;
      if ( kbGetTechStatus(cTechAge1Aztec) >= cTechStatusActive ) {
         startingTech = choose(kbGetTechStatus(cTechAge1Quetz)>=cTechStatusActive, cTechStartingUnitsQuetz, cTechStartingUnitsAztec);
         deathmatchTech = cTechDeathmatchAztec;
      }
      else if ( kbGetTechStatus(cTechAge1Maya) >= cTechStatusActive ) {
         startingTech = cTechStartingUnitsMaya;
         deathmatchTech = cTechDeathmatchAll;  //TODO: Fix when done!
      }
      else if ( kbGetTechStatus(cTechAge1Inca) >= cTechStatusActive ) {
         startingTech = cTechStartingUnitsInca;
         deathmatchTech = cTechDeathmatchAll;  //TODO: Fix when done!
      }
      else if ( get_my_culture() == cCultureEgyptian ) {
         startingTech = cTechStartingUnitsEgyptian;
         deathmatchTech = cTechDeathmatchEgyptian;
      }
      else if ( get_my_culture() == cCultureNorse ) {
         startingTech = choose(get_my_civ() == cCivThor, cTechStartingUnitsThor, cTechStartingUnitsNorse);
         deathmatchTech = cTechDeathmatchNorse;
      }
      else if ( get_my_culture() == cCultureAtlantean ) {
         startingTech = cTechStartingUnitsAtlantean;
         deathmatchTech = cTechDeathmatchAtlantean;
      }
      else if ( get_my_culture() == cCultureChinese ) {
         startingTech = cTechStartingUnitsChinese;
         deathmatchTech = cTechDeathmatchChinese;
      }

      if ( (aiGetGameMode() == cGameModeDeathmatch) && (kbGetTechStatus(deathmatchTech) < cTechStatusActive) )
         trTechSetStatus( playerid, deathmatchTech, cTechStatusActive );
     //If triggers forbid us from starting units, or we've already died (even possible this early?).
      bool doStartup = false;
      if ( (kbGetTechStatus(startingTech) == cTechStatusUnobtainable) || kbHasPlayerLost(playerid) )
         doStartup = true;
      else if ( kbUnitCount(playerid, cUnitTypeSettlementsThatTrainVillagers, cUnitStateAlive) > 0 ) {
         trTechSetStatus( playerid, startingTech, cTechStatusActive );
         doStartup = true;
      }
      if ( doStartup ) {
         set_int( STARTING_UNITS_TECH, startingTech, playerid );
         if ( deathmatchTech == cTechDeathmatchAztec )  //Needed for Sea of Worms.
            mutate( find_unit(get_int(BASIC_QUERY, playerid), cUnitTypeAbstractFortress), cUnitTypeHillFort );
         continue;
      }
      setupStillNeeded = true;
   }
   if ( setupStillNeeded == false )
      xsDisableSelf();
   else if ( trTime() >= 11 )
      xsSetRuleMinIntervalSelf( 1 );
}

void RuleFuncImmediateSetup(int playerid=-1, int basicQid=-1, int cultureid=-1)
{
  //kbLookAtAllUnitsOnMap() does not give undeserved LOS anywhere. It updates the player's KB with info on ALL entities on the map.
  //Anything in the LOS of an AI is KB-updated automatically. LookAt would give info on fog/blackmap units, ie an unfair advantage.
  //But for humans/gaia, some funcs will return wrong result unless LookAt is called first.
   xsSetContextPlayer( playerid );
   if ( kbIsPlayerHuman(playerid) ) {
      kbLookAtAllUnitsOnMap();
      return;
   }

  //Check if there are any pending commands.
   int planid = GetAIDataPlan( playerid );
   if ( aiPlanGetAllowUnderAttackResponse(planid) == false )
      return;
   aiPlanSetAllowUnderAttackResponse( planid, false );

  //Wish I had discovered this much earlier, but AIs can directly communicate with the script using data plans!
   const int STRING_VARS = 0;
   const int SENDOUT_TO = 1;
   const int CAPTURE_MODE_SET = 3;

  //Send Custom Chats!
   int number = aiPlanGetNumberUserVariableValues( planid, STRING_VARS );
   for(i=0; < number) {
      string sText = aiPlanGetUserVariableString( planid, STRING_VARS, i );
      if ( sText == "" ) continue;
      sText = GetPlayerName(playerid)+": "+sText;

      int sendTo = aiPlanGetUserVariableInt( planid, SENDOUT_TO, i );
      if ( sendTo < 5000 ) {  //A player ID.
         trChatSendSpoofedToPlayer( playerid, sendTo, sText );
         continue;
      }
      sendTo = sendTo - 10000;
      if ( sendTo == cPlayerRelationAny )  //Else a relation.
         trChatSendSpoofed( playerid, sText );
      else for(i2=1; < cNumberPlayers)
         if ( diplomacy(playerid, i2) == sendTo )
            trChatSendSpoofedToPlayer( playerid, i2, sText );
   }
   aiPlanSetNumberUserVariableValues( planid, STRING_VARS, xsMin(3,number), true );

   if ( cultureid == cTechAge1Aztec ) {
     //Capture Mode Toggle.
      number = aiPlanGetUserVariableInt( planid, CAPTURE_MODE_SET, 0 );
      if ( number >= 0 ) {
         if (number == 0) ExitCaptureMode( playerid );
         else             EnterCaptureMode( playerid );
         aiPlanSetUserVariableInt( planid, CAPTURE_MODE_SET, 0, -1 );
      }
   }
//   else if ( cultureid == cTechAge1Maya ) {  //MAYA.
//   }
//   else if ( cultureid == cTechAge1Inca ) {  //INCA.
//   }
}

//============================================================================================================
int _BartersUntilCooldown(int ageid=-1) {  //NOTE: This func must support invalid age-ids!
   return( choose(ageid < cAge4, 3, -1) );
}
void UpdateBarterLimitOnAgeUp(int playerid=-1, int ageid=-1) {
   if ( get_int(BARTER_COOLDOWN, playerid) > 0 )  //When cooldown finishes, it will correctly update the limits.
      return;
   int difference = _BartersUntilCooldown(ageid) - _BartersUntilCooldown(ageid-1);
   if ( difference != 0 ) {
      add_int( cUnitTypeFoodToWood, difference, playerid );
      add_int( cUnitTypeWoodToFood, difference, playerid );
   }
}
void _BarterCooldown(int playerid=-1, int toFoods=0, int toWoods=0) {
   int toFoodLimit = kbGetBuildLimit( playerid, cUnitTypeWoodToFood );
   int toWoodLimit = kbGetBuildLimit( playerid, cUnitTypeFoodToWood );
   if ( (toFoods >= toFoodLimit) || (toWoods >= toWoodLimit) ) {
      trChatSendToPlayer( 0, playerid, "Bartering ability is in cooldown." );
      subtract_int( cUnitTypeFoodToWood, toWoodLimit, playerid );
      subtract_int( cUnitTypeWoodToFood, toFoodLimit, playerid );
      add_int( BARTER_COOLDOWN, 2, playerid );
      xsEnableRule( "BarterRecharge"+playerid );
   }
   else {
      add_int( cUnitTypeWoodToFood, (toWoods - toFoods), playerid );
      add_int( cUnitTypeFoodToWood, (toFoods - toWoods), playerid );
   }
}

//RETURNS: Total num of food/wood swap token units in queue, even if not all are processed.
//This allows the RetVal to be used in short-circuit calcs in the SpecialPowersTokensXXX() funcs.
int TianguisExchange(int playerid=-1, int qid=-1, bool addToMitaOffset=false)
{
   static int _meso_market_techid = cTechMesoMarketStart;
   static int _meso_food_remaining = TIANGUIS_PRICE_INTERVAL;  //"Remaining" until price change.
   static int _meso_wood_remaining = TIANGUIS_PRICE_INTERVAL;
   static int _meso_food_reward = 160;  //Sync with stringtable desc.
   static int _meso_wood_reward = 160;
   int toFoods = kbUnitCount( playerid, cUnitTypeWoodToFood, cUnitStateBuilding );
   int toWoods = kbUnitCount( playerid, cUnitTypeFoodToWood, cUnitStateBuilding );
   int barterTokens = toFoods + toWoods;
   if ( barterTokens <= 0 )
      return( 0 );
   transform_results( qid, cUnitTypeMarketMeso, cUnitTypeMarketMeso, true );  //Clear production queue (and everything else :'( ).

  //If you queue both at the same time, assume it was an accident and go with the net-difference.
   toFoods = xsMax( 0, toFoods - toWoods );
   toWoods = xsMax( 0, toWoods - toFoods );
   if ( (toFoods + toWoods) == 0 )
      return( barterTokens );
   if ( _BartersUntilCooldown(kbGetAge()) >= 0 )
      _BarterCooldown( playerid, toFoods, toWoods );

   int foodGrant = 0;
   int woodGrant = 0;
   int techStart = _meso_market_techid;
   for(i=0; < toFoods) {
      foodGrant = foodGrant + _meso_food_reward;
      if ( _meso_market_techid == cTechMesoMarketG2 )
         continue;
      _meso_food_remaining--;
      _meso_wood_remaining++;
      if ( _meso_food_remaining > 0 )
         continue;
      _meso_food_remaining = TIANGUIS_PRICE_INTERVAL;
      _meso_wood_remaining = TIANGUIS_PRICE_INTERVAL;
      _meso_food_reward = _meso_food_reward - 10;
      _meso_wood_reward = _meso_wood_reward + 10;
      _meso_market_techid--;
   }
   for(i=0; < toWoods) {
      woodGrant = woodGrant + _meso_wood_reward;
      if ( _meso_market_techid == cTechMesoMarketO )
         continue;
      _meso_wood_remaining--;
      _meso_food_remaining++;
      if ( _meso_wood_remaining > 0 )
         continue;
      _meso_wood_remaining = TIANGUIS_PRICE_INTERVAL;
      _meso_food_remaining = TIANGUIS_PRICE_INTERVAL;
      _meso_wood_reward = _meso_wood_reward - 10;
      _meso_food_reward = _meso_food_reward + 10;
      _meso_market_techid++;
   }
   if ( _meso_market_techid != techStart ) {  //Price change!
      for(loopid=1; < cNumberPlayers) {
         trTechSetStatus( loopid, techStart, cTechStatusUnobtainable );
         trTechSetStatus( loopid, _meso_market_techid, cTechStatusActive );
      }
   }

   woodGrant = woodGrant - (kbUnitCostPerResource(cUnitTypeWoodToFood, cResourceWood) * toFoods);
   foodGrant = foodGrant - (kbUnitCostPerResource(cUnitTypeFoodToWood, cResourceFood) * toWoods);
   trPlayerGrantResources( playerid, "Food", foodGrant );
   trPlayerGrantResources( playerid, "Wood", woodGrant );
   if ( addToMitaOffset )
      add_float( MITA_ACCUMULATOR, xsMax(0,foodGrant) + xsMax(0,woodGrant) );
   return( barterTokens );
}

void BarterReFunc(int playerid=-1) {
   add_int( BARTER_COOLDOWN, -1, playerid );
   if ( get_int(BARTER_COOLDOWN, playerid) <= 0 ) {
      int threshold = _BartersUntilCooldown( kbGetAgeForPlayer(playerid) );
      set_int( cUnitTypeFoodToWood, threshold, playerid );
      set_int( cUnitTypeWoodToFood, threshold, playerid );
      xsDisableRule( "BarterRecharge"+playerid );
   }
}
rule BarterRecharge1  minInterval 60 maxInterval 60 group _dod1 runImmediately  { BarterReFunc(1); }
rule BarterRecharge2  minInterval 60 maxInterval 60 group _dod2 runImmediately  { BarterReFunc(2); }
rule BarterRecharge3  minInterval 60 maxInterval 60 group _dod3 runImmediately  { BarterReFunc(3); }
rule BarterRecharge4  minInterval 60 maxInterval 60 group _dod4 runImmediately  { BarterReFunc(4); }
rule BarterRecharge5  minInterval 60 maxInterval 60 group _dod5 runImmediately  { BarterReFunc(5); }
rule BarterRecharge6  minInterval 60 maxInterval 60 group _dod6 runImmediately  { BarterReFunc(6); }
rule BarterRecharge7  minInterval 60 maxInterval 60 group _dod7 runImmediately  { BarterReFunc(7); }
rule BarterRecharge8  minInterval 60 maxInterval 60 group _dod8 runImmediately  { BarterReFunc(8); }
rule BarterRecharge9  minInterval 60 maxInterval 60 group _dod9 runImmediately  { BarterReFunc(9); }
rule BarterRecharge10 minInterval 60 maxInterval 60 group _dod10 runImmediately { BarterReFunc(10); }
rule BarterRecharge11 minInterval 60 maxInterval 60 group _dod11 runImmediately { BarterReFunc(11); }
rule BarterRecharge12 minInterval 60 maxInterval 60 group _dod12 runImmediately { BarterReFunc(12); }

int NewCultureTitanFix(int playerid=-1, int basicQid=-1, int titanType=0)
{
   int number = transform_results( basicQid, cUnitTypeTitanAtlantean, titanType );
   if ( number > 0 )
      if ( kbUnitGetMovementType(titanType) != kbUnitGetMovementType(cUnitTypeTitanAtlantean) )
         trModifyProtounit( kbGetProtoUnitName(titanType), playerid, cModifyMovementType, cMovementTypeAmphibious );
   return( number );
}

//==========================================================================================================================
void _NativeUnitSpawn(int playerid=-1, int qid=-1, int buildingType=-1, int spawnThis1=0, int qty1=0,
                      int spawnThis2=0, int qty2=0) {
   int buildingid = find_unit( qid, buildingType );
   if ( buildingid >= 0 )
      BannerArmySpawn( playerid, buildingid, true, spawnThis1, qty1, spawnThis2, qty2 );
}
void _AgeAdvanceHandler(int playerid=-1, int ageid=-1)  //NOT called (currently) for Ages 1 or 5.
{
   UpdateBarterLimitOnAgeUp( playerid, ageid );
   NativeAgeUpHandler( playerid, ageid );

   if ( ageid == cAge2 )
      set_int( HUITZ_CAPTURE_TIMER, 0, playerid );
   else if ( ageid == cAge3 )
      trModifyProtounit( "Ahuizotl", playerid, cModifyMovementType, cMovementTypeAmphibious );
}

//Concept: Since there are likely to be many more script-required techs for Maya/Inca, streamline the handling of them in
//one place, while keeping the added burden on RuleFunc()s negligable. (Otherwise *each* requires 12 rules + a func to handle.)
//1. Each script-required tech is assigned a bitmask that it adds to the script_techs value, or add 1 & a goofy indicator effect.
//2. RuleFunc then only needs to check script_techs > 0 and then call this func.
//3. This func will do all the necessary effects, set script_techs back to 0 (negate ad hoc goofy effect if needed).
void HandleAllScriptTechs(int playerid=-1, int basicQid=-1)
{
   int ivalue = get_int( SCRIPT_TECHS, playerid );
   subtract_int( SCRIPT_TECHS, ivalue, playerid );
  //Classical Age.
   if ( BitAndPowOf2(ivalue, 2048) )
      _AgeAdvanceHandler( playerid, cAge2 );
  //Heroic Age.
   if ( BitAndPowOf2(ivalue, 32768) )
      _AgeAdvanceHandler( playerid, cAge3 );
  //Mythic Age.
   if ( BitAndPowOf2(ivalue, 65536) )
      _AgeAdvanceHandler( playerid, cAge4 );
  //Nahua Time-Based Villager-spawn.
   if ( BitAndPowOf2(ivalue, 16384) )
      _NativeUnitSpawn( playerid, basicQid, cUnitTypeEmbassyNahua, cUnitTypeVillagerAztec, xsRound(RewardPerMinute(0.4, 30.0)) + 2 );
  //Native Painters.
   if ( BitAndPowOf2(ivalue, 8192) )
      RevealLOS( playerid, RELATION_ANY, cUnitTypeRelic );
  //Chichimec Time-Based Archer-spawn.
   if ( BitAndPowOf2(ivalue, 4096) )
      _NativeUnitSpawn( playerid, basicQid, cUnitTypeEmbassyChichimecs, cUnitTypeChiArcher, xsRound(RewardPerMinute(0.4, 30.0)) );
  //Tlaxcaltec Wagon Spawn.
   if ( BitAndPowOf2(ivalue, 1024) )
      _NativeUnitSpawn( playerid, basicQid, cUnitTypeEmbassyTlaxcaltecs, cUnitTypePorter, kbGetAge()+1 );
  //Huastec techs.
   if ( BitAndPowOf2(ivalue, 512) )
      NatHuastecFestivalHandler( playerid );
   if ( BitAndPowOf2(ivalue, 64) ) {
      int protoid = choose( get_my_culture() == cTechAge1Maya, cUnitTypeCottonFarmBig, cUnitTypeCottonFarm );
      trUnforbidProtounit( playerid, kbGetProtoUnitName(protoid) );
   }
  //Zapotec techs.
   if ( BitAndPowOf2(ivalue, 256) )
      NatZapotecFestivalHandler( playerid );
   if ( BitAndPowOf2(ivalue, 128) )
      NatZapotecTombsOfDaniBaan( playerid, basicQid );

  //Huastec Resource Tree (reserves mask 1 and 2).
   int reward = get_int( HUASTEC_TREE_TYPE, playerid );
   if ( (reward >= 0) && (reward < 100) ) {  //Just got 1+ of the resource-tree choice techs.
      reward = xsClamp( cUnitTypeHuaResourceTree+(reward-1), cUnitTypeHuaTreeFood, cUnitTypeHuaTreeGold );
      transform_results( basicQid, cUnitTypeHuaResourceTree, reward );
      set_int( HUASTEC_TREE_TYPE, reward, playerid );
   }
  //Tarascans Bronze Working.
   if ( kbProtoUnitAvailable(cUnitTypeRockSnow) ) {
      trForbidProtounit( playerid, "Rock Snow" );
      trChatSend( 0, "Tarascan Bronze Working: Player="+playerid );
      //xsEnableRule( "TarFreeArmorTech"+playerid );
   }
}

//======================================================================================================================
void _DisableWrongFishBoats(int playerid=-1, int cultureid=-1) {
   if ( cultureid != cCultureGreek )
      trForbidProtounit( playerid, "Fishing Ship Greek" );
   if ( cultureid != cCultureNorse )
      trForbidProtounit( playerid, "Fishing Ship Norse" );
   if ( cultureid != cCultureEgyptian )
      trForbidProtounit( playerid, "Fishing Ship Egyptian" );
   if ( cultureid != cCultureAtlantean )
      trForbidProtounit( playerid, "Fishing Ship Atlantean" );
   if ( cultureid != cCultureChinese )
      trForbidProtounit( playerid, "Fishing Ship Chinese" );
   if ( cultureid != cTechAge1Aztec )
      trForbidProtounit( playerid, "FishingShipAztec" );
   if ( cultureid != cTechAge1Maya )
      trForbidProtounit( playerid, "FishingShipMaya" );
//   if ( cultureid != cTechAge1Inca )
//      trForbidProtounit( playerid, "FishingShipInca" );
}

void AllCivsNewMechanics(int playerid=-1, int cultureid=-1, int basicQid=-1, float fTimeFactor=1.0, int landTlalocans=-1)
{
  //Fix all fishboats becoming enabled when capturing a Water Tlalocan (weird AoM bug).
   int number0 = choose( cultureid != cCultureGreek, cUnitTypeFishingShipGreek, cUnitTypeFishingShipNorse );
   if ( kbProtoUnitAvailable(number0) )
      _DisableWrongFishBoats( playerid, cultureid );

  //Cheaper Tlaloque enabled by having a Land/Water Tlalocan.
   if ( landTlalocans < 0 )
      landTlalocans = kbUnitCount( playerid, cUnitTypeTlalocan, cUnitStateAlive );
   int number1 = landTlalocans;
   if ( landTlalocans == 0 )
      number1 = kbUnitCount( playerid, cUnitTypeTlalocanWater, cUnitStateAlive );
   if ( (kbGetTechStatus(cTechTlaloque2Effect) >= cTechStatusActive) != (number1 > 0) )
      trTechSetStatus( playerid, cTechTlaloque2Effect, choose(number1>0, cTechStatusActive, cTechStatusUnobtainable) );

  //Script Techs.
   if ( get_int(SCRIPT_TECHS, playerid) > 0 )
      HandleAllScriptTechs( playerid, basicQid );

  //Native-Related.
   if ( NativeCodeRequired() )
      PlayerNativeMechanics( playerid, basicQid, fTimeFactor );

  //The Feathered Serpent (moved out of Natives because of Quetz's new tech).
   kbUnitQuerySetUnitType( basicQid, cUnitTypeTolBigSerpent );
   kbUnitQuerySetActionType( basicQid, cActionRangedAttack );
   number0 = kbUnitQueryExecute( basicQid );
   kbUnitQuerySetActionType( basicQid, -1 );
   number1 = kbUnitQueryExecute( basicQid );
   if ( number1 > number0 )  //Anything besides combat.
      add_resource_grant( lowest_resource_type(), (fTimeFactor * (number1-number0)) );
   kbUnitQueryResetResults( basicQid );

  //Relic Rewards.
   float fRate = get_float( cUnitTypeRelic );
   float fFWRate = kbProtoUnitGetTrainPoints( cUnitTypeRelic );
   if ( (fFWRate + fRate) > 0.0 ) {
      number0 = get_int( NUM_RELICS, playerid );
      if ( number0 > 0 ) {
         float fAmount = fTimeFactor * xsMin(number0, 8);
         fFWRate = fFWRate * fAmount;
         fRate = fRate * fAmount;
         food_grant = food_grant + fFWRate;
         wood_grant = wood_grant + fFWRate;
         gold_grant = gold_grant + fRate;
      }
   }
}

//TODO:
//1. Resource Trickles (ie, techtree-based).
//2. Plenty Vault & KOTH version (KOTH version requires that tech).

include "rotg_aztecs.xs";
include "rotg_mayans.xs";
include "rotg_incas.xs";

//All players excluding Mother Nature will have, in order, as part of their civ-setup:
//A) PreLaunch (1+)
//B) Launch<NewCiv> (0 or 1)
//C) PostLaunch
void AllCivsPreLaunch(int playerid=-1)
{
   set_civ_and_culture( playerid, kbGetCiv(), kbGetCulture() );
   int qid = CreateQuery( playerid, -1, cUnitStateAlive, false );
   set_int( BASIC_QUERY, qid, playerid );

   if ( kbGetCiv() == cCivOdin ) {
      qid = CreateQuery( playerid, cUnitTypeLogicalTypeOdinRegenerates, cUnitStateAlive, false );
      kbUnitQuerySetActionType( qid, cActionIdle );
      set_int( ODIN_REGEN_QRY, qid, playerid );
   }
   trForbidProtounit( playerid, "CottonFarm" );  //Enabled only thru native tech.
   trForbidProtounit( playerid, "CottonFarmBig" );
   if ( NativeCodeRequired() )
      NativesPreLaunch( playerid );
}
void NewCivPreLaunch(int playerid=-1)
{
   xsDisableRule( "NewCivSelect"+playerid );
   int barters = _BartersUntilCooldown( cAge1 );
   set_int( cUnitTypeFoodToWood, barters, playerid );
   set_int( cUnitTypeWoodToFood, barters, playerid );
   set_int( BARTER_COOLDOWN, 0, playerid );
}
void AllCivsPostLaunch(int playerid=-1, int civid=-1)
{
   int techid = -1;
   for(techid=cTechAge1Tezca; <= cTechAge1Quetz)
      if ( techid != civid ) trTechSetStatus(playerid, techid, cTechStatusUnobtainable);
   for(techid=cTechAge1Itzam; <= cTechAge1AhPuch)
      if ( techid != civid ) trTechSetStatus(playerid, techid, cTechStatusUnobtainable);
   for(techid=cTechAge1Vira; <= cTechAge1Inti)
      if ( techid != civid ) trTechSetStatus(playerid, techid, cTechStatusUnobtainable);
   if ( kbGetTechStatus(cTechRandomAztec) != cTechStatusActive )
      trTechSetStatus( playerid, cTechRandomAztec, cTechStatusUnobtainable );
   if ( kbGetTechStatus(cTechRandomMaya) != cTechStatusActive )
      trTechSetStatus( playerid, cTechRandomMaya, cTechStatusUnobtainable );
   if ( kbGetTechStatus(cTechRandomInca) != cTechStatusActive )
      trTechSetStatus( playerid, cTechRandomInca, cTechStatusUnobtainable );
   if ( civid != cCivShennong )
      trTechSetStatus( playerid, cTechAge1Shennong, cTechStatusUnobtainable );
  //SotT Fix/Editor helper.
   techid = choose( (get_my_culture() != cTechAge1Maya) || (kbGetTechStatus(cTechSecretsoftheTitans) == cTechStatusUnobtainable),
                    cTechSecretsoftheTitansMaya, cTechSecretsoftheTitans );
   trTechSetStatus( playerid, techid, cTechStatusUnobtainable );

   string godName = get_civ_name( civid );
   trSetCivilizationNameOverride( playerid, godName );
   if ( civid < 200 ) {  //An old-civ cCivXXX constant, not a new civ cTechAge1XXX.
      xsDisableRule( "NewCivSelect"+playerid );
      xsEnableRule( "OldCiv"+playerid );
      return;
   }
  //We're playing of the new civs...
   trChatSend( 0, GetPlayerName(playerid)+" is playing as "+godName );
   if ( aiIsMultiplayer() || (playerid != 1) )  //Gadget changes apply to all players, so only do this in SP.
      return;
   if ( civid == cTechAge1Camaxtli )
      civid = cTechAge1Huitz;
   if ( ((civid >= cTechAge1Tezca) && (civid <= cTechAge1Quetz))  //Aztec UIs.
      ) {
      godName = xsSubString( kbGetTechName(civid), 100000, 4 );
      gadgetUnreal( "showerBarPanel" );
      gadgetUnreal( "titleBarPanel" );
      gadgetUnreal( "buttonBarPanel-menuButton" );
      gadgetUnreal( "buttonBarPanel" );
      gadgetUnreal( "mainBackgroundBar" );
      for(i=2; <= 4) {
         gadgetUnreal( "showerBarPane"+i );
         gadgetReal( "showerBarPane"+i+godName );
      }
      gadgetReal( "showerBarPanel"+godName );
      gadgetReal( "titleBarPanel"+godName );
      gadgetReal( "buttonBarPanel-menuButton"+godName );
      gadgetReal( "buttonBarPanel"+godName );
      gadgetReal( "mainBackgroundBar"+godName );
   }
}

//==========================================================================================================
void ExtMechHealingSpringHeal(int springid=-1, float fHealRate=-1.0)
{
   int playerid = kbUnitGetOwner( springid );
   if ( playerid <= 0 )
      return;
   xsSetContextPlayer( playerid );
   int healQid = get_int( HEALING_SPRING_QRY, playerid );
   if ( healQid < 0 ) {
      healQid = CreateQuery( -1, cUnitTypeLogicalTypeAffectedByHealingSpring, cUnitStateAlive, false );
      kbUnitQuerySetActionType( healQid, cActionIdle );
      kbUnitQuerySetMaximumDistance( healQid, 20.5 );
      set_int( HEALING_SPRING_QRY, healQid, playerid );
   }
   kbUnitQuerySetPosition( healQid, kbUnitGetPosition(springid) );

  //Self-owned units.
   kbUnitQuerySetPlayerRelation( healQid, cPlayerRelationSelf );
   damage_results( healQid, -1, fHealRate );
  //Allies (must check action here).
   kbUnitQuerySetPlayerRelation( healQid, cPlayerRelationAlly );
   int results = kbUnitQueryExecute( healQid );
   for(i=0; < results) {
      int unitid = kbUnitQueryGetResult( healQid, i );
      if ( kbUnitGetAnimationActionType(unitid) == cActionIdle )
         damage_unit( unitid, fHealRate );
   }
   kbUnitQueryResetResults( healQid );
   xsSetContextPlayer( 0 );
}

void ExtMechBrokenSettlementFix(int settlementQid=-1)
{
  //This before searching for brokens; gives detected broken ones 1 sec to correct.
   static int tcid1 = -1;
   static int tcid2 = -1;
   if ( tcid1 != tcid2 ) {
      for(i=0; < 2) {
         int unitid = choose( i==0, tcid1, tcid2 );
         if ( unitid < 0 )
            continue;
         if ( kbUnitIsType(unitid, cUnitTypeSettlementsThatTrainVillagers) )  //It built correctly.
            continue;
         change_proto( unitid, cUnitTypeSettlementLevel1 );
         trMutateSelected( cUnitTypeSettlementLevel1 );
      }
      tcid1 = -1;
      tcid2 = -1;
   }

  //Search for broken ones. Don't use percent_damaged here! Some players are still claiming to get the "offset square"
  //bug (happens when the "fix" is applied to a non-broken one). Current >= Max seems not to produce false-positives.
   int brokens = 0;
   int results = kbUnitQueryExecute( settlementQid );
   for(i=0; < results) {
      unitid = kbUnitQueryGetResult( settlementQid, i );
      xsSetContextPlayer( kbUnitGetOwner(unitid) );
      bool isBroken = (kbUnitGetCurrentHitpoints(unitid) >= kbUnitGetMaximumHitpoints(unitid));
      xsSetContextPlayer( 0 );
      if ( isBroken == false )
         continue;
      if ( tcid1 == -1 )      tcid1 = unitid;
      else if ( tcid2 == -1 ) tcid2 = unitid;
      brokens++;
      if ( brokens >= 2 )
         break;
   }
   kbUnitQueryResetResults( settlementQid );
}
void ExtMechDeadLlamaFix(int llamaQid=-1)
{
/*
   int results = kbUnitQueryExecute( llamaQid );
   for(i=0; < results) {
      int llamaid = kbUnitQueryGetResult( llamaQid, i );
      if ( percent_damaged(llamaid) < 100 )
         continue;
      trMutateSelected( cUnitTypeLlamaDead );
      trUnitConvert( 0 );
      trUnitDelete( false );
   }
   kbUnitQueryResetResults( llamaQid );
*/
}

//=====================================================================================================================
void reset_nearest_extmech_qid(int nearestQid=-1) {
   kbUnitQuerySetPlayerID( nearestQid, -1, false );
   kbUnitQuerySetPlayerRelation( nearestQid, cPlayerRelationEnemy );
   kbUnitQuerySetMaximumDistance( nearestQid, 8.0 );
}

//=====================================================================================================================
//Thanks to kbUnitGetOwner() (like I'd been pestering for!) these no longer need a player-based loop!
rule ExtendedMechanics highFrequency priority 100 active
{
   static int extMechQid = -1;
   static int nearestQid = -1;
   static int settlementQid = -1;
   const int llamaQid = -1;  //TODO: Change to static and setup query when ready!
   static int prevMS = 0;
   static int prevMSforMS = 0;

  //A bit arduous to keep track of 2 times here...
  //The Per-Execution timer.
   int timerMS = trTimeMS();
   int tracker = timerMS - prevMSforMS;
   if ( tracker < 125 )
      return;
   prevMSforMS = timerMS;
  //The Per-Second timer.
   tracker = timerMS - prevMS;
   bool perSecond = (tracker >= 1000);
   if ( perSecond ) {
      float fTimeFactor = (0.001 * tracker);
      prevMS = timerMS;
   }

   xsSetContextPlayer( 0 );
   kbLookAtAllUnitsOnMap();
   if ( extMechQid < 0 ) {
      extMechQid = CreateQuery( RELATION_ANY, cUnitTypeMythUnitGodPower, cUnitStateAlive, false );
      nearestQid = CreateQuery( RELATION_ENEMY, -1, cUnitStateAliveOrBuilding, false );
      kbUnitQuerySetMaximumDistance( nearestQid, 8.0 );
      kbUnitQuerySetAscendingSort( nearestQid, true );
      settlementQid = CreateQuery( RELATION_ENEMY, cUnitTypeSettlement, cUnitStateBuilding, false );
   }

  //Bit-masking would be so good here :(. Need a series of special-handler flags for things where each token can't
  //be handled individually. A handler func is called post-loop.
   bool doDeathsAwaken = false;
   bool jagNightFound = false;
   int number = kbUnitQueryExecute( extMechQid );
   for(i=0; < number) {
      int unitid = kbUnitQueryGetResult( extMechQid, i );
      int protounit = kbUnitGetProtoUnitID( unitid );
     //Healing Spring.
      if ( protounit == cUnitTypeHealingSpringObject ) {
         if ( perSecond )
            ExtMechHealingSpringHeal( unitid, (fTimeFactor * -3.6) );
      }
     //Tarascan Healing Relic.
      else if ( protounit == cUnitTypeTarHealingRelic ) {
         if ( perSecond )
            damage_units_in_area( unitid, kbUnitGetOwner(unitid), cUnitTypeLogicalTypeAffectedByRestoration, 10.8, (fTimeFactor * -1.5) );
      }
     //ENTER Capture Mode.
      else if ( protounit == cUnitTypeCaptureEnter ) {
         EnterCaptureMode( kbUnitGetOwner(unitid) );
         destroy( unitid );
      }
     //EXIT Capture Mode.
      else if ( protounit == cUnitTypeCaptureExit ) {
         ExitCaptureMode( kbUnitGetOwner(unitid) );
         destroy( unitid );
      }
     //Great Abundance.
      else if ( protounit == cUnitTypeAbundanceToken ) {
         if ( kbUnitGetAnimationActionType(unitid) == cActionEmpower )
            continue;
         trUnitSelectClear();
         trUnitSelectByID( unitid );
         kbUnitQuerySetUnitType( nearestQid, cUnitTypeDropsite );
         kbUnitQuerySetPosition( nearestQid, kbUnitGetPosition(unitid) );
         if ( kbUnitQueryExecute(nearestQid) > 0 )
            trUnitDoWorkOnUnit( trGetUnitScenarioName(kbUnitQueryGetResult(nearestQid, 0)), -1 );
         else
            trUnitDelete( false );
         kbUnitQueryResetResults( nearestQid );
      }
     //Delete Prisoners (how I wish Lifespan and ForceToGaia worked here...)
      else if ( protounit == cUnitTypeDeletePrisoner ) {
	   convert( unitid, 0 );
	   trUnitDelete( false );
      }
     //Hand of Fire.
      else if ( protounit == cUnitTypeHandOfFireToken ) {
         ExtMechHandOfFireVillagerTransport( unitid, nearestQid );
         if ( perSecond )
 	      damage_units_in_area( unitid, kbUnitGetOwner(unitid), cUnitTypeLogicalTypeCanBeHealed, 10.5, (fTimeFactor * -1.0) );
      }
     //Tsunami.
      else if ( protounit == cUnitTypeTsunami ) {
         if ( perSecond )
            UnitCreate( 0, cUnitTypeBigSplash, 1, kbUnitGetPosition(unitid), 90 );
      }
     //Jaguar Night.
      else if ( protounit == cUnitTypeJaguarGP ) {
         if ( jagNightFound == false )
            jagNightFound = ExtMechJaguarNightPreCheck( unitid );
         if ( perSecond )
            ExtMechJaguarNightPredator( unitid );
      }
     //Decoys.
      else if ( protounit == cUnitTypeDecoyToken )
         change_proto( unitid, choose(kbAreaGetType(kbAreaGetIDByPosition(kbUnitGetPosition(unitid))) == cAreaTypeWater,
            cUnitTypeDecoyShip, choose((i%2) == 0, cUnitTypeDecoyInfantry, cUnitTypeDecoyArcher)) );
     //Tlalocan building.
      else if ( protounit == cUnitTypeCaptureWait )
         change_proto( unitid, choose(kbAreaGetType(kbAreaGetIDByPosition(kbUnitGetPosition(unitid))) != cAreaTypeWater,
            cUnitTypeTlalocan, cUnitTypeTlalocanWater) );
     //Deaths Awakening.
      else if ( protounit == cUnitTypeDeathsAwakenToken )
         doDeathsAwaken = true;
     //Qilins heal on death.
      else if ( protounit == cUnitTypeQilinHeal ) {
	   damage_units_in_area( unitid, kbUnitGetOwner(unitid), cUnitTypeLogicalTypeCanBeHealed, 10.0, -50.0 );
	   trUnitDelete( false );
      }
     //Native Embassies.
      else if ( protounit == cUnitTypeEmbassyBuilderToken )
         ExtMechEmbassyBuilderHandler( unitid, nearestQid );
     //Volcano God Power.
      else if ( protounit == cUnitTypeVolcanoDeadToken ) { //5,6=Hades2. 5,9=Hades9.
         TerrainChange( kbUnitGetPosition(unitid), 5.2, 5,9, true, true );
         mutate( unitid, cUnitTypeTartarianRubble );
         trUnitConvert( 0 );
      }
     //Tarascan Natives.
      else if ( protounit == cUnitTypeTarHeroDeathToken )
         ExtMechTarascanHeroDeath( unitid );
      else if ( protounit == cUnitTypeTarGPToken )
         ExtMechTarascanHeartPower( unitid );
     //Chichimec Natives.
      else if ( protounit == cUnitTypeChichiGPToken )
         ExtMechChichimecVisionPower( unitid );
     //Nahua Natives.
      else if ( protounit == cUnitTypeNahuaGPToken )
         ExtMechNahuaCursePower( unitid, nearestQid );
     //Otomi Natives.
      else if ( protounit == cUnitTypeOtomiGPToken )
         ExtMechStoneAncestorsPower( unitid, nearestQid );
     //Tlaxcaltec Natives.
      else if ( protounit == cUnitTypeTlaxGPToken )
         ExtMechCamaxtliPowerCast( unitid, nearestQid );
     //Toltec Natives.
      else if ( protounit == cUnitTypeTolGPToken )
         ExtMechToltecSerpentPower( unitid );
     //Mixtec Natives.
      else if ( protounit == cUnitTypeMixGPToken )
         ExtMechDzahuisRagePower( unitid, nearestQid );
     //Huastec Natives.
      else if ( protounit == cUnitTypeHuastecGPToken )
         ExtMechHuastecPowerCast( unitid );
     //Error/Unhandled.
      else
         trChatSend( 0, "ERR: Unhandled MythUnitGodPower! ID="+unitid+", Proto="+kbGetProtoUnitName(protounit)+", AnimAct="+kbUnitGetAnimationActionType(unitid) );
   }  //End: for()
   kbUnitQueryResetResults( extMechQid );

   if ( doDeathsAwaken )
      ExtMechCastDeathsAwakening();
   if ( perSecond ) {
      ExtMechBrokenSettlementFix( settlementQid );
      ExtMechDeadLlamaFix( llamaQid );
      if ( NativeCodeRequired() )
         ExtMechGlobalNativeHandler();
   }
}

//=====================================================================================================================
//RETURNS: New MS timer to assign to (since normal civs don't use array/get_time_modifier).
int OldCivRuleFunc(int playerid=-1, int prevMS=0)  //find_this
{
   int qid = get_int( BASIC_QUERY, playerid );
   int currentMS = trTimeMS();
   float fTimeFactor = 0.001 * (currentMS - prevMS);
   int cultureid = kbGetCultureForPlayer( playerid );

  //Preliminary stuff.
   RuleFuncImmediateSetup( playerid, qid, cultureid );

  //Everybody stuff.
   AllCivsNewMechanics( playerid, cultureid, qid, fTimeFactor );

  //Temple of Healing.
   if ( kbGetTechStatus(cTechTempleofHealing) >= cTechStatusActive ) {
      float fHealRate = fTimeFactor * -18.0;
      kbUnitQuerySetUnitType( qid, cUnitTypeTemple );
      kbUnitQuerySetActionType( qid, cActionHeal );
      int number = kbUnitQueryExecute( qid );
      for(i=0; < number) {
         int targetid = kbUnitGetTargetUnitID( kbUnitQueryGetResult(qid, i) );
         if ( kbUnitGetAnimationActionType(targetid) == cActionIdle )
            damage_unit( targetid, fHealRate );
      }
      kbUnitQueryResetResults( qid );
      kbUnitQuerySetActionType( qid, -1 );
   }

  //Odin Regenerate fix.
   if ( kbGetCiv() == cCivOdin )
      damage_results( get_int(ODIN_REGEN_QRY, playerid), -1, (fTimeFactor * -1.8) );

  //Give resources at end.
   GrantResourcesOwed( playerid );
   return( currentMS );
}
rule OldCiv1  minInterval 1 maxInterval 1 priority 100 group _dod1 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(1, prevMS); }
rule OldCiv2  minInterval 1 maxInterval 1 priority 100 group _dod2 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(2, prevMS); }
rule OldCiv3  minInterval 1 maxInterval 1 priority 100 group _dod3 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(3, prevMS); }
rule OldCiv4  minInterval 1 maxInterval 1 priority 100 group _dod4 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(4, prevMS); }
rule OldCiv5  minInterval 1 maxInterval 1 priority 100 group _dod5 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(5, prevMS); }
rule OldCiv6  minInterval 1 maxInterval 1 priority 100 group _dod6 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(6, prevMS); }
rule OldCiv7  minInterval 1 maxInterval 1 priority 100 group _dod7 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(7, prevMS); }
rule OldCiv8  minInterval 1 maxInterval 1 priority 100 group _dod8 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(8, prevMS); }
rule OldCiv9  minInterval 1 maxInterval 1 priority 100 group _dod9 inactive  { static int prevMS=0; prevMS=OldCivRuleFunc(9, prevMS); }
rule OldCiv10 minInterval 1 maxInterval 1 priority 100 group _dod10 inactive { static int prevMS=0; prevMS=OldCivRuleFunc(10, prevMS); }
rule OldCiv11 minInterval 1 maxInterval 1 priority 100 group _dod11 inactive { static int prevMS=0; prevMS=OldCivRuleFunc(11, prevMS); }
rule OldCiv12 minInterval 1 maxInterval 1 priority 100 group _dod12 inactive { static int prevMS=0; prevMS=OldCivRuleFunc(12, prevMS); }

//============================================================================================================================
//Lol, I proposed this as a joke, and here we are!
rule RabbitBreed minInterval 180 runImmediately active
{
   static int maxRabbits = -1;
   if ( maxRabbits < 0 ) {
      int starting = trPlayerUnitCountSpecific( 0, "Rabbit" );
      if ( starting > 0 )
         maxRabbits = xsMax( (starting * 2), 25 );
      else
         xsDisableSelf();
      return;
   }
   xsSetContextPlayer( 0 );
   kbLookAtAllUnitsOnMap();
   int rabbitQid = kbUnitQueryCreate( "RabbitQry" );
   ConfigQuery( rabbitQid, RELATION_ANY, cUnitTypeRabbit, cUnitStateAlive, false );
   kbUnitQuerySetPosition( rabbitQid, kbGetMapCenter() );
   kbUnitQuerySetAscendingSort( rabbitQid, true );
   int results = kbUnitQueryExecute( rabbitQid );
   int allowed = xsMin( maxRabbits - results, results );
   if ( allowed > 0 ) {
      bool oneAlive = false;
      for(i=results-1; >= 0) {
         int rabbitid = kbUnitQueryGetResult( rabbitQid, i );
         if ( kbUnitGetCurrentHitpoints(rabbitid) <= 0.0 )
            continue;
         oneAlive = true;
         UnitCreate( kbUnitGetOwner(rabbitid), cUnitTypeRabbit, 1, kbUnitGetPosition(rabbitid), trTimeMS() );
         allowed--;
         if ( allowed <= 0 )
            break;
      }
      if ( oneAlive == false ) {
         maxRabbits = -1;
         kbUnitQueryDestroy( rabbitQid );
         xsDisableSelf();
         return;
      }
   }
   kbUnitQueryResetResults( rabbitQid );
}

//===================================================================================================================
int AIPlayerSetupHandler(int playerid=-1) {
   if ( kbGetCiv() != cCivShennong ) {
      int planid = GetAIDataPlan( playerid );
      if ( planid >= 0 )
         if ( xsSubString(aiPlanGetName(planid), 4) == "rotg" ) {  //Set to Aztec AI file.
            trPlayerKillAllGodPowers( playerid );
            trSetCivAndCulture( playerid, cCivShennong, cCultureChinese );
            if ( aiPlanGetMilitary(planid) == false )
               trTechSetStatus( playerid, cTechSecretsoftheTitans, cTechStatusUnobtainable );
         }
   }
   return( AI_NEWCIV_CHOOSE_TIME_MS );
}

void NewCivSelectFunc(int playerid=-1)
{
   xsSetContextPlayer( playerid );
   int mseconds = 10000;
   if ( kbIsPlayerHuman(playerid) == false )
      mseconds = AIPlayerSetupHandler( playerid );
   if ( kbGetTechStatus(cTechAge1Meso) >= cTechStatusActive ) {
      NewCivPreLaunch( playerid );
     //AZTEC SELECT.
      int selection = cTechAge1Tezca;
      if ( kbGetTechStatus(cTechAge1Huitz) >= cTechStatusActive )
         selection = cTechAge1Huitz;
      else if ( kbGetTechStatus(cTechAge1Quetz) >= cTechStatusActive )
         selection = cTechAge1Quetz;
      else if ( kbGetTechStatus(cTechAge1Camaxtli) >= cTechStatusActive )
         selection = cTechAge1Camaxtli;
      else if ( kbGetTechStatus(cTechRandomAztec) >= cTechStatusActive ) {
         selection = (cTechAge1Tezca + RNG_Slow(3));
         trTechSetStatus( playerid, selection, cTechStatusActive );
      }
     //MAYAN SELECT.
      else if ( kbGetTechStatus(cTechAge1Itzam) >= cTechStatusActive )
         selection = cTechAge1Itzam;
      else if ( kbGetTechStatus(cTechAge1AhPuch) >= cTechStatusActive )
         selection = cTechAge1AhPuch;
      else if ( kbGetTechStatus(cTechAge1Chaac) >= cTechStatusActive )
         selection = cTechAge1Chaac;
      else if ( kbGetTechStatus(cTechRandomMaya) >= cTechStatusActive ) {
         selection = (cTechAge1Itzam + RNG_Slow(3));
         trTechSetStatus( playerid, selection, cTechStatusActive );
      }
     //INCA SELECT.
      else if ( kbGetTechStatus(cTechAge1Vira) >= cTechStatusActive )
         selection = cTechAge1Vira;
      else if ( kbGetTechStatus(cTechAge1Inti) >= cTechStatusActive )
         selection = cTechAge1Inti;
      else if ( kbGetTechStatus(cTechAge1Pacha) >= cTechStatusActive )
         selection = cTechAge1Pacha;
      else if ( kbGetTechStatus(cTechRandomInca) >= cTechStatusActive ) {
         selection = (cTechAge1Vira + RNG_Slow(3));
         trTechSetStatus( playerid, selection, cTechStatusActive );
      }

      if ( (selection >= cTechAge1Tezca) && (selection <= cTechAge1Camaxtli) )
         LaunchAztec( playerid, selection );
      else if ( (selection >= cTechAge1Itzam) && (selection <= cTechAge1AhPuch) )
         LaunchMaya( playerid, selection );
      else //if ( (selection >= cTechAge1Vira) && (selection <= cTechAge1Inti) )
         LaunchInca( playerid, selection );
      AllCivsPostLaunch( playerid, selection );
   }
   else if ( kbGetTechStatus(cTechAge1Shennong) >= cTechStatusActive ) {
      AllCivsPostLaunch( playerid, cCivShennong );
      trSetCivilizationNameOverride( playerid, "Shennong" );
      trChatSendSpoofed( playerid, GetPlayerName(playerid)+" is playing as Shennong." );
      TRSelectUnimportantUnit();
      trUnitChangeInArea( playerid, playerid, "Hill Fort", "Castle", 9.9e12 );
      trUnitChangeInArea( playerid, playerid, "Counter Building", "Stable Chinese", 9.9e12 );
      trUnitChangeInArea( playerid, playerid, "Barracks", "Barracks Chinese", 9.9e12 );
      trUnitChangeInArea( playerid, playerid, "VillagerAztec", "Villager Chinese", 9.9e12 );
   }
   else if ( trTimeMS() >= mseconds ) {
      if ( kbGetCiv() == cCivShennong )
         trTechSetStatus( playerid, cTechAge1Shennong, cTechStatusActive );
      else
         AllCivsPostLaunch( playerid, kbGetCiv() );
   }
}
//I try to avoid highFrequency (every decisecond!) due to lag concerns (even tho trigs do seem well-optimized).
//But when launching your civ, it needs to respond right-away!
rule NewCivSelect1  highFrequency inactive { NewCivSelectFunc(1); }
rule NewCivSelect2  highFrequency inactive { NewCivSelectFunc(2); }
rule NewCivSelect3  highFrequency inactive { NewCivSelectFunc(3); }
rule NewCivSelect4  highFrequency inactive { NewCivSelectFunc(4); }
rule NewCivSelect5  highFrequency inactive { NewCivSelectFunc(5); }
rule NewCivSelect6  highFrequency inactive { NewCivSelectFunc(6); }
rule NewCivSelect7  highFrequency inactive { NewCivSelectFunc(7); }
rule NewCivSelect8  highFrequency inactive { NewCivSelectFunc(8); }
rule NewCivSelect9  highFrequency inactive { NewCivSelectFunc(9); }
rule NewCivSelect10 highFrequency inactive { NewCivSelectFunc(10); }
rule NewCivSelect11 highFrequency inactive { NewCivSelectFunc(11); }
rule NewCivSelect12 highFrequency inactive { NewCivSelectFunc(12); }

//================================================================================================================
rule FakeCheats highFrequency active {
   int doPop = 0;
   for(playerid=1; < cNumberPlayers) {
      if ( kbIsPlayerHuman(playerid) == false )
         continue;
      if ( trChatHistoryContains("cc pop", playerid) )
         doPop++;
   }
   if ( doPop > 0 ) {
      trChatHistoryClear();
      for(playerid=1; < cNumberPlayers) {
         trModifyProtounit( "House", playerid, cModifyBuildLimit, doPop*2 );
         trModifyProtounit( "Manor", playerid, cModifyBuildLimit, doPop );
      }
   }
   if ( trTime() >= 15 )
      xsDisableSelf();
}

rule ROTGMain runImmediately active
{
   xsDisableSelf();
   if ( kbGetUnitTypeName(cUnitTypeUnit) != "Unit" )
      trChatSendSpoofed( 0, "ERR: Unit constants mismatch detected! Run MakeConstantsFile to correct." );
   else if ( (kbGetTechName(cTechFireStrike) != "FireStrike") || (kbGetTechName(cNumberTechs) != "Invalid") )
      trChatSendSpoofed( 0, "ERR: Tech constants mismatch detected! Run MakeConstantsFile to correct." );
   else
      trChatSendSpoofed( 0, "rotg_script loaded successfully." );

  //Nature Setup stuff.
   xsSetContextPlayer( 0 );
   if ( aiIsMultiplayer() )  //Allow nature units to be manually tasked in PvP games.
      aiSet( "", 0 );        //HUGE shouout to nottud for this solution!
   InitMath();
   kbLookAtAllUnitsOnMap();
   kbAreaCalculate();
   for(resourceid=0; <= cResourceFavor)
      trPlayerGrantResources( 0, ResourceName(resourceid), xsStringToInt("2147483647") );
   set_civ_and_culture( 0, kbGetCiv(), kbGetCulture() );
   ROTG_NativesInit();
   trModifyProtounit( "Tartarian Rubble", 0, cModifyLifespan, -31.0 );  //Crater of VolcanoGP stays there.

  //Player-Specific stuff and calcs.
   for(playerid=1; < cNumberPlayers) {
      xsSetContextPlayer( playerid );
      AllCivsPreLaunch( playerid );
      if ( (kbGetCiv() != cCivShennong) && kbIsPlayerHuman(playerid) )
         AllCivsPostLaunch( playerid, kbGetCiv() );
      else {
         trSetCivilizationNameOverride( playerid, " " );
         xsEnableRule( "NewCivSelect"+playerid );
      }
     //SNEAKY: Give devs a "secret code" to unlock content in our test scenarios we want hiden from the public.
      if ( kbUnitCount(playerid, cUnitTypeKastorAdult, cUnitStateAlive) == 2 )
         continue;
      for(i=0; < 3) {
         trTechSetStatus( playerid, cTechAge1Itzam+i, cTechStatusUnobtainable );
         trTechSetStatus( playerid, cTechAge1Vira+i, cTechStatusUnobtainable );
      }
      trTechSetStatus( playerid, cTechRandomMaya, cTechStatusUnobtainable );
      trTechSetStatus( playerid, cTechRandomInca, cTechStatusUnobtainable );
   }
}
